@@ -25,4 +25,4 @@
 
 #include "xap_Types.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Win32FrameImpl.h"
