@@ -87,7 +87,7 @@ filter_result MenuFilter::Filter(BMessage *message, BHandler **target) {
 #if 0
 	if(m_pBeOSFrame->getCurrentView() && pAction->hasGetStateFunction())
 	{
-		if(pAction->isCheckable())
+		if(pAction->isCheckable() || pAction->isRadio())
 		{
 		
 			EV_Menu_ItemState mis = pAction->getMenuItemState(m_pBeOSFrame->getCurrentView());
@@ -219,7 +219,10 @@ const char ** _ev_GetLabelName(XAP_BeOSApp * pBeOSApp,
 	const char * szLabelName;
 	
 	if (pAction->hasDynamicLabel())
-		szLabelName = pAction->getDynamicLabel(pBeOSFrame,pLabel);
+{
+		szLabelName = pAction->getDynamicLabel(pLabel);
+
+}
 	else
 		szLabelName = pLabel->getMenuLabel();
 
@@ -239,7 +242,7 @@ const char ** _ev_GetLabelName(XAP_BeOSApp * pBeOSApp,
 			EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 			UT_ASSERT(pEM);						// make sure it's bound to something
 
-			const EV_EditEventMapper * pEEM = pBeOSFrame->getEditEventMapper();
+			const EV_EditEventMapper * pEEM = pBeOSApp->getEditEventMapper();
 			UT_ASSERT(pEEM);
 
 			const char * string = pEEM->getShortcutFor(pEM);
@@ -350,7 +353,7 @@ bool EV_BeOSMenu::synthesize()
 		UT_ASSERT(pAction);
 		EV_Menu_Label * pLabel = m_pMenuLabelSet->getLabel(id);
 		UT_ASSERT(pLabel);
-
+//		printf("label: %s \n",pLabel);//jun
 		// get the name for the menu item
 		const char * szLabelName = NULL;
                 const char * szMnemonicName = NULL;
@@ -359,18 +362,20 @@ bool EV_BeOSMenu::synthesize()
 		{
 		case EV_MLF_Normal:	{
 
-						if(pView && pAction->hasGetStateFunction()) 
-						{
-						EV_Menu_ItemState mis = pAction->getMenuItemState(pView);
-//	                            if (mis & EV_MIS_Gray)
-//                                	bEnable = false;
-                                if (mis & EV_MIS_Toggled)
-                                	bCheck = true;
-                                else
-                                	bCheck = false;
-                        }
-#if 0
 			bool bEnable = true;
+
+			if(pView && pAction->hasGetStateFunction()) 
+			{
+				EV_Menu_ItemState mis = pAction->getMenuItemState(pView);
+				if (mis & EV_MIS_Gray)
+					bEnable = false;
+				if (mis & EV_MIS_Toggled)
+					bCheck = true;
+				else
+					bCheck = false;
+			}
+
+#if 0
                         bool bCheck = false;
 
                         if (pAction->hasGetStateFunction()) {
@@ -468,19 +473,9 @@ bool EV_BeOSMenu::synthesize()
 				
 				int32 iLength = strlen(szLabelName);
 				char* buffer = new char[2*(iLength+1)];
-			        
-        		memset(buffer, 0, 2*(iLength+1));
-
-				int32 destLength = 2*(iLength + 1);
-				int32 state =0;
-			
-				convert_to_utf8(B_ISO1_CONVERSION , szLabelName , &iLength ,  buffer , &destLength , &state);
-				buffer[destLength] = '\0';
-				
-				accel = _ev_convert(buf, buffer);
-				
-				
-				
+	        		memset(buffer, 0, 2*(iLength+1));
+				printf("label: %s \n",szLabelName);//jun
+				accel = _ev_convert(buf, szLabelName);
 				pMenu = top(stack);
 				if (!pMenu)			//Skip bogus first item
 				{
@@ -493,8 +488,11 @@ bool EV_BeOSMenu::synthesize()
 				newmesg->AddInt32(ABI_BEOS_MENU_EV_NAME, id);
                 
 				BMenuItem *pMenuItem = new BMenuItem(buf, newmesg, key,modifiers);
+				printf("menu: %s \n",buf);//jun
 				if( pAction->isCheckable() )
 					pMenuItem->SetMarked( (bCheck == true) );
+
+				pMenuItem->SetEnabled(bEnable);
 					
 				pMenu->AddItem(pMenuItem);
 				
@@ -523,13 +521,7 @@ bool EV_BeOSMenu::synthesize()
 				int32 iLength = strlen(szLabelName);
 				char* buffer = new char[2*(iLength+1)];
 
-				int32 destLength = 2*(iLength + 1);
-				int32 state =0;
-			
-				convert_to_utf8(B_ISO1_CONVERSION , szLabelName , &iLength ,  buffer , &destLength , &state);
-				buffer[destLength] = '\0';
-
-				accel = _ev_convert(buf, buffer);
+				accel = _ev_convert(buf, szLabelName);
 
 				pMenu = new BMenu(buf);		//Accellerator ignored
 				if (!pMenu) 
@@ -576,7 +568,7 @@ bool EV_BeOSMenu::synthesize()
 		case EV_MLF_BeginPopupMenu:
 			xxx_UT_DEBUGMSG(("MENU: Begin popup menu \n"));
                         break;
-                case EV_MLF_EndPopupMenu:
+		case EV_MLF_EndPopupMenu:
 			xxx_UT_DEBUGMSG(("MENU: End popup menu \n"));
                         break;
 
@@ -658,28 +650,28 @@ bool EV_BeOSMenuPopup::synthesizeMenuPopup(XAP_BeOSFrame * pFrame)
 
 		// get the name for the menu item
 		const char * szLabelName = NULL;
-                const char * szMnemonicName = NULL;
+		const char * szMnemonicName = NULL;
 
 		switch (pLayoutItem->getMenuLayoutFlags())
 		{
 		case EV_MLF_Normal:	{
 #if 0
 			bool bEnable = true;
-                        bool bCheck = false;
+			bool bCheck = false;
 
-                        if (pAction->hasGetStateFunction()) {
-                        	EV_Menu_ItemState mis = pAction->getMenuItemState(pView);
-                                if (mis & EV_MIS_Gray)
-                                	bEnable = false;
-                                if (mis & EV_MIS_Toggled)
-                                	bCheck = true;
-                        }                 
+			if (pAction->hasGetStateFunction()) {
+				EV_Menu_ItemState mis = pAction->getMenuItemState(pView);
+				if (mis & EV_MIS_Gray)
+					bEnable = false;
+				if (mis & EV_MIS_Toggled)
+					bCheck = true;
+			}
 #endif
 
 			const char **data = _ev_GetLabelName(m_pBeOSApp, m_pBeOSFrame, pAction, pLabel);
-                        //const char ** data = _ev_GetLabelName(m_pBeOSApp, m_pBeOSFrame, pAction, pLabel);
-                        szLabelName = data[0];
-                        szMnemonicName = data[1];
+			//const char ** data = _ev_GetLabelName(m_pBeOSApp, m_pBeOSFrame, pAction, pLabel);
+			szLabelName = data[0];
+			szMnemonicName = data[1];
 
 			BString betterString(szMnemonicName);
 			int index,modifiers;
@@ -754,30 +746,21 @@ bool EV_BeOSMenuPopup::synthesizeMenuPopup(XAP_BeOSFrame * pFrame)
 				(szLabelName) ? szLabelName : "NULL", 
 				(szMnemonicName) ? szMnemonicName : "NULL")); 
 			if (szLabelName && *szLabelName) {
-			
-
-        
 				char buf[1024];
 				// convert label into proper version and get accelerators
 				
 				int32 iLength = strlen(szLabelName);
 				char* buffer = new char[2*(iLength+1)];
 			        
-        		memset(buffer, 0, 2*(iLength+1));
+				memset(buffer, 0, 2*(iLength+1));
 
-				int32 destLength = 2*(iLength + 1);
-				int32 state =0;
-			
-				convert_to_utf8(B_ISO1_CONVERSION , szLabelName , &iLength ,  buffer , &destLength , &state);
-				buffer[destLength] = '\0';
-				
-				accel = _ev_convert(buf, buffer);
-				
+				accel = _ev_convert(buf, szLabelName);
 				//UT_ASSERT(pMenu);
 				BMessage *newmesg = new BMessage(ABI_BEOS_MENU_EV);
 				newmesg->AddInt32(ABI_BEOS_MENU_EV_NAME, id);
                 
 				BMenuItem *pMenuItem = new BMenuItem(buf, newmesg, key,modifiers);
+//				pMenuItem->SetEnabled(bEnable);
 				pMenuBar->AddItem(pMenuItem);
 				
 				delete [] buffer;
@@ -805,14 +788,7 @@ bool EV_BeOSMenuPopup::synthesizeMenuPopup(XAP_BeOSFrame * pFrame)
 				int32 iLength = strlen(szLabelName);
 				char* buffer = new char[2*(iLength+1)];
 
-				int32 destLength = 2*(iLength + 1);
-				int32 state =0;
-			
-				convert_to_utf8(B_ISO1_CONVERSION , szLabelName , &iLength ,  buffer , &destLength , &state);
-				buffer[destLength] = '\0';
-
-				accel = _ev_convert(buf, buffer);
-
+				accel = _ev_convert(buf, szLabelName);
 				pMenu = new BMenu(buf);		//Accellerator ignored
 				if (!pMenu) 
 				{
