@@ -1,22 +1,22 @@
 /* AbiSource Program Utilities
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
- 
+
 
 
 
@@ -38,7 +38,7 @@ typedef UT_uint32 EV_EditMouseContext;								/* may not be ORed */
 // defined at the bottom of this file -- if you dont you will spend
 // hours trying to work out why your functions do not get called, as
 // I did :) (Tomas)
-#define EV_EMC__MASK__			((EV_EditMouseContext) 0xf0000000)
+#define EV_EMC__MASK__			((EV_EditMouseContext) 0xf8000000)
 #define EV_EMC_UNKNOWN			((EV_EditMouseContext) 0x10000000)
 #define EV_EMC_TEXT				((EV_EditMouseContext) 0x20000000)
 #define EV_EMC_LEFTOFTEXT		((EV_EditMouseContext) 0x30000000)
@@ -47,10 +47,15 @@ typedef UT_uint32 EV_EditMouseContext;								/* may not be ORed */
 #define EV_EMC_IMAGESIZE		((EV_EditMouseContext) 0x60000000)
 #define EV_EMC_FIELD			((EV_EditMouseContext) 0x70000000)
 #define EV_EMC_HYPERLINK		((EV_EditMouseContext) 0x80000000)
-
-#ifdef BIDI_ENABLED
 #define EV_EMC_RIGHTOFTEXT		((EV_EditMouseContext) 0x90000000)
-#endif
+#define EV_EMC_REVISION		    ((EV_EditMouseContext) 0xa0000000)
+#define EV_EMC_VLINE            ((EV_EditMouseContext) 0xb0000000)
+#define EV_EMC_HLINE            ((EV_EditMouseContext) 0xc0000000)
+#define EV_EMC_FRAME            ((EV_EditMouseContext) 0xd0000000)
+#define EV_EMC_VISUALTEXTDRAG   ((EV_EditMouseContext) 0xe0000000)
+#define EV_EMC_TOPCELL          ((EV_EditMouseContext) 0x18000000)
+#define EV_EMC_TOC              ((EV_EditMouseContext) 0x28000000)
+#define EV_EMC_POSOBJECT        ((EV_EditMouseContext) 0x38000000)
 
 // NB: the following two values are not included in EV_COUNT_EMC
 // because they are not used in the bindings, and are, therefore,
@@ -64,11 +69,11 @@ typedef UT_uint32 EV_EditMouseContext;								/* may not be ORed */
 #define EV_EMC_HYPERLINKTEXT      ((EV_EditMouseContext) 0x000000002)
 #define EV_EMC_HYPERLINKMISSPELLED ((EV_EditMouseContext) 0x000000001)
 
-#define EV_EMC_ToNumber(emc)			(((emc)&EV_EMC__MASK__)>>28)
+#define EV_EMC_ToNumber(emc)			(((emc)&EV_EMC__MASK__)>>27)
 
 
 typedef UT_uint32 EV_EditModifierState;								/* may be ORed */
-#define EV_EMS__MASK__			((EV_EditModifierState) 0x0f000000)
+#define EV_EMS__MASK__			((EV_EditModifierState) 0x07000000)
 #define EV_EMS_SHIFT			((EV_EditModifierState) 0x01000000)
 #define EV_EMS_CONTROL			((EV_EditModifierState) 0x02000000)
 #define EV_EMS_ALT				((EV_EditModifierState) 0x04000000)
@@ -124,11 +129,7 @@ typedef UT_uint32 EV_EditBits;	/* union of all the above bits */
 
 #define EV_COUNT_EMB			6		// simple count (not 'OR')
 #define EV_COUNT_EMO			6		// simple count (not 'OR')
-#ifdef BIDI_ENABLED
-#define EV_COUNT_EMC			9		// simple count (not 'OR')
-#else
-#define EV_COUNT_EMC			8		// simple count (not 'OR')
-#endif
+#define EV_COUNT_EMC			17		// simple count (not 'OR')
 
 #define EV_IsMouse(eb)			(((eb) & EV_EMO__MASK__))
 #define EV_IsKeyboard(eb)		(((eb) & EV_EKP__MASK__))
