@@ -80,17 +80,17 @@ EV_Menu_LabelSet::EV_Menu_LabelSet(EV_Menu_LabelSet * pLabelSet)
 {
 	m_stLanguage = pLabelSet->getLanguage();
 	m_first = pLabelSet->getFirst();
-	const UT_Vector * vecLabels = pLabelSet->getAllLabels();
+	const UT_GenericVector<EV_Menu_Label *> * vecLabels = pLabelSet->getAllLabels();
 	UT_uint32 i = 0;
 	for(i=0; i< vecLabels->getItemCount(); i++)
 	{
-	    EV_Menu_Label * pEvl = (EV_Menu_Label *) vecLabels->getNthItem(i);
+	    const EV_Menu_Label * pEvl = vecLabels->getNthItem(i);
 		EV_Menu_Label * pNewLab = NULL;
 		if(pEvl != NULL)
 		{
 		    pNewLab = new EV_Menu_Label(pEvl->getMenuId(),pEvl->getMenuLabel(),pEvl->getMenuStatusMessage());
 		}
-		m_labelTable.addItem( (void *) pNewLab);
+		m_labelTable.addItem(pNewLab);
 	}
 }
 
@@ -103,25 +103,21 @@ bool EV_Menu_LabelSet::setLabel(XAP_Menu_Id id,
 								const char * szMenuLabel,
 								const char * szStatusMsg)
 {
-	void *tmp;
+	EV_Menu_Label * pTmpLbl;
 	XAP_Menu_Id last = m_first + m_labelTable.size();
 	
 	if (id < m_first || id >= last)
 		return false;
 
 	UT_sint32 index = id - m_first;
+
 	EV_Menu_Label *label = new EV_Menu_Label(id, szMenuLabel, szStatusMsg);
-	UT_sint32 error = m_labelTable.setNthItem(index, label, &tmp);
-	EV_Menu_Label * pTmpLbl = static_cast<EV_Menu_Label *> (tmp);
+	UT_sint32 error = m_labelTable.setNthItem(index, label, &pTmpLbl);
 	DELETEP(pTmpLbl);
 	return (error == 0);
 }
 
-#ifdef __MRC__
-EV_Menu_Label * EV_Menu_LabelSet::getLabel(XAP_Menu_Id id)
-#else
 EV_Menu_Label * EV_Menu_LabelSet::getLabel(XAP_Menu_Id id) const
-#endif
 {
 	XAP_Menu_Id last = m_first + m_labelTable.size();
 	if (id < m_first || id >= last)
@@ -129,18 +125,21 @@ EV_Menu_Label * EV_Menu_LabelSet::getLabel(XAP_Menu_Id id) const
 
 	UT_uint32 index = (id - m_first);
 	
-	EV_Menu_Label * pLabel = static_cast<EV_Menu_Label *> (m_labelTable.getNthItem(index));
+	EV_Menu_Label * pLabel = m_labelTable.getNthItem(index);
 
 	if (!pLabel)
 	{
-		UT_DEBUGMSG(("WARNING: %s translation for menu id [%d] not found.\n", m_stLanguage.c_str(), id));
+		if (id != 0)
+		{
+			UT_DEBUGMSG(("WARNING: %s translation for menu id [%d] not found.\n", m_stLanguage.c_str(), id));
+		}
 		// NOTE: only translators should see the following strings
 		// NOTE: do *not* translate them
 		pLabel = new EV_Menu_Label(id, "TODO", "untranslated menu item");
 		
 		// Add to label table so memory is freed.
 		// Note: Need to cast away constness so we can add the label.
-		((EV_Menu_LabelSet *)this)->addLabel(pLabel);
+		(static_cast<EV_Menu_LabelSet *>(const_cast<EV_Menu_LabelSet *>(this)))->addLabel(pLabel);
 	}
 
 	UT_ASSERT(pLabel && (pLabel->getMenuId() == id));
@@ -149,7 +148,6 @@ EV_Menu_Label * EV_Menu_LabelSet::getLabel(XAP_Menu_Id id) const
 
 bool EV_Menu_LabelSet::addLabel(EV_Menu_Label *pLabel)
 {
-	UT_DEBUGMSG(("JCA: EV_Menu_LabelSet::addLabel\n"));
 	UT_ASSERT(pLabel);
 	XAP_Menu_Id size_table = m_labelTable.size();
 
@@ -162,11 +160,12 @@ bool EV_Menu_LabelSet::addLabel(EV_Menu_Label *pLabel)
 		m_labelTable.pop_back();
 		size_table = m_labelTable.size();
 	}
-	
-	UT_ASSERT(pLabel->getMenuId() == size_table + m_first);
+//
+// This assert always fires for me 6/6/2202 so I'm commenting it out.	
+//	UT_ASSERT(pLabel->getMenuId() == size_table + m_first);
 	m_labelTable.push_back(pLabel);
 
-	return (size_table + 1 == (XAP_Menu_Id) m_labelTable.size());
+	return (size_table + 1 == static_cast<XAP_Menu_Id>(m_labelTable.size()));
 }
 
 const char * EV_Menu_LabelSet::getLanguage() const
