@@ -21,6 +21,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
+#include <errno.h>
 #include <photon/Pf.h>
 #include <photon/PhRender.h>
 
@@ -56,6 +57,27 @@ const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
 {
 	return pszFontFamily;
 }
+
+bool GR_QNXGraphics::OSCIsValid() {
+#ifdef WITH_OSC
+if(m_pOSC == NULL) 
+	return false;
+
+if(m_pDraw->area.size.w != m_pOSC->dim.w || m_pDraw->area.size.h != m_pOSC->dim.h)
+{
+PhDCSetCurrent(0);
+PhDCRelease(m_pOSC);
+m_pOSC = PdCreateOffscreenContext(0,m_pDraw->area.size.w,m_pDraw->area.size.h,0);
+PtDamageWidget(m_pDraw);
+}
+/*if(PdIsOffscreenValid(m_pOSC) == -1) {
+	PhDCRelease((PhDrawContext_t *)m_pOSC);
+	m_pOSC = NULL;
+	return false;
+}*/
+#endif
+return true;
+}
 /***
  Initialization/Teardown for drawing on a widget outside of the normal
  stream of "damage" events
@@ -66,24 +88,30 @@ int GR_QNXGraphics::DrawSetup() {
 	if (m_pPrintContext) {
 		return 0;
 	}
-	//sometimes abi is stupid and calls a raw draw function without making sure we want to draw it (ie, Left ruler in normal view mode), therefor we do a simple check if our widget is realized or not...
 	if(PtWidgetIsRealized(m_pDraw) == 0) return -1;
 
-	m_pGC_old=PgSetGC(m_pGC);
+#ifdef WITH_OSC
+	if(OSCIsValid()) {
+		PtFlush();
+		if(PhDCSetCurrent(m_pOSC) == NULL)
+			UT_ASSERT(0); //we don't need a seperate GC as a DC already have one. 
+	} else {
+		UT_ASSERT(0);
+		//TODO: try to create  a new one!
+	}
+#else
+	m_pGC_old = PgSetGC(m_pGC);
+#endif
 
 	//Set the region and the draw offset
 	PgSetRegion(PtWidgetRid(PtFindDisjoint(m_pDraw)));
 
+#ifndef WITH_OSC 
 	PtWidgetOffset(m_pDraw, &m_OffsetPoint);
 	m_OffsetPoint.y+=m_pDraw->area.pos.y;
 	m_OffsetPoint.x+=m_pDraw->area.pos.x;
 	PgSetTranslation (&m_OffsetPoint,0); //replace translation with this one.	
-
-/*	PhPoint_t trans;
-	trans.x = -m_OffsetPoint.x;
-	trans.y = -m_OffsetPoint.y;
-	PdSetOffscreenTranslation(m_pOSC,&trans);*/
-
+#endif
 	//Always clip to the canvas
 	PhRect_t _rdraw;
 	PtCalcCanvas(m_pDraw, &_rdraw);
@@ -104,8 +132,7 @@ int GR_QNXGraphics::DrawSetup() {
 /*	PhPoint_t abs;
 	PtGetAbsPosition(m_pDraw,&abs.x,&abs.y);
 	PhDeTranslateRect(&_rdraw,&abs);*/
-	PgSetUserClip(&_rdraw);
-
+	PgSetUserClip(&_rdraw); //honors the translation.
 
 	return 0;
 
@@ -118,53 +145,48 @@ int GR_QNXGraphics::DrawTeardown() {
 		return 0;
 	}
 
-	//Remove the clipping (only one for now)
-//	PtClipRemove();
-	//Reset the translation
-/*	m_OffsetPoint.x *= -1;
-	m_OffsetPoint.y *= -1;
-	PgSetTranslation(&m_OffsetPoint, 0);*/
-
-
-/*	PhRect_t rect;
-	rect.ul.x = rect.ul.y = 0;
-	rect.lr.x = m_pDraw->area.size.w;
-	rect.lr.y = m_pDraw->area.size.h;
-
-	PhRect_t rect2;
-
-	rect2.lr.x = m_OffsetPoint.x + m_pDraw->area.size.w;
-	rect2.lr.y = m_OffsetPoint.y + m_pDraw->area.size.h;
-	rect2.ul.x = m_OffsetPoint.x;
-	rect2.ul.y = m_OffsetPoint.y;
-
-	PgContextBlit(m_pOSC,&rect,NULL,&rect2); */
-
+#ifdef WITH_OSC
+	PhDCSetCurrent(0);
+	PgFlush();
+#else
 	PgSetUserClip(NULL);
 	PgClearTranslation();
-
 	PgSetGC(m_pGC_old);
-//	PhDCSetCurrent(m_pOldDC);
+#endif
 
 	return 0;
 }
 
 
+GR_Graphics *GR_QNXGraphics::graphicsAllocator(GR_AllocInfo& allocInfo) 
+{
+	GR_QNXAllocInfo &allocator = (GR_QNXAllocInfo&)allocInfo;
+	return new GR_QNXGraphics(allocator.m_win,allocator.m_draw,XAP_App::getApp());
+}
 GR_QNXGraphics::GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App *app)
+:
+	m_pWin(win),
+	m_pDraw(draw),
+	m_pFont(NULL),
+	m_pFontGUI(NULL),
+	m_pClipList(NULL),
+	m_iLineWidth(tlu(1)),
+	m_currentColor(Pg_BLACK),
+	m_pPrintContext(NULL),
+	m_iAscentCache(-1),
+	m_iDescentCache(-1),
+	m_iHeightCache(-1)	
 {
 	m_pApp = app;
-	m_pWin = win;
-	m_pDraw = draw;
-	m_pGC = PgCreateGC(0);
-	m_pFont = NULL;
-	m_pFontGUI = NULL;
-	m_pClipList = NULL;
-	m_iLineWidth = 1;
-	m_currentColor = Pg_BLACK;
-	m_pPrintContext = NULL;
-	m_iAscentCache = m_iDescentCache = m_iHeightCache -1;
 
-//	m_pOSC = PdCreateOffscreenContext(0,draw->area.size.w,draw->area.size.h,NULL);
+#ifdef WITH_OSC
+	memset(&m_DamagedArea,0,sizeof(m_DamagedArea));
+
+	m_pOSC = PdCreateOffscreenContext(0,draw->area.size.w,draw->area.size.h,NULL);
+#else
+	m_pGC = PgCreateGC(0);
+#endif
+	
 	m_pFontCx = PfAttachCx("/dev/phfont",240000);
 	
 	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
@@ -175,15 +197,21 @@ GR_QNXGraphics::GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App *app
 
 GR_QNXGraphics::~GR_QNXGraphics()
 {
-	DELETEP(m_pFontGUI);
+	_destroyFonts ();
+
 	UT_VECTOR_PURGEALL(UT_Rect*, m_vSaveRect);
 
 	for (UT_uint32 i = 0; i < m_vSaveRectBuf.size (); i++) {
 	  PhImage_t * pImg = (PhImage_t	*)m_vSaveRectBuf.getNthItem (i);
 	  PgShmemDestroy(pImg);
 	}
+#ifdef WITH_OSC
+	PtFlush();
+	PhDCSetCurrent(0); //JIC
+	PhDCRelease(m_pOSC);
+#else
 	PgDestroyGC(m_pGC);
-//	PhDCRelease(m_pOSC);
+#endif
 	PfDetach(m_pFontCx);
 }
 
@@ -206,7 +234,7 @@ bool GR_QNXGraphics::queryProperties(GR_Graphics::Properties gp) const
 	}
 }
 
-void GR_QNXGraphics::setLineProperties ( double inWidthPixels, 
+void GR_QNXGraphics::setLineProperties ( double inWidth, 
 					  GR_Graphics::JoinStyle inJoinStyle,
 					  GR_Graphics::CapStyle inCapStyle,
 					  GR_Graphics::LineStyle inLineStyle )
@@ -245,7 +273,7 @@ if(inLineStyle == LINE_ON_OFF_DASH)
 if(inLineStyle == LINE_DOUBLE_DASH)
 	PgSetStrokeDash((const unsigned char *)"\10\4",2,0x10000);  //Same as gtk?
 
-PgSetStrokeWidth((int)inWidthPixels);
+PgSetStrokeWidth(static_cast<int>(tduD(inWidth)));
 PgSetStrokeCap(capstyle);
 PgSetStrokeJoin(joinstyle);
 DRAW_END
@@ -277,6 +305,13 @@ void GR_QNXGraphics::flush(void)
 /***
  Character Operations
 ***/
+/*
+ This function gets called a lot ... ideally it should be as fast as
+possible, but due to the fact that we call it outside of the normal
+draw stream, we can't always assume that other things are properly
+set (ie font, colour etc) so we end up doing it again ourselves. It
+also licks that we have to do the translation at this layer.
+*/
 void GR_QNXGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 							   int iLength, UT_sint32 xoff, UT_sint32 yoff,
 							   int * pCharWidths)
@@ -293,37 +328,37 @@ if(pCharWidths == NULL && iLength > 1) {
 	for(int i=0;i<iLength;i++)
 		ucs2buffer[i] = (UT_UCS2Char)pChars[iCharOffset+i];
 
-	PhPoint_t pos = {tdu(xoff),tdu(getFontAscent()) + tdu(yoff)};
+	PhPoint_t pos = {_tduX(xoff),tdu(getFontAscent()) + _tduY(yoff)};
 	DRAW_START
-	GR_CaretDisabler caretDisabler(getCaret());
-	PgSetTextColor(m_currentColor);	
-	PgSetFont(m_pFont->getDisplayFont());
+//	PgSetTextColor(m_currentColor);	
+//	PgSetFont(m_pFont->getDisplayFont());
 	PgDrawText((const char *)ucs2buffer,iLength*2,&pos,Pg_TEXT_WIDECHAR|Pg_TEXT_LEFT);
 	DRAW_END
+	setDamage(pos.x,pos.y,tdu(getFontHeight()),m_pDraw->area.size.w);
 }
 else
 	for(int i=0;i<iLength;i++) {
 		drawChar(pChars[iCharOffset + i],xoff,yoff);
 		if(pCharWidths && i < iLength - 1)
-			xoff += pCharWidths[i];
+			xoff += pCharWidths[iCharOffset + i];
 }
 }
 
 void GR_QNXGraphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 {
-PhPoint_t pos = {tdu(xoff),tdu(yoff)+ tdu(getFontAscent())};
+PhPoint_t pos = {_tduX(xoff),_tduY(yoff)+ tdu(getFontAscent())};
 UT_UCS2Char ucs2buffer[2] = {(UT_UCS2Char)Char,0};
 
-GR_CaretDisabler caretDisabler(getCaret());
 DRAW_START
 
-PgSetTextColor(m_currentColor);	
-PgSetFont(m_pFont->getDisplayFont());
+//PgSetTextColor(m_currentColor);	
+//PgSetFont(m_pFont->getDisplayFont());
 PgDrawText((const char *)&ucs2buffer,2,&pos,Pg_TEXT_WIDECHAR|Pg_TEXT_LEFT);
 DRAW_END
+setDamage(pos.x,pos.y,tdu(getFontHeight()),m_pDraw->area.size.w);
 }
 
-UT_uint32 GR_QNXGraphics::measureUnRemappedChar(const UT_UCSChar c)
+UT_sint32 GR_QNXGraphics::measureUnRemappedChar(const UT_UCSChar c)
 {
 float fWidth;
 //All other platforms do this, so why not..
@@ -354,7 +389,7 @@ GR_Font * GR_QNXGraphics::getGUIFont(void)
 }
 
 
-GR_Font * GR_QNXGraphics::findFont(const char* pszFontFamily,
+GR_Font * GR_QNXGraphics::_findFont(const char* pszFontFamily,
 									const char* pszFontStyle,
 									const char* /*pszFontVariant*/,
 									const char* pszFontWeight,
@@ -415,6 +450,9 @@ void GR_QNXGraphics::setFont(GR_Font * pFont)
 	{ 
 		m_pFont = qnxFont;
 	}
+	DRAW_START
+	PgSetFont(qnxFont->getDisplayFont());
+	DRAW_END
 }
 
 UT_uint32 GR_QNXGraphics::getFontAscent()
@@ -459,6 +497,7 @@ FontQueryInfo qinfo;
 	if(PfQueryFontInfoCx(m_pFontCx,font->getFont(),&qinfo) == -1)
 		return 0; 
 return (qinfo.descender * getResolution() /getDeviceResolution());
+
 }
 
 UT_uint32 GR_QNXGraphics::getFontHeight(GR_Font * fnt)
@@ -517,33 +556,34 @@ void GR_QNXGraphics::getColor(UT_RGBColor& clr)
 
 void GR_QNXGraphics::setColor(const UT_RGBColor& clr)
 {
-	//printf("GR: setColor %d %d %d \n", clr.m_red, clr.m_blu, clr.m_grn);
 	m_currentColor = PgRGB(clr.m_red, clr.m_grn, clr.m_blu);
-	//Defer actually setting the color until we stroke something
+	DRAW_START
+	PgSetFillColor(m_currentColor);
+	PgSetTextColor(m_currentColor);
+	PgSetStrokeColor(m_currentColor);
+	DRAW_END
 }
 
 
 void GR_QNXGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
 			      UT_sint32 x2, UT_sint32 y2)
 {
-	if(getCaret() && getCaret()->isEnabled())
-		GR_CaretDisabler caretDisabler(getCaret());
-		DRAW_START
+	DRAW_START
 
 
-	x1 = tdu(x1);
-	x2 = tdu(x2);
-	y1 = tdu(y1);
-	y2 = tdu(y2);
+	x1 = _tduX(x1);
+	x2 = _tduX(x2);
+	y1 = _tduY(y1);
+	y2 = _tduY(y2);
 
-	PgSetFillColor(m_currentColor);
-	PgSetStrokeColor(m_currentColor);
-	PgSetStrokeWidth(m_iLineWidth);
 	PgDrawILine(x1, y1, x2, y2);
+
 	DRAW_END
+
+	setDamage(__min(x1,x2),__min(y1,y2),__max(y1,y2),__max(x1,x2)); 
 }
 
-void GR_QNXGraphics::getCoverage(UT_Vector &coverage)
+void GR_QNXGraphics::getCoverage(UT_NumberVector &coverage)
 {
 FontQueryInfo info;
 const char *font;
@@ -553,13 +593,16 @@ return;
 coverage.clear();
 memset(&info,0,sizeof(info));
 PfQueryFontInfoCx(m_pFontCx,font,&info);
-coverage.push_back((void*)info.lochar);
-coverage.push_back((void*)(info.hichar - info.lochar));
+coverage.push_back(info.lochar);
+coverage.push_back((info.hichar - info.lochar));
 }
 
 void GR_QNXGraphics::setLineWidth(UT_sint32 iLineWidth)
 {
 	m_iLineWidth = tdu(iLineWidth);
+	DRAW_START
+		PgSetStrokeWidth(m_iLineWidth);
+	DRAW_END
 }
 
 void GR_QNXGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
@@ -567,20 +610,19 @@ void GR_QNXGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
 {
 	int old;
 
-	GR_CaretDisabler caretDisabler(getCaret());
 	DRAW_START
 	old = PgSetDrawMode(Pg_DrawModeSRCXOR);
-
-	x1 = tdu(x1);
-	x2 = tdu(x2);
-	y1 = tdu(y1);
- 	y2 = tdu(y2);
+	
+	x1 = _tduX(x1);
+	x2 = _tduX(x2);
+	y1 = _tduY(y1);
+ 	y2 = _tduY(y2);
 	PgSetStrokeXORColor(Pg_WHITE,Pg_BLACK);
-	PgSetStrokeWidth(m_iLineWidth);
 	PgDrawILine(x1, y1, x2, y2);
 	PgFlush();
 	PgSetDrawMode(old);
 	DRAW_END
+	setDamage(__min(x1,x2),__min(y1,y2),__max(y1,y2),__max(x1,x2)); 
 }
 
 void GR_QNXGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
@@ -591,11 +633,10 @@ void GR_QNXGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
 	PhPoint_t* points = new PhPoint_t[nPoints];
 	UT_ASSERT(pts);
 	for (UT_uint32 i = 0;i < nPoints;i++){
-		points[i].x = tdu(pts[i].x);
-		points[i].y = tdu(pts[i].y);
+		points[i].x = _tduX(pts[i].x);
+		points[i].y = _tduY(pts[i].y);
 	}
 	newc = PgRGB(c.m_red, c.m_grn, c.m_blu);
-	GR_CaretDisabler caretDisabler(getCaret());
 	DRAW_START
 	oldc = PgSetFillColor(newc);
 	
@@ -604,11 +645,11 @@ void GR_QNXGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
 
 	PgSetFillColor(oldc);
 	DRAW_END
+	setDamage(0,0,-1,-1);
 }
 
 void GR_QNXGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	for (UT_uint32 k=1; k<nPoints; k++)
 	{
 		drawLine(pts[k-1].x,pts[k-1].y, pts[k].x,pts[k].y);
@@ -619,16 +660,16 @@ void GR_QNXGraphics::invertRect(const UT_Rect* pRect)
 {
 	int old;
 	UT_ASSERT(pRect);
-	GR_CaretDisabler caretDisabler(getCaret());
 
 	DRAW_START
 
 	old = PgSetDrawMode(Pg_DrawModeSRCXOR);
-	PgSetFillXORColor(Pg_WHITE,m_currentColor); //XXX: Should probably be Pg_BLACK ? 
-	PgDrawIRect(tdu(pRect->left), tdu(pRect->top), tdu(pRect->left)+tdu(pRect->width), tdu(pRect->top)+tdu(pRect->height), Pg_DRAW_FILL);
+	PgSetFillXORColor(Pg_WHITE,m_currentColor);
+	PgDrawIRect(_tduX(pRect->left), _tduY(pRect->top), _tduX(pRect->left)+_tduR(pRect->width), _tduY(pRect->top)+_tduR(pRect->height), Pg_DRAW_FILL);
 	PgSetDrawMode(old);
 
 	DRAW_END
+	setDamage(tdu(pRect->left),tdu(pRect->top),tdu(pRect->height),tdu(pRect->width));
 }
 
 void GR_QNXGraphics::setClipRect(const UT_Rect* pRect)
@@ -637,10 +678,10 @@ void GR_QNXGraphics::setClipRect(const UT_Rect* pRect)
 	if (pRect)
 	{
 		PhRect_t r;
-		r.ul.x = tdu(pRect->left);
-		r.ul.y = tdu(pRect->top);
-		r.lr.x = r.ul.x + tdu(pRect->width);
-		r.lr.y = r.ul.y + tdu(pRect->height);
+		r.ul.x = _tduX(pRect->left);
+		r.ul.y = _tduY(pRect->top);
+		r.lr.x = r.ul.x + _tduR(pRect->width);
+		r.lr.y = r.ul.y + _tduR(pRect->height);
 //		UT_ASSERT(!m_pClipList);		//Only one item for now
 	if (m_pClipList || (m_pClipList = PhGetTile())) {
 			m_pClipList->rect = r;
@@ -659,20 +700,21 @@ void GR_QNXGraphics::setClipRect(const UT_Rect* pRect)
 void GR_QNXGraphics::fillRect(const UT_RGBColor & c, UT_sint32 x, UT_sint32 y,
 			      UT_sint32 w, UT_sint32 h)
 {
-	PgColor_t newc,oldc;
-	x = tdu(x);
-	y = tdu(y);
-	w = tdu(w);
-	h = tdu(h);
+	PgColor_t newc;
+	x = _tduX(x);
+	y = _tduY(y);
+	w = _tduR(w);
+	h = _tduR(h);
 
 	newc = PgRGB(c.m_red, c.m_grn, c.m_blu);
-	GR_CaretDisabler caretDisabler(getCaret());
 	DRAW_START
 	
-	oldc = PgSetFillColor(newc);
+	newc = PgSetFillColor(newc);
+//	printf("fillRect RGB %d,%d %d/%d w/ %08x\n", x, y, w, h, newc);
 	PgDrawIRect(x, y, x+w, y+h, Pg_DRAW_FILL);
-	PgSetFillColor(oldc);
+	PgSetFillColor(newc); //restore color.
 	DRAW_END
+	setDamage(x,y,h,w);
 }
 
 inline void adjust_rect(PhRect_t *rect, PhPoint_t *offset) {
@@ -693,7 +735,6 @@ inline void adjust_rect(PhRect_t *rect, PhPoint_t *offset) {
 	}
 }
 
-//TODO: FIXup this code!
 void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 {
 	PhRect_t  rect;
@@ -709,6 +750,8 @@ void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	{
 		return;
 	}
+
+	//TODO: this + OSC?!??!
 	PtCalcCanvas(m_pDraw, &rect);
 	UT_sint32 iddy = labs(ddy);
 	offset.x = ddx;
@@ -716,6 +759,9 @@ void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	bool bEnableSmooth = m_pApp->isSmoothScrollingEnabled();
 	bEnableSmooth = bEnableSmooth && (iddy < 30) && (ddx == 0);
 
+	#ifdef WITH_OSC
+	bEnableSmooth=false; //XXX TEMPORARY!!
+	#endif
 	if(bEnableSmooth) 
 	{
 		if(ddy < 0)
@@ -740,11 +786,21 @@ void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 		}
 	} else 
 	{
+#ifndef WITH_OSC
 		PhPoint_t shift;
 		PtWidgetOffset(m_pDraw, &shift);
 		PhTranslateRect(&rect, &shift);	
 		adjust_rect(&rect, &offset);
 		PhBlit(PtWidgetRid(PtFindDisjoint(m_pDraw)), &rect, &offset);
+#else
+		PhRect_t dst_rect;
+		
+		dst_rect.ul.x = rect.ul.x + offset.x; //should be 0... 
+		dst_rect.lr.x = rect.lr.x - offset.x;
+		dst_rect.ul.y = rect.ul.y + offset.y;
+		dst_rect.lr.y = rect.lr.y - offset.y;
+		PgContextBlit(m_pOSC,&rect,m_pOSC,&dst_rect);
+#endif
 	}
 }
 
@@ -755,24 +811,35 @@ void GR_QNXGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 	GR_CaretDisabler caretDisabler(getCaret());
 	PhPoint_t shift,offset;
 	PhRect_t rect;
+	PhRect_t dst_rect;
 
 	rect.ul.x = tdu(x_src);
 	rect.ul.y = tdu(y_src);
 	rect.lr.x = tdu(width+x_src);
 	rect.lr.y = tdu(height+y_src);
 
+	dst_rect.ul.x = tdu(x_dest);
+	dst_rect.ul.y = tdu(y_dest);
+	dst_rect.lr.x = tdu(width+x_dest);
+	dst_rect.lr.y = tdu(height+y_dest);
+
+	#ifndef WITH_OSC
 	offset.x = tdu(x_dest - x_src);
 	offset.y = tdu(y_dest - y_src); 
 
 	PtWidgetOffset(m_pDraw, &shift);
 	PhTranslateRect(&rect, &shift);	
 	PhBlit(PtWidgetRid(PtFindDisjoint(m_pDraw)), &rect, &offset);
+	#else
+
+	PgContextBlit(m_pOSC,&rect,m_pOSC,&dst_rect);
+	#endif
 }
 
 void GR_QNXGraphics::clearArea(UT_sint32 x, UT_sint32 y,
 				 UT_sint32 width, UT_sint32 height)
 {
-	UT_RGBColor clrWhite(255,255,255);
+	static const UT_RGBColor clrWhite(255,255,255);
 	fillRect(clrWhite, x, y, width, height);
 }
 
@@ -781,8 +848,8 @@ void GR_QNXGraphics::clearArea(UT_sint32 x, UT_sint32 y,
 ***/
 GR_Image* GR_QNXGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
 {
-	iDisplayWidth = tdu(iDisplayWidth);
-	iDisplayHeight = tdu(iDisplayHeight);
+	iDisplayWidth = _tduR(iDisplayWidth);
+	iDisplayHeight = _tduR(iDisplayHeight);
 	/* GR_QNXImage* pImg = NULL; */
 	GR_Image* pImg = NULL;
    	if (iType == GR_Image::GRT_Raster)
@@ -798,8 +865,8 @@ void GR_QNXGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 {
 
 	UT_ASSERT(pImg);
-	xDest = tdu(xDest);
-	yDest = tdu(yDest);
+	xDest = _tduX(xDest);
+	yDest = _tduY(yDest);
 
 
    	if (pImg->getType() != GR_Image::GRT_Raster) {
@@ -811,12 +878,12 @@ void GR_QNXGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 	PhImage_t * image = pQNXImage->getData();
 	PhPoint_t pos;
 
-	GR_CaretDisabler caretDisabler(getCaret());
 	DRAW_START
 
 	pos.x = xDest; pos.y = yDest;
 	PgDrawPhImage(&pos,image,NULL);
 	DRAW_END
+	setDamage(pos.x,pos.y,image->size.h,image->size.w);
 }
 
 void GR_QNXGraphics::setColorSpace(GR_Graphics::ColorSpace /* c */)
@@ -922,9 +989,12 @@ unsigned short cursor_number=0;
 	case GR_CURSOR_WAIT:
 		cursor_number= Ph_CURSOR_WAIT;
 		break;
+	case GR_CURSOR_CROSSHAIR:
+		cursor_number = Ph_CURSOR_CROSSHAIR;
+		break;
 	}
 
-	PtSetResource(m_pWin,Pt_ARG_CURSOR_TYPE,cursor_number,0);
+	PtSetResource(PtGetParent(m_pWin,PtWindow),Pt_ARG_CURSOR_TYPE,cursor_number,0);
 }
 
 GR_Graphics::Cursor GR_QNXGraphics::getCursor(void) const
@@ -938,9 +1008,9 @@ void GR_QNXGraphics::setColor3D(GR_Color3D c)
 	m_currentColor = m_3dColors[c];
 	DRAW_START
 
-	GR_CaretDisabler caretDisabler(getCaret());
 	PgSetStrokeColor(m_currentColor);
 	PgSetFillColor(m_currentColor);
+	PgSetTextColor(m_currentColor); //do we need this?
 
 	DRAW_END
 }
@@ -957,18 +1027,20 @@ void GR_QNXGraphics::init3dColors()
 void GR_QNXGraphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h)
 {
 	UT_ASSERT(c < COUNT_3D_COLORS);
+	PgColor_t oldcolor;
 
-	x = tdu(x);
-	y = tdu(y);
-	w = tdu(w);
-	h = tdu(h);
+	x = _tduX(x);
+	y = _tduX(y);
+	w = _tduR(w);
+	h = _tduR(h);
 
-	GR_CaretDisabler caretDisabler(getCaret());
 	DRAW_START
-	PgSetFillColor(m_3dColors[c]);
+	oldcolor = PgSetFillColor(m_3dColors[c]);
 //	fprintf(stderr,"FillRect 3D %d,%d %d/%d w %08x\n", x, y, x+w, y+w, m_3dColors[c]);
 	PgDrawIRect(x, y, x+w, y+h, Pg_DRAW_FILL);
+	PgSetFillColor(oldcolor);
 	DRAW_END
+	setDamage(x,y,h,w);
 }
 
 void GR_QNXGraphics::fillRect(GR_Color3D c, UT_Rect &r)
@@ -1006,6 +1078,8 @@ bool GR_QNXGraphics::startPrint(void) {
 
 	if (m_pPrintContext) {
 		PpStartJob(m_pPrintContext);
+		PtFlush();
+		PtHold(); //increment "flux" count. no widgets will be redrawn until after print!
 		PpContinueJob(m_pPrintContext);
 	}
 
@@ -1020,32 +1094,20 @@ bool GR_QNXGraphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		pageNumber, iWidth, iHeight, bPortrait, (szPageLabel) ? szPageLabel : "", m_bPrintNextPage));
 	UT_ASSERT(m_pPrintContext);
 
-	iWidth = tdu(iWidth);
-	iHeight = tdu(iHeight);
+	iWidth = _tduR(iWidth);
+	iHeight = _tduR(iHeight);
 
 	if (!m_pPrintContext) {
 		return false;
 	}
 
-	if (!m_bPrintNextPage) {		/* First page do setup */
-#if 0
-		PhDim_t 	size;
-
-#define DPI_LEVEL 92
-		/* Adjust for non-printable margins? */
-		size.w = ((iWidth) * DPI_LEVEL) / 1000;
-		size.h = ((iHeight) * DPI_LEVEL) / 1000;
-		UT_DEBUGMSG(("Source size %d/%d \n", size.w, size.h));
-		PpPrintSetPC(m_pPrintContext, INITIAL_PC, 0, Pp_PC_SOURCE_SIZE, &size);
-
-		PpPrintOpen(m_pPrintContext);
-		PpPrintStart(m_pPrintContext);
-#endif
+	if(!m_bPrintNextPage) {
+		//do some setup things..
+		m_bPrintNextPage=true;
 	} else {
+		PgFlush();
 		PpPrintNewPage(m_pPrintContext);
 	}
-
-	m_bPrintNextPage = true;
 	return true;
 }
 
@@ -1054,24 +1116,50 @@ bool GR_QNXGraphics::endPrint(void) {
 	UT_ASSERT(m_pPrintContext);
 
 	if (m_pPrintContext) {
+		PgFlush();
 		PpSuspendJob(m_pPrintContext);
 		PpEndJob(m_pPrintContext);
+		PtRelease();
 		PtDamageWidget(m_pWin);
 		PgFlush();
 	}
 	return true;
 }
 
+/* capture the area and convert it to a image */
+GR_Image * GR_QNXGraphics::genImageFromRectangle(const UT_Rect & r) {
+	PhRect_t rect;
+	short int x,y;
+	PhImage_t *pImgshmem,*pImg;
+
+	//since this reads from the visual screen, make sure it's up to date!
+	//XXX:TODO:FIXME!!
+	PgFlush();
+	PgWaitDrawComplete();
+
+	PtGetAbsPosition(m_pDraw,&x,&y);
+	rect.ul.x=x + _tduX(r.left);
+	rect.ul.y=y + _tduY(r.top);
+	rect.lr.x= rect.ul.x + _tduR(r.width);
+	rect.lr.y= rect.ul.y + _tduR(r.height);
+	pImgshmem =PgReadScreen(&rect,NULL);
+
+	//PgReadScreen is kind of stupid and you need to free that image using PgShmemDestroy, therefor duplicate it so GR_QNXImage won't go nuts.
+
+	pImg = PiDuplicateImage(pImgshmem,NULL);
+	PgShmemDestroy(pImgshmem);
+	GR_QNXImage *img = new GR_QNXImage("ScreenShot");
+	img->setData(pImg);
+	return static_cast<GR_Image *>(img);
+}
+
 void GR_QNXGraphics::saveRectangle(UT_Rect &r, UT_uint32 iIndx)
 {
 	
   PhRect_t rect;
   short int x,y;
+	UT_Rect *oldR=NULL;
 
-	r.left =  tdu(r.left);
-	r.top =   tdu(r.top);
-	r.width = tdu(r.width);
-	r.height =tdu(r.height); 
 #if 0
 	// had to turn this off -- the fact the the rectangle coordinances
 	// are the same does not guarantee that the bitmap is (e.g., in a
@@ -1086,25 +1174,24 @@ void GR_QNXGraphics::saveRectangle(UT_Rect &r, UT_uint32 iIndx)
     return;
   }
 #endif
-		
-  void * oldR = NULL;
-  m_vSaveRect.setNthItem(iIndx, (void*)new UT_Rect(r),&oldR);
+
+  m_vSaveRect.setNthItem(iIndx, new UT_Rect(r),&oldR);
   if(oldR)
-    delete (UT_Rect*)oldR;
+    delete oldR;
   
   PhImage_t	*pImg;
 
   PtGetAbsPosition(m_pDraw,&x,&y);
-  rect.ul.x=x + r.left;
-  rect.ul.y=y + r.top;
-  rect.lr.x= rect.ul.x + r.width;
-  rect.lr.y= rect.ul.y + r.height;
+  rect.ul.x=x + _tduX(r.left);
+  rect.ul.y=y + _tduY(r.top);
+  rect.lr.x= rect.ul.x + _tduR(r.width);
+  rect.lr.y= rect.ul.y + _tduR(r.height);
   pImg =PgReadScreen(&rect,NULL);
 
-  void * oldC = NULL;
-  m_vSaveRectBuf.setNthItem(iIndx, (void*) pImg, &oldC);
+  PhImage_t * oldC = NULL;
+  m_vSaveRectBuf.setNthItem(iIndx,pImg, &oldC);
   if(oldC)
-    PgShmemDestroy((PhImage_t *)oldC);
+    PgShmemDestroy(oldC);
   
   return;
 }
@@ -1116,19 +1203,90 @@ void GR_QNXGraphics::restoreRectangle(UT_uint32 iIndx)
 
   if((r && pImg))
     {
-      PhPoint_t pos;
-      
-      DRAW_START
-	
-				pos.x=r->left;
-        pos.y=r->top;
+	PhPoint_t pos;
+
+	DRAW_START
+	pos.x=_tduX(r->left);
+	pos.y=_tduY(r->top);
       
 	PgDrawPhImage(&pos,pImg,0);
-      
-      DRAW_END
+	setDamage(pos.x,pos.y,pImg->size.h,pImg->size.w);
+	DRAW_END
     }
   return;
 }
+
+void GR_QNXGraphics::_beginPaint() {
+#ifdef WITH_OSC
+memset(&m_DamagedArea,0,sizeof(m_DamagedArea));
+#endif
+}
+
+//small helper function for endDraw that isn't encapsulated in DRAW_START/DRAW_END
+void GR_QNXGraphics::blitScreen() {
+
+#if 0 
+if(!m_DamagedArea.size.w || !m_DamagedArea.size.h)  //don't blit if nothing changed.
+	return;
+#endif
+#ifdef WITH_OSC
+if(OSCIsValid()) {
+	PhArea_t screenarea;
+	PhPoint_t pnt;
+	PtWidgetOffset(m_pDraw,&pnt);
+	screenarea.pos.x = pnt.x + m_DamagedArea.pos.x;
+	screenarea.pos.y = pnt.y + m_DamagedArea.pos.y;
+	screenarea.size.w = m_DamagedArea.size.w;
+	screenarea.size.h = m_DamagedArea.size.h;
+	PgFlush();
+	PgContextBlitArea(m_pOSC,NULL,NULL,NULL);//&m_DamagedArea,NULL,&screenarea);
+	PgFlush();
+	UT_DEBUGMSG(("Blitted m_pOSC(0x%x) (%d,%d,%d,%d) to (%d,%d,%d,%d)",m_pOSC,m_DamagedArea.pos.x,m_DamagedArea.pos.y,m_DamagedArea.size.w,m_DamagedArea.size.h,screenarea.pos.x,screenarea.pos.y,screenarea.size.w,screenarea.size.h));
+}
+#endif
+}
+
+void GR_QNXGraphics::_endPaint() {
+//blit to screen...
+//we might need to do some magic here to make sure we don't blit the whole area everytime.. might be slow.. :P
+//in every draw function, have a rect which you extend with the area which is being modified, then we'll reset it after drawing here...
+	DRAW_START
+	blitScreen();
+	DRAW_END
+}
+
+void GR_QNXGraphics::setDamage(int ulx,int uly,int h,int w)  {
+#if WITH_OSC
+
+	if(ulx < 0 || uly < 0 || h < 0 || w < 0) UT_ASSERT(0);
+
+	UT_DEBUGMSG(("SetDamage input= %d,%d,%d,%d",ulx,uly,h,w));
+	//check clipping.. ?
+	if(ulx >= m_pDraw->area.size.w || uly >= m_pDraw->area.size.h) return;
+
+	int lrx,lry;
+
+	lrx = ulx+w;
+	lry = uly+h;
+
+	if(lrx > m_pDraw->area.size.w) w=m_pDraw->area.size.w - ulx;
+	if(lry > m_pDraw->area.size.h) h=m_pDraw->area.size.h - uly;
+
+	if(w == -1) w = m_pDraw->area.size.w - ulx; 
+	if(h == -1) h = m_pDraw->area.size.h - uly;
+
+	if(m_DamagedArea.size.w < w)
+		m_DamagedArea.size.w = w + ((ulx>m_DamagedArea.pos.x) ? (ulx - m_DamagedArea.pos.x) : 0);
+	if(m_DamagedArea.size.h < h)
+		m_DamagedArea.size.h = h + ((uly>m_DamagedArea.pos.y) ? (uly - m_DamagedArea.pos.y) : 0); 
+	if(m_DamagedArea.pos.x > ulx)
+		m_DamagedArea.pos.x = ulx;
+	if(m_DamagedArea.pos.y > uly)
+		m_DamagedArea.pos.y = uly;
+	xxx_UT_DEBUGMSG(("SetDamage output=%d,%d,%d,%d",m_DamagedArea.pos.x,m_DamagedArea.pos.y,m_DamagedArea.size.h,m_DamagedArea.size.w));
+#endif
+}
+
 /* FONT ROUTINES */
 
 const char *QNXFont::getFont()
@@ -1202,4 +1360,11 @@ if(PfExtent(&rect,NULL,PfConvertFontID(m_120ptFontID),NULL,NULL,(char *)ucs4str,
 
 return (rect.lr.x - min(rect.ul.x,0));
 }
+void GR_QNXGraphics::setZoomPercentage(UT_uint32 iZoom)
+{
+	DELETEP (m_pFontGUI);
+	if(m_pFont)
+		m_pFont->deleteDisplayFont();
 
+	GR_Graphics::setZoomPercentage (iZoom); // chain up
+}
