@@ -23,6 +23,7 @@
 #include "xap_Prefs.h"
 #include "xap_Frame.h"
 #include "xav_View.h"
+#include "ut_Win32OS.h"
 
 extern "C"  UT_uint16    wvLangToLIDConverter(const char * lang);
 extern "C"  const char * wvLIDToLangConverter(UT_uint16);
@@ -169,6 +170,7 @@ class GR_Win32USPRenderInfo : public GR_RenderInfo
 				delete [] s_pJustify;  s_pJustify  = NULL;
 				delete [] s_pLogAttr;  s_pLogAttr  = NULL;
 				delete [] s_pChars;    s_pChars    = NULL;
+				delete [] s_pGoffsets; s_pGoffsets = NULL;
 				s_iAdvancesSize = 0;
 
 				s_pOwnerDraw = NULL;
@@ -201,7 +203,11 @@ class GR_Win32USPRenderInfo : public GR_RenderInfo
 			if(s_pChars) { delete [] s_pChars; s_pChars = NULL; }
 			s_pChars    = new WCHAR[iSize];
 
-			UT_return_val_if_fail(s_pAdvances && s_pJustifiedAdvances && s_pJustify && s_pLogAttr && s_pChars, false);
+			if(s_pGoffsets) { delete [] s_pGoffsets; s_pGoffsets = NULL; }
+			s_pGoffsets  = new GOFFSET[iSize];
+			
+			UT_return_val_if_fail(s_pAdvances && s_pJustifiedAdvances && s_pJustify &&
+								  s_pLogAttr && s_pChars && s_pGoffsets, false);
 
 			s_iAdvancesSize = iSize;
 			
@@ -245,6 +251,7 @@ class GR_Win32USPRenderInfo : public GR_RenderInfo
 	static int *     s_pJustify;
 	static SCRIPT_LOGATTR * s_pLogAttr;
 	static WCHAR *   s_pChars;
+	static GOFFSET * s_pGoffsets;
 	
 	// static buffer ownerhip: we use static buffers to store various positioning information; the
 	// following members identify the instance that last modified these buffers
@@ -265,6 +272,7 @@ WCHAR *         GR_Win32USPRenderInfo::s_pChars             = NULL;
 GR_RenderInfo * GR_Win32USPRenderInfo::s_pOwnerDraw         = NULL;
 GR_RenderInfo * GR_Win32USPRenderInfo::s_pOwnerCP           = NULL;
 GR_RenderInfo * GR_Win32USPRenderInfo::s_pOwnerChar         = NULL;
+GOFFSET *       GR_Win32USPRenderInfo::s_pGoffsets          = NULL;
 
 
 GR_Win32USPGraphics::GR_Win32USPGraphics(HDC hdc, HWND hwnd, XAP_App * pApp)
@@ -609,8 +617,26 @@ void GR_Win32USPGraphics::_setupFontOnDC(GR_Win32USPFont *pFont, bool bZoomMe)
 	}
 	else if (bZoomMe)
 	{
+		// this branch gets only executed for screen draw
+		// Win32 GDI uses a bigger font size for screen operations than the user request;
+		// this is a real pain as it makes doing WYSIWIG near impossible. One of the
+		// consequences of this is that if we do layout using printer metrics (in order to
+		// achieve true WYSIWIG), the letters, particularly, the descending parts, can be
+		// drawn outwith the line rectangle (and so leave pixel dirt, etc.)
+		// In order to avoid that, we actually request a slightly smaller font from the
+		// system if the physical pixel size drops below certain (heristically determined) value.
 		zoom = getZoomPercentage();
-		pixels = (UT_uint32)((double)pFont->getUnscaledHeight()* (double)zoom/100.0 + 0.5) ;
+		double dZoom = (double)zoom;
+		double dSizeFactor = dZoom * (double)pFont->getUnscaledHeight()/(double)getDeviceResolution();
+		// 10 pt font on 96dpi is 13px
+		if(dSizeFactor <= 14.0)
+		{
+			// The numerical constant is heuristic
+			// 2% decrease on a 96 dpi screen works out 1 pixel adjustment
+			dZoom /= 1.02;
+		}
+		
+		pixels = (UT_uint32)((double)pFont->getUnscaledHeight()* dZoom/100.0) ;
 	}
 	else if(getPrintDC())
 	{
@@ -902,9 +928,9 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 			{
 				delete [] pGlyphs;
 				delete [] pVa;
-				delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;
-				delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;
-				delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;
+				if(RI->m_pAdvances) {delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;}
+				if(RI->m_pGoffsets) {delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;}
+				if(RI->m_pJustify)  {delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;}
 			}
 
 			bCopyGlyphs = true; // glyphs not in RI
@@ -953,6 +979,10 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 		RI->m_iIndicesSize = iGlyphBuffSize;
 		RI->m_pIndices = pGlyphs;
 		RI->m_pVisAttr = pVa;
+
+		if(RI->m_pAdvances) {delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;}
+		if(RI->m_pGoffsets) {delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;}
+		if(RI->m_pJustify)  {delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;}
 	}
 	else if(!bDeleteGlyphs && bCopyGlyphs)
 	{
@@ -970,9 +1000,9 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 			// we also have to delete the other related arrays we do
 			// not need just yet to ensure that the size of all the
 			// arrays will remain in sync
-			delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;
-			delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;
-			delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;
+			if(RI->m_pAdvances) {delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;}
+			if(RI->m_pGoffsets) {delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;}
+			if(RI->m_pJustify)  {delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;}
 			
 			UT_return_val_if_fail(RI->m_pIndices && RI->m_pVisAttr, false);
 		
@@ -987,6 +1017,10 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 		// glyphs are already in the RI, just need to set the correct
 		// size for the buffers
 		RI->m_iIndicesSize = iGlyphBuffSize;
+
+		if(RI->m_pAdvances) {delete [] RI->m_pAdvances; RI->m_pAdvances = NULL;}
+		if(RI->m_pGoffsets) {delete [] RI->m_pGoffsets; RI->m_pGoffsets = NULL;}
+		if(RI->m_pJustify)  {delete [] RI->m_pJustify;  RI->m_pJustify  = NULL;}
 	}
 	else
 	{
@@ -1047,11 +1081,10 @@ UT_sint32 GR_Win32USPGraphics::getTextWidth(GR_RenderInfo & ri)
 {
 	UT_return_val_if_fail(ri.getType() == GRRI_WIN32_UNISCRIBE, 0);
 	GR_Win32USPRenderInfo & RI = (GR_Win32USPRenderInfo &)ri;
-
-	UT_uint32 iZoom = getZoomPercentage();
 	GR_Win32USPFont * pFont = (GR_Win32USPFont*)RI.m_pFont;
 
-	if(iZoom != RI.m_iZoom || pFont->getPrintDC() != getPrintDC() || RI.m_hdc != getPrintDC())
+	// NB -- do not have to check for zoom as the internal metrics is always for 100%
+	if(pFont->getPrintDC() != getPrintDC() || RI.m_hdc != getPrintDC())
 	{
 		measureRenderedCharWidths(ri);
 	}
@@ -1131,10 +1164,10 @@ void GR_Win32USPGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 		return;
 	}
 
-
-	if(iZoom != RI.m_iZoom || pFont->getPrintDC() != getPrintDC())
+	// NB -- do not have to check for zoom as the internal metrics is always for 100%
+	if(pFont->getPrintDC() != getPrintDC())
 	{
-		// this happens when we change zoom without any other changes
+		// this happens when we change printer
 		// we need to recalculate the widths ...
 		measureRenderedCharWidths(ri);
 	}
@@ -1158,6 +1191,12 @@ void GR_Win32USPGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 		iNextAdvance = (UT_sint32)((double)_tduX(iWidth) * m_fXYRatio);
 		RI.s_pAdvances[i] = iNextAdvance - iAdvance;
 		iAdvance = iNextAdvance;
+
+		// I use tdu here, not _tduX, _tduY, because the goffsets are adjustments relative
+		// to the x,y offsets of the character
+		RI.s_pGoffsets[i].du = (long)((double)tdu(RI.m_pGoffsets[i].du) * m_fXYRatio);
+		RI.s_pGoffsets[i].dv = tdu(RI.m_pGoffsets[i].dv);
+		
 		
 		if(RI.m_pJustify)
 		{
@@ -1168,7 +1207,14 @@ void GR_Win32USPGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 		}
 	}
 
- 	// RI.m_iZoom  = iZoom;
+	// there are two situations in which we need to recalculate the static positioning
+	// buffers, (a) the buffer is not ours, or, (b) we have last calculated its contents
+	// at different zomm level. Furthermore, if we detect a zoom change, we need to
+	// invalidate the screen ascent value for the current font
+	if(RI.m_iZoom != iZoom)
+		pFont->setScreenAscent(0);
+	
+	RI.m_iZoom = iZoom;
 	RI.s_pOwnerDraw = &ri;
 }
 
@@ -1184,8 +1230,8 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 	UT_return_if_fail(pItem && pFont);
 
 	UT_sint32 xoff = (UT_sint32)((double)_tduX(RI.m_xoff) * m_fXYRatio);
-	UT_sint32 yoff = _tduY(RI.m_yoff);
 
+	// we will deal with yoff later
 	if(RI.m_iLength == 0)
 		return;
 	
@@ -1229,6 +1275,34 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 	// need to make sure that the HDC has the correct font set
 	// we scale this font
 	_setupFontOnDC(pFont, true);
+
+	// Now deal with y offset: RI.m_yoff is the top of the run, based on the ascent of the
+	// font; if we are drawing on screen and using printer metrics to do the layout, the
+	// ascent of the screen font can be smaller/greater and so we need to adjust the yoff
+	// accordingly
+	UT_sint32 yoff;
+
+	if(m_bPrint)
+	{
+		yoff = _tduY(RI.m_yoff);
+	}
+	else
+	{
+		UT_sint32 iAscentScreen = pFont->getScreenAscent();
+
+		if(!iAscentScreen)
+		{
+			TEXTMETRIC tm = { 0 };
+			GetTextMetrics(m_hdc, &tm);
+			iAscentScreen = (UT_sint32)((double)tm.tmAscent*(double)getResolution()*100.0/
+										((double)getDeviceResolution()*(double)getZoomPercentage()));
+		
+			pFont->setScreenAscent(iAscentScreen);
+		}
+
+		UT_sint32 iAscentPrint = pFont->getTextMetric().tmAscent * getResolution() / getDeviceResolution();
+		yoff = _tduY(RI.m_yoff + iAscentPrint - iAscentScreen);
+	}
 	
 	int * pJustify = RI.m_pJustify && RI.m_bRejustify ? RI.s_pJustify + iGlyphOffset : NULL;
 	
@@ -1255,7 +1329,7 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 								  iGlyphCount,
 								  RI.s_pAdvances + iGlyphOffset,
 								  pJustify,
-								  RI.m_pGoffsets);
+								  RI.s_pGoffsets + iGlyphOffset);
 
 	pItem->m_si.a.eScript = eScript;
 	//RI.m_bRejustify = false; -- the docs are misleading; rejustification is always needed
@@ -1269,7 +1343,6 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 
 void GR_Win32USPGraphics::setPrintDC(HDC dc)
 {
-#if 1
 	// only do this for screen graphics
 	if(!m_bPrint && dc != m_printHDC)
 	{
@@ -1314,12 +1387,11 @@ void GR_Win32USPGraphics::setPrintDC(HDC dc)
 					GR_Graphics * pG = pView->getGraphics();
 
 					if(pG == this)
-						pView->remeasureChars();
+						pView->rebuildLayout();
 				}
 			}
 		}
 	}
-#endif
 }
 
 
@@ -1335,8 +1407,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	if(!RI.m_pAdvances)
 		RI.m_pAdvances = new int[RI.m_iIndicesSize];
 
-		if(!RI.m_pGoffsets)
-			RI.m_pGoffsets = new GOFFSET[RI.m_iIndicesSize];
+	if(!RI.m_pGoffsets)
+		RI.m_pGoffsets = new GOFFSET[RI.m_iIndicesSize];
 
 	UT_uint32 iZoom = getZoomPercentage();
 	
@@ -1344,6 +1416,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	UT_uint32 iPoints = (UT_uint32)pFont->getPointSize();
 #endif
 
+	bool bFontSetUpOnDC = false;
+	
 	// the script cache is always containing data for 100% zoom, we scale widths manually later
 	// but we need to refresh it if the printer changed
 	//if(iZoom != RI.m_iZoom)
@@ -1360,9 +1434,18 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		{
 			// we also need to remeasure the font metrics as well
 			// and scale it down for the screen
+			_setupFontOnDC(pFont, false);
+			bFontSetUpOnDC = true;
+			
 			TEXTMETRIC tm = { 0 };
 			GetTextMetrics(getPrintDC(), &tm);
 
+#if 0 //def DEBUG
+			HDC printHDC = UT_GetDefaultPrinterDC();
+			TEXTMETRIC tm2 = { 0 };
+			GetTextMetrics(printHDC, &tm2);
+			DeleteDC(printHDC);
+#endif
 			pFont->setHeight(MulDiv(tm.tmHeight, getDeviceResolution(), m_nPrintLogPixelsY));
 			pFont->setAscent(MulDiv(tm.tmAscent, getDeviceResolution(), m_nPrintLogPixelsY));
 			pFont->setDescent(MulDiv(tm.tmDescent, getDeviceResolution(), m_nPrintLogPixelsY));
@@ -1377,12 +1460,22 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		// need to make sure that the HDC has the correct font set
 		// we do not scale the font by zoom
-		_setupFontOnDC(pFont, false);
+		if(!bFontSetUpOnDC)
+		{
+			_setupFontOnDC(pFont, false);
+			bFontSetUpOnDC = true;
+		}
+		
 		hdc = m_printHDC ? m_printHDC : m_hdc;
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
 		RI.m_hdc = hdc;
 	}
+	else
+	{
+		RI.m_hdc = pFont->getPrintDC();
+	}
+	
 
 	
 	// need to disapble shaping if call to ScriptShape failed ...
@@ -1398,6 +1491,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		UT_ASSERT_HARMLESS( hdc == 0 );
 		_setupFontOnDC(pFont, false);
+		bFontSetUpOnDC = true;
+		
 		hdc = m_printHDC ? m_printHDC : m_hdc;
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
@@ -1412,7 +1507,7 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	pItem->m_si.a.eScript = eScript;
 
 	// remember the zoom at which we calculated this ...
-	RI.m_iZoom = iZoom;
+	// RI.m_iZoom = iZoom;
 
 	if(RI.s_pOwnerDraw == & ri)
 	{
@@ -1427,6 +1522,10 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	}
 
 	// now convert the whole lot to layout units
+	double dWidth = 0;
+	double dDeviceWidth = 0;
+	double dPrevDeviceWidth = 0;
+	
 	if(getPrintDC())
 	{
 		double fXYRatio = m_fXYRatio * m_fXYRatioPrint;
@@ -1434,8 +1533,16 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
 			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution()
+									   /((double)m_nPrintLogPixelsY*fXYRatio);
+			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
+			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()
 											/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
+			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()
+											/((double)m_nPrintLogPixelsY) + 0.5);
 		}
 
 		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
@@ -1453,7 +1560,16 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
 			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()/
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution()/
+									   ((double)getDeviceResolution()*m_fXYRatio);
+			
+			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
+			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()/
+											((double)getDeviceResolution()*m_fXYRatio) + 0.5);
+			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()/
 											((double)getDeviceResolution()) + 0.5);
 		}
 
@@ -1472,8 +1588,17 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
 			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()/
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution()/
+									   ((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio);
+			
+			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
+			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()/
 											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
+			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()/
+											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING) + 0.5);
 		}
 
 		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
@@ -2313,10 +2438,9 @@ GR_Win32USPFont *  GR_Win32USPFont::newFont(LOGFONT &lf, double fPoints, HDC hdc
 GR_Win32USPFont::GR_Win32USPFont(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 	: GR_Win32Font(lf, fPoints, hdc, printHDC),
 	  m_sc(NULL),
-	  m_printHDC(NULL)
+	  m_printHDC(NULL),
+	  m_iScreenAscent(0)
 {
-	// we need to get text metrix for the font afresh
-	// GetTextMetrics(hdc, m_tm);
 };
 
 void GR_Win32USPFont::_clearAnyCachedInfo()
@@ -2328,7 +2452,6 @@ void GR_Win32USPFont::_clearAnyCachedInfo()
 	}
 }
 
-
 GR_Win32USPFont::~GR_Win32USPFont()
 {
 	GR_Win32USPGraphics::fScriptFreeCache(&m_sc);
