@@ -21,12 +21,11 @@
 
 #include "gr_Image.h"
 #include "ut_bytebuf.h"
-
+#include "ut_svg.h"
 #include "ut_assert.h"
 
 GR_Image::GR_Image()
-  : m_szName(""), m_iLayoutWidth(0), m_iLayoutHeight(0),
-    m_iDisplayWidth(0), m_iDisplayHeight(0)
+  : m_szName(""), m_iDisplayWidth(0), m_iDisplayHeight(0)
 {
 }
 
@@ -47,6 +46,17 @@ void GR_Image::getName ( UT_String & copy ) const
   copy = m_szName;
 }
 
+/*!
+ * Scale our image to rectangle given by rec. The dimensions of rec
+ * are calculated in logical units.
+ * Overriden by platform implementation if needed. Default is to set 
+ * display size.
+ */
+void GR_Image::scaleImageTo(GR_Graphics * pG, const UT_Rect & rec)
+{
+	setDisplaySize(pG->tdu(rec.width), pG->tdu(rec.height));
+}
+
 void GR_Image::setName ( const char * name )
 {
   m_szName = name;
@@ -59,34 +69,19 @@ void GR_Image::setName ( const UT_String & name )
 
 GR_Image::GRType GR_Image::getBufferType(const UT_ByteBuf * pBB)
 {
-   const char * buf = (const char*)pBB->getPointer(0);
+   const char * buf = reinterpret_cast<const char*>(pBB->getPointer(0));
+   UT_uint32 len = pBB->getLength();
+
+   if (len < 6) return GR_Image::GRT_Unknown;
+
    char * comp1 = "\211PNG";
    char * comp2 = "<89>PNG";
-   if (!(strncmp((const char*)buf, comp1, 4)) || !(strncmp((const char*)buf, comp2, 6)))
+   if (!(strncmp(static_cast<const char*>(buf), comp1, 4)) || !(strncmp(static_cast<const char*>(buf), comp2, 6)))
      return GR_Image::GRT_Raster;
-   else {
-      UT_uint32 len = pBB->getLength();
-      if (len > 1000) len = 1000; // only scan first 1000 bytes
-      UT_uint32 off = 0;
-      for (;;) {
-	 while (off < len &&
-		(buf[off] == ' ' || buf[off] == '\t' || 
-		 buf[off] == '\n' || buf[off] == '\r')) off++;
-	 if (buf[off] == '<') {
-	    if ((buf[off+1] == 's' || buf[off+1] == 'S') &&
-		(buf[off+2] == 'v' || buf[off+2] == 'V') &&
-		(buf[off+3] == 'g' || buf[off+3] == 'G') &&
-		(buf[off] == ' ' || buf[off] == '\t' ||
-		 buf[off] == '\n' || buf[off] == '\r'))
-	      return GR_Image::GRT_Vector;
-	    else {
-	       off++;
-	       while (off < len && buf[off] != '>') off++;
-	    }
-	 }
-	 else return GR_Image::GRT_Unknown;
-      }
-   }
+
+   if (UT_SVG_recognizeContent (buf,len))
+     return GR_Image::GRT_Vector;
+
    return GR_Image::GRT_Unknown;
 }
 
@@ -112,28 +107,13 @@ bool GR_Image::convertToBuffer(UT_ByteBuf** ppBB) const
   return false; 
 }
 
-bool GR_Image::convertFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight) 
+bool GR_Image::convertFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iWidth, UT_sint32 iHeight) 
 { 
   // default no impl
+  UT_ASSERT_NOT_REACHED ();
   return false; 
 }
 
-void GR_Image::setLayoutSize(UT_sint32 iLayoutWidth, UT_sint32 iLayoutHeight) 
-{ 
-  m_iLayoutWidth = iLayoutWidth; 
-  m_iLayoutHeight = iLayoutHeight;
-}
-
-UT_sint32 GR_Image::getLayoutWidth(void) const 
-{ 
-  return m_iLayoutWidth;
-}
-
-UT_sint32 GR_Image::getLayoutHeight(void) const 
-{ 
-  return m_iLayoutHeight;
-}
-
 GR_Image::GRType GR_Image::getType() const
 { 
 //
@@ -145,7 +125,8 @@ GR_Image::GRType GR_Image::getType() const
 	return GRT_Raster;
 }
 
-bool GR_Image::render(GR_Graphics *pGR, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight) const 
+bool GR_Image::render(GR_Graphics *pGR, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight)
 { 
+  UT_ASSERT_NOT_REACHED ();
   return false; 
 }
