@@ -24,19 +24,35 @@
 
 #include "ut_timer.h"
 
+#ifdef XP_TARGET_COCOA
+# include "ut_map.h"
+typedef struct NSMutableDictionary;
+typedef struct NSLock;
+#endif
+
+
 class UT_UNIXTimer : public UT_Timer
 {
 public:
-	UT_UNIXTimer(UT_TimerCallback pCallback, void* pData);
+	UT_UNIXTimer(UT_WorkerCallback pCallback, void* pData);
 	virtual ~UT_UNIXTimer();
 
 	virtual UT_sint32 set(UT_uint32 iMilliseconds);
 	virtual void stop();
 	virtual void start();
-	
 private:
 	UT_sint32 m_iMilliseconds;
 	UT_uint32 m_iGtkTimerId;
+#ifdef XP_TARGET_COCOA
+	/* these are here for Cocoa timer */
+	static NSLock* s_timerMutex;
+	static NSMutableDictionary* s_timerIds;
+	static int s_lastTimerId;
+
+	friend void _checkLock(void);
+	friend void XAP_stopCocoaTimer (UT_uint32 timerId);
+	friend UT_uint32 XAP_newCocoaTimer (UT_uint32 time, int (*proc)(void *), void *p);
+#endif
 };
 
 #endif /* UT_UNIXTIMER_H */
