@@ -27,7 +27,11 @@
 OSVERSIONINFO& UT_GetWinVersion(void);
 bool UT_IsWinNT(void);
 bool UT_IsWin2K(void);
+bool UT_IsWin95(void);
 
-DLGTEMPLATE * WINAPI UT_LockDlgRes(HINSTANCE hinst, LPCSTR lpszResName);
+DLGTEMPLATE * WINAPI UT_LockDlgRes(HINSTANCE hinst, LPCTSTR lpszResName);
 
+char * UT_GetDefaultPrinterName();
+
+HDC  UT_GetDefaultPrinterDC();
 #endif /* UT_Win32OS_H */
