@@ -98,6 +98,7 @@ static UT_LangRecord s_Table[] =
 	{"ka-GE",		NULL, XAP_STRING_ID_LANG_KA_GE, UTLANG_LTR},
 	{"kn-IN",       NULL, XAP_STRING_ID_LANG_KN_IN, UTLANG_LTR},     // is Kannada LTR?
 	{"ko-KR",		NULL, XAP_STRING_ID_LANG_KO_KR, UTLANG_VERTICAL},	// TODO also UTLANG_LTR, Hipi: What about ko-KP?
+	{"ku",			NULL, XAP_STRING_ID_LANG_KU, UTLANG_LTR},	
 	{"kw-GB",		NULL, XAP_STRING_ID_LANG_KW_GB, UTLANG_LTR},	
 	{"la-IT",		NULL, XAP_STRING_ID_LANG_LA_IT, UTLANG_LTR},	// Hipi: Should be just "la"
 	{"lo-LA",       NULL, XAP_STRING_ID_LANG_LO_LA, UTLANG_LTR},
@@ -201,30 +202,38 @@ static int s_compareB(const void * l, const void *e)
 #endif
 }
 
+/*!
+ Set the language name attribut in the language table. It takes the localized
+ names from the available stringset. This function is supplied to set the names
+ after the stringset is known to the application.
+*/
+void UT_Language_updateLanguageNames() 
+{
+    const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+    UT_return_if_fail(pSS);
+    
+    for(UT_uint32 i = 0; i < NrElements(s_Table); i++)
+    {
+        s_Table[i].m_szLangName = const_cast<XML_Char *>(pSS->getValue(s_Table[i].m_nID));
+    }
+
+    qsort(&s_Table[0], NrElements(s_Table), sizeof(UT_LangRecord), s_compareQ);
+}
+
 
 bool UT_Language::s_Init = true;
 
-
 /*!
  The constructor looks up the translations for the language code and sorts the table
   alphabetically by the language name
  */
 UT_Language::UT_Language()
 {
-	if(s_Init) //only do this once
-	{
-		const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
-
-		UT_return_if_fail(pSS);
-		
-		for(UT_uint32 i = 0; i < NrElements(s_Table); i++)
-		{
-			s_Table[i].m_szLangName = const_cast<XML_Char *>(pSS->getValue(s_Table[i].m_nID));
-		}
-
-		qsort(&s_Table[0], NrElements(s_Table), sizeof(UT_LangRecord), s_compareQ);
-		s_Init = false;
-	}
+	if(s_Init) // do this once here
+    {
+        UT_Language_updateLanguageNames();
+        s_Init = false;
+    }
 }
 
 UT_uint32 UT_Language::getCount()
