@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Program Utilities
  * Copyright (C) 2001
  * 
@@ -25,16 +27,54 @@
  * 02111-1307, USA.
  */
 
-#include "ut_iconv.h"
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
 
+#include "ut_iconv.h"
+#include "ut_assert.h"
+#include "ut_debugmsg.h"
+
+#if 0 //defined(XP_UNIX_TARGET_GTK) || defined(XP_MAC_TARGET_MACOSX)
+#include <glib.h>
+#define UT_ICONV_USING_GLIB
+#else
+/* make freebsd happy */
+extern "C" {
+#include <iconv.h>
+}
+#endif
+
 #include "xap_EncodingManager.h"
 
 /************************************************************************/
 /************************************************************************/
 
+/* UGLY UGLY Iconv hack for operating systems with strange declartions
+   for iconv.  Why, oh why can't they all be the same? <sob> I will
+   suffer in the afterlife for this - sam - dec 2000 
+
+   Update - the folks repsonsible for the Single Unix Specification
+   are responsible for this.  They will suffer even more in the
+   afterlife than I will. - sam - mar 2001
+
+   Update - I ditched this, it was breaking builds.  Besides, the platforms
+   that would have been broken by using const have since changed to allow it.
+   AIX and hpux are my only concerns, and hpux is under the care of someone
+   else who will look into it.  Enjoy your afterlife, sam. - MG - jun 2002
+*/
+
+#if defined (WIN32) || defined(__QNXNTO__) || defined(__CYGWIN__) ||  \
+(defined (__MACH__) && defined (__APPLE__)) || \
+(defined(TARGET_OS_MAC) && TARGET_OS_MAC) || \
+defined(__BEOS__) || defined (__AIX__) || \
+(defined(__linux__) && defined(__powerpc__) && (__GLIBC__ <= 2) && (__GLIBC_MINOR__ <= 1))
+
+#define ICONV_CONST const
+#else
+#define ICONV_CONST
+#endif
+
 /*
  * This file represents my own personal assault on iconv, the most horrid
  * utility ever, which is yet somehow still essential.
@@ -53,8 +93,6 @@
  * 2) wrapper class around an iconv_t handle
  */
 
-#define INVALID_ICONV_HANDLE (iconv_t)-1
-
 /*!
  * This class is a nice wrapper around an iconv_t type
  */
@@ -69,12 +107,14 @@ auto_iconv::auto_iconv(UT_iconv_t iconv)
 auto_iconv::auto_iconv(const char * in_charset, const char *out_charset)
   UT_THROWS((UT_iconv_t))
 {
-  UT_iconv_t cd = UT_iconv_open (out_charset, in_charset);
-  
-  if (!UT_iconv_isValid(cd))
-    UT_THROW(cd);
+	m_h = UT_ICONV_INVALID;
 
-  m_h = cd;
+	UT_iconv_t cd = UT_iconv_open (out_charset, in_charset);
+	
+	if (!UT_iconv_isValid(cd))
+		UT_THROW(cd);
+	
+	m_h = cd;
 }
 
 /*!
@@ -108,6 +148,116 @@ UT_iconv_t auto_iconv::getHandle ()
 // everything below this line is extern "C"
 //
 
+static const char * s_ucs2_internal = 0;
+static const char * s_ucs4_internal = 0;
+
+static const char * s_ucs2_list[] = {
+	"UCS-2-INTERNAL",
+	"UCS-2-LE",
+	"UCS-2-BE",
+	"UCS-2LE",
+	"UCS-2BE",
+	"UCS2",
+	0
+};
+
+static const char * s_ucs4_list[] = {
+	"UCS-4-INTERNAL",
+	"UCS-4-LE",
+	"UCS-4-BE",
+	"UCS-4LE",
+	"UCS-4BE",
+	"UCS4",
+	0
+};
+
+static void s_internal_init ()
+{
+	static const char * latin = "ISO-8859-1";
+
+	UT_iconv_t handle = UT_ICONV_INVALID;
+
+	s_ucs2_internal = 0;
+	s_ucs4_internal = 0;
+
+	const char ** pszEnc = s_ucs2_list;
+	while (*pszEnc)
+		{
+			if (!UT_iconv_isValid(handle = UT_iconv_open (*pszEnc, latin)))
+				{
+					pszEnc++;
+					continue;
+				}
+			const char ibuf = 0x20;
+			const char * iptr = &ibuf;
+			size_t ilen = 1;
+			UT_UCS2Char obuf[2];
+			char * optr = reinterpret_cast<char *>(obuf);
+			size_t olen = 2;
+
+			bool success = ((size_t)(-1) != UT_iconv (handle, &iptr, &ilen, &optr, &olen));
+
+			UT_iconv_close (handle);
+			handle = UT_ICONV_INVALID;
+
+			if (success) success = (olen == 0);
+			if (success) success = (obuf[0] == 0x20);
+			if (success)
+				{
+					s_ucs2_internal = *pszEnc;
+					break;
+				}
+			pszEnc++;
+		}
+	UT_ASSERT(s_ucs2_internal);
+	if (s_ucs2_internal == 0)
+		{
+			s_ucs2_internal = s_ucs2_list[0];
+			UT_DEBUGMSG(("WARNING! this test failed to determine correct UCS-2 setting!\n"));
+		}
+	UT_DEBUGMSG(("using '%s' for UCS-2 internal\n", s_ucs2_internal));
+
+	pszEnc = s_ucs4_list;
+	while (*pszEnc)
+		{
+			if (!UT_iconv_isValid(handle = UT_iconv_open (*pszEnc, latin)))
+				{
+					pszEnc++;
+					continue;
+				}
+			const char ibuf = 0x20;
+			const char * iptr = &ibuf;
+			size_t ilen = 1;
+			UT_UCS4Char obuf[4];
+			char * optr = reinterpret_cast<char *>(&obuf);
+			size_t olen = 4;
+
+			bool success = ((size_t)(-1) != UT_iconv (handle, &iptr, &ilen, &optr, &olen));
+
+			UT_iconv_close (handle);
+			handle = UT_ICONV_INVALID;
+
+			if (success) success = (olen == 0);
+			if (success) success = (obuf[0] == 0x20);
+			if (success)
+				{
+					s_ucs4_internal = *pszEnc;
+					break;
+				}
+			pszEnc++;
+		}
+	UT_ASSERT(s_ucs4_internal);
+	if (s_ucs4_internal == 0)
+		{
+			s_ucs4_internal = s_ucs4_list[0];
+			UT_DEBUGMSG(("WARNING! this test failed to determine correct UCS-4 setting!\n"));
+		}
+	UT_DEBUGMSG(("using '%s' for UCS-4 internal\n", s_ucs4_internal));
+}
+
+/*!
+ * \return the internal iconv UCS-2 charset name
+ */
 const char * ucs2Internal ()
 {
 #if defined(WIN32)
@@ -122,10 +272,32 @@ const char * ucs2Internal ()
   return "UCS2";
 #else
   // general case, found by hub and dom
-  if (XAP_EncodingManager__swap_stou)
-    return "UCS-2BE";
-  else
-    return "UCS-2LE";
+	if (s_ucs2_internal == 0) 
+		s_internal_init ();
+	return s_ucs2_internal;
+#endif
+}
+
+/*!
+ * \return the internal iconv UCS-4 charset name
+ */
+const char * ucs4Internal ()
+{
+#if defined(WIN32)
+  // we special-case the win32 build, otherwise spelling and other stuff
+  // just doesn't work
+  return "UCS-4LE";
+#elif defined(_LIBICONV_H)
+  // libiconv seems to prefer UCS-4-INTERNAL to UCS-4BE and UCS-4LE
+  return "UCS-4-INTERNAL";
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+  // we special case the BSDs since spelling just doesn't work
+  return "UCS4";
+#else
+  // general case, found by hub and dom
+	if (s_ucs4_internal == 0) 
+		s_internal_init ();
+	return s_ucs4_internal;
 #endif
 }
 
@@ -133,22 +305,27 @@ const char * ucs2Internal ()
 /************************************************************************/
 
 /*!
- * Returns true is the internal handle is valid, false is not
+ * Returns true if the internal handle is valid, false if not
  */
 int UT_iconv_isValid ( UT_iconv_t cd )
 {
-  return (cd != INVALID_ICONV_HANDLE);
+  return (cd != UT_ICONV_INVALID);
 }
 
 UT_iconv_t  UT_iconv_open( const char* to, const char* from )
 {
   if ( to && from )
-    return iconv_open( to, from );
-  return INVALID_ICONV_HANDLE;
+#ifdef UT_ICONV_USING_GLIB
+	  return (UT_iconv_t)g_iconv_open(to, from);
+#else
+  return (UT_iconv_t)iconv_open( to, from );
+#endif
+
+  return UT_ICONV_INVALID;
 }
 
-size_t  UT_iconv( UT_iconv_t cd, const char **inbuf, 
-		  size_t *inbytesleft, char **outbuf, size_t *outbytesleft )
+size_t UT_iconv( UT_iconv_t cd, const char **inbuf, 
+				 size_t *inbytesleft, char **outbuf, size_t *outbytesleft )
 {
   // this should take care of iconv problems with different compilers
   // known issues:
@@ -159,14 +336,22 @@ size_t  UT_iconv( UT_iconv_t cd, const char **inbuf,
   if ( !UT_iconv_isValid ( cd ) )
     return (size_t)-1;
 
-  ICONV_CONST char ** buf = (ICONV_CONST char**)(inbuf);
-  return iconv( cd, buf, inbytesleft, outbuf, outbytesleft );
+#ifdef UT_ICONV_USING_GLIB
+  return g_iconv((GIConv)cd, (char **)inbuf, inbytesleft, outbuf, outbytesleft);
+#else
+  return iconv( (iconv_t)cd, (ICONV_CONST char **)inbuf, inbytesleft, outbuf, outbytesleft );
+#endif
 }
 
 int  UT_iconv_close( UT_iconv_t cd )
 {
   if ( UT_iconv_isValid ( cd ) )
-    return iconv_close( cd );
+#ifdef UT_ICONV_USING_GLIB
+    return g_iconv_close( (GIConv) cd );
+#else
+    return iconv_close( (iconv_t)cd );
+#endif
+
   return -1;
 }
 
@@ -176,26 +361,26 @@ void UT_iconv_reset(UT_iconv_t cd)
     // http://www.abisource.com/mailinglists/abiword-dev/01/April/0135.html
     if (XAP_EncodingManager::get_instance()->cjk_locale())
 		UT_iconv(cd, NULL, NULL, NULL, NULL);
-};
+}
 
 /*!
- * Borrowed from GLib 2.0 and modified
+ * Borrowed from GLib 2.0 and (heavily) modified
  *
- * str - Pointer to the input string.
- * len - Length of the input string to convert.
- * from_codeset - The "codeset" of the string pointed to by 'str'.
- * to_codeset - The "codeset" we want for the output.
- * bytes_read - optional, supply NULL if you don't want this.
- * bytes_written - optional, supply NULL if you don't want this.
+ * \param str Pointer to the input string.
+ * \param len Length of the input string to convert.
+ * \param from_codeset The "codeset" of the string pointed to by 'str'.
+ * \param to_codeset The "codeset" we want for the output.
+ * \param bytes_read optional, supply NULL if you don't want this.
+ * \param bytes_written optional, supply NULL if you don't want this.
  *
- * Returns a freshly allocated output string, which is terminated by
+ * \return Returns a freshly allocated output string, which is terminated by
  * a zero byte. Note that if the output codeset's terminator is not
  * a zero byte (e.g., UCS-2, where it is two zero bytes), you can 
  * get correct termination by including the input string's terminator 
  * in the length passed as 'len'. E.g., if 'str' is null-terminated 
  * US-ASCII "foo", given 'len' as 4.
  *
- * TODO: Check for out-of-memory allocations etc.
+ * \todo Check for out-of-memory allocations etc.
  */
 char * UT_convert(const char*	str,
 		  UT_sint32	len,
@@ -262,7 +447,7 @@ char * UT_convert_cd(const char *str,
 	size_t outbuf_size = ((len + 3) & ~3) + 15;
 	size_t outbytes_remaining = outbuf_size - 1; /* -1 for nul */
 
-	char* pDest = (char*)malloc(outbuf_size);
+	char* pDest = static_cast<char*>(malloc(outbuf_size));
 	char* outp = pDest;
 
 	bool have_error = false;
@@ -289,13 +474,13 @@ char * UT_convert_cd(const char *str,
 
 		      /* glibc's iconv can return E2BIG even if there is space
 		       * remaining if an internal buffer is exhausted. The
-		       * folllowing is a heuristic to catch this. The 16 is
+		       * following is a heuristic to catch this. The 16 is
 		       * pretty arbitrary.
 		       */
 		      if (used + 16 > outbuf_size)
 		        {
 		          outbuf_size = outbuf_size  + 15;
-		          pDest = (char*)realloc(pDest, outbuf_size);
+		          pDest = static_cast<char*>(realloc(pDest, outbuf_size));
 
 		          outp = pDest + used;
 		          outbytes_remaining = outbuf_size - used - 1; /* -1 for nul */
@@ -344,4 +529,3 @@ char * UT_convert_cd(const char *str,
 
 	return pDest;
 }
-
