@@ -352,6 +352,9 @@ UT_printf_string_upper_bound (const char* format,
 #  else /* va_list is a pointer */
 #  define VA_COPY(ap1, ap2)	  ((ap1) = (ap2))
 #  endif /* va_list is a pointer */
+#  if defined (__GNUC__)
+#  define VA_COPY(ap1,ap2)     __va_copy((ap1),(ap2))
+#  endif
 #endif /* !VA_COPY */
 
 UT_String& UT_String_vprintf (UT_String & inStr, const char *format,
