@@ -48,7 +48,8 @@ class XAP_CocoaFrameImpl;
 - (void)setTextView:(id <NSTextInput>)tv;
 - (id <NSTextInput>)textView;
 
-- (NSArray*)getToolbars;
+- (NSArray *)getToolbars;
+- (NSString *)getToolbarSummaryID;
 @end
 
 
@@ -78,6 +79,7 @@ public:
 										   const char * szToolbarLabelSetKey, const char * szToolbarLabelSetDefaultValue);
 */
 	virtual void _initialize();
+	virtual void                notifyViewChanged(AV_View * pView);
 //	virtual	XAP_Frame *			cloneFrame() = 0;
 //	virtual UT_Error   			loadDocument(const char * szFilename, int ieft) = 0;
 //	virtual UT_Error                        loadDocument(const char * szFilename, int ieft, bool createNew) = 0;
