@@ -19,7 +19,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <locale.h>
 #include <math.h>
 
 #include "ut_types.h"
@@ -44,6 +43,7 @@
 
 #include "xap_Prefs.h"
 #include "xap_App.h"
+#include "ut_locale.h"
 
 // the resolution that we report to the application (pixels per inch).
 #define PS_RESOLUTION		72
@@ -94,6 +94,8 @@ PS_Graphics::PS_Graphics(const char * szFilename,
 
 PS_Graphics::~PS_Graphics()
 {
+	_destroyFonts ();
+
 	FREEP(m_szPageSizeName);
 	FREEP(m_szFilename);
 }
@@ -113,7 +115,6 @@ bool PS_Graphics::queryProperties(GR_Graphics::Properties gp) const
 	}
 }
 
-#ifndef WITH_PANGO
 void PS_Graphics::setFont(GR_Font* pFont)
 {
 	UT_ASSERT(pFont);
@@ -159,7 +160,6 @@ UT_uint32 PS_Graphics::getFontDescent()
   return getFontDescent(m_pCurrentFont);
 }
 
-
 void PS_Graphics::getCoverage(UT_NumberVector& coverage)
 {
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -175,7 +175,7 @@ UT_uint32 PS_Graphics::getFontHeight()
 	return getFontAscent(m_pCurrentFont) + getFontDescent(m_pCurrentFont);
 }
 	
-UT_uint32 PS_Graphics::measureUnRemappedChar(const UT_UCSChar c)
+UT_sint32 PS_Graphics::measureUnRemappedChar(const UT_UCSChar c)
 {
 	// FIXME we should really be getting stuff from the font in layout units,
 	// FIXME but we're not smart enough to do that yet
@@ -183,7 +183,6 @@ UT_uint32 PS_Graphics::measureUnRemappedChar(const UT_UCSChar c)
 		* ((double)getResolution() / (double)getDeviceResolution());
 	return static_cast<UT_uint32>(rint(fWidth));
 }
-#endif //#ifndef WITH_PANGO
 
 UT_uint32 PS_Graphics::getDeviceResolution(void) const
 {
@@ -218,12 +217,12 @@ GR_Font* PS_Graphics::getGUIFont()
  * asked.  It will do a fuzzy match to find the font (using the aliases
  * found in fonts.conf
  */
-GR_Font * PS_Graphics::findFont(const char* pszFontFamily,
-								const char* pszFontStyle,
-								const char* pszFontVariant,
-								const char* pszFontWeight,
-								const char* pszFontStretch,
-								const char* pszFontSize)
+GR_Font * PS_Graphics::_findFont(const char* pszFontFamily,
+								 const char* pszFontStyle,
+								 const char* pszFontVariant,
+								 const char* pszFontWeight,
+								 const char* pszFontStretch,
+								 const char* pszFontSize)
 {
 	XAP_UnixFont* pUnixFont = XAP_UnixFontManager::pFontManager->findNearestFont(pszFontFamily, pszFontStyle, pszFontVariant, pszFontWeight,
 																				 pszFontStretch, pszFontSize);
@@ -243,7 +242,7 @@ GR_Font * PS_Graphics::findFont(const char* pszFontFamily,
 	UT_uint32 k, count;
 	for (k = 0, count = m_vecFontList.getItemCount(); k < count; k++)
 	{
-		PSFont * psf = static_cast<PSFont *>(m_vecFontList.getNthItem(k));
+		PSFont * psf = m_vecFontList.getNthItem(k);
 		UT_ASSERT(psf);
 
 		// is this good enough for a match?
@@ -259,7 +258,7 @@ GR_Font * PS_Graphics::findFont(const char* pszFontFamily,
 	}
 
 	// wasn't already there, add it
-	m_vecFontList.addItem(static_cast<void *>(pFont));
+	m_vecFontList.addItem(pFont);
 
 	// it's always the last in the list
 	UT_uint32 n = m_vecFontList.getItemCount() - 1;
@@ -268,7 +267,6 @@ GR_Font * PS_Graphics::findFont(const char* pszFontFamily,
 	return pFont;
 }
 
-#ifndef WITH_PANGO
 void PS_Graphics::drawGlyph(UT_uint32 Char, UT_sint32 xoff, UT_sint32 yoff)
 {
 	UT_ASSERT_NOT_REACHED ();
@@ -282,7 +280,7 @@ void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 		return;
 
 	if (!pCharWidths) {
-		_drawCharsUTF8(pChars, iCharOffset, iLength, tdu(xoff), tdu(yoff));
+		_drawCharsUTF8(pChars, iCharOffset, iLength, xoff, yoff);
 		return;
 	}
 
@@ -295,11 +293,11 @@ void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 
 	for (UT_sint32 i = 0; i < iLength; i++) {
 		UT_UCS4Char ch = pChars[iCharOffset + i];
-		advance += pCharWidths [i];
+		advance += pCharWidths [iCharOffset + i];
 
 		if (UT_UCS4_isspace (ch) || (UT_isOverstrikingChar(ch) != UT_NOT_OVERSTRIKING) || last_was_overstriking) {
 			if (!utf8.empty()) {
-				_drawCharsUTF8(utf8.ucs4_str().ucs4_str(), 0, utf8.size(), tdu (xoff + prevAdvance), tdu(yoff));
+				_drawCharsUTF8(utf8.ucs4_str().ucs4_str(), 0, utf8.size(), xoff + prevAdvance, yoff);
 				utf8.clear ();
 			}
 
@@ -319,7 +317,7 @@ void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	
 	// chars remain - flush buffer
 	if (!utf8.empty ())
-		_drawCharsUTF8(utf8.ucs4_str().ucs4_str(), 0, utf8.size(), tdu (xoff + prevAdvance), tdu(yoff));
+		_drawCharsUTF8(utf8.ucs4_str().ucs4_str(), 0, utf8.size(), xoff + prevAdvance, yoff);
 }
 
 void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset,
@@ -344,7 +342,7 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 	// each glyph.  PostScript interprets the yoff as the baseline,
 	// which doesn't match this expectation.  Adding the ascent of the
 	// font will bring it back to the correct position.
-	yoff += tdu (getFontAscent());
+	yoff += getFontAscent();
 
 	// unsigned buffer holds Latin-1 data to character code 255
 	char buf[LINE_BUFFER_SIZE];
@@ -372,15 +370,14 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 		}
 
 		currentChar = *pS;
-		if (!((currentChar >= 'a' && currentChar <= 'z') || (currentChar >= 'A' && currentChar <= 'Z') ||
-			  currentChar == ' '))
+		if (currentChar < ' ' || currentChar > '~' || currentChar == '(' || currentChar == ')' || currentChar == '\\')
 		{
 			// if currentChar is not an english character, we will have to write it
 			// using parentheses
 			if(open_bracket)
 			{
 				open_bracket = false;
-				sprintf(static_cast<char *>(pD),") %d %d MS\n",xoff,yoff);
+				sprintf(static_cast<char *>(pD),") %d %d MS\n",tdu(xoff),tdu(yoff));
 				xoff += curwidth;
 				curwidth =0;
 				m_ps->writeBytes(buf);
@@ -389,7 +386,7 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 			else if(!using_names)
 			{
 				xoff += curwidth;
-				sprintf(static_cast<char *>(pD)," %d %d MV ",xoff,yoff);
+				sprintf(static_cast<char *>(pD)," %d %d MV ",tdu(xoff),tdu(yoff));
 				curwidth =0;
 				pD = buf + strlen(buf);
 				using_names = true;
@@ -436,14 +433,14 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 
 			*pD++ = static_cast<char>(currentChar);
 		}
-		curwidth += tdu (measureUnRemappedChar(currentChar));
+		curwidth += measureUnRemappedChar(currentChar);
 		xxx_UT_DEBUGMSG((" width %d curwidth %d xoff %d curwidth+xoff %d char %c \n", measureUnRemappedChar(currentChar),curwidth, xoff,xoff+curwidth,static_cast<char>(currentChar)));
 		pS++;
 	}
 	if(open_bracket)
 	{
 		*pD++ = ')';
-		sprintf(static_cast<char *>(pD)," %d %d MS\n",xoff,yoff);
+		sprintf(static_cast<char *>(pD)," %d %d MS\n",tdu(xoff),tdu(yoff));
 	}
 	else
 	{
@@ -455,7 +452,6 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 	if(ae)
 		delete ae;
 }
-#endif // #ifndef WITH_PANGO
 
 void PS_Graphics::drawLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2, UT_sint32 y2)
 {
@@ -809,7 +805,7 @@ void PS_Graphics::_emit_DocumentNeededResources(void)
 
 	for (k=0; k<kLimit; k++)
 	{
-		PSFont * psf = static_cast<PSFont *>(m_vecFontList.getNthItem(k));
+		PSFont * psf = m_vecFontList.getNthItem(k);
 		if(bFontKeyword)
 		{
 			vec.addItem(static_cast<void *>(UT_strdup("font")));
@@ -833,12 +829,7 @@ void PS_Graphics::_emit_DocumentNeededResources(void)
 	}
 
 	// TODO add any other resources here
-	bool bEmbedFonts;
-	XAP_App::getApp()->getPrefsValueBool(reinterpret_cast<const XML_Char *>(XAP_PREF_KEY_EmbedFontsInPS), &bEmbedFonts);
-	if(bEmbedFonts)
-	  m_ps->formatComment("DocumentSuppliedResources",&vec);
-	else
-	  m_ps->formatComment("DocumentNeededResources",&vec);
+	m_ps->formatComment("DocumentSuppliedResources",&vec);
 
 	UT_VECTOR_FREEALL(char*, vec);
 }
@@ -849,114 +840,71 @@ void PS_Graphics::_emit_IncludeResource(void)
    	UT_uint32 k,n;
    	UT_uint32 kLimit = m_vecFontList.getItemCount();
 
-	// we want to have a checkbox in the Preferences that would allow
-	// to disable splating of the fonts into the output, since people who
-	// use Ghostscript my simply register their fonts with GS and do not
-	// need them in the document
-	bool bEmbedFonts;
-	XAP_App::getApp()->getPrefsValueBool(reinterpret_cast<const XML_Char *>(XAP_PREF_KEY_EmbedFontsInPS), &bEmbedFonts);
-	UT_DEBUGMSG(("bEmbedFonts: %d\n",bEmbedFonts));
+	// fonts are always embedded in the document now, as this was causing way
+	// too many bugreports; just try to live with it that the document 
+	// is a bit bigger now; see bug 6585, comment 14
 
-    if(bEmbedFonts)
-    {
-    	for (k=0; k<kLimit; k++)
-    	{
-    		char buf[128];
-
-    		PSFont * psf = static_cast<PSFont *>(m_vecFontList.getNthItem(k));
-
-    		// Instead of including the resources, we actually splat the fonts
-    		// into the document.  This looks really slow... perhaps buffer line
-    		// by line or in larger chunks the font data.
-    		XAP_UnixFont * unixfont = psf->getUnixFont();
-
-			UT_DEBUGMSG(("ps: Look at Embedding font number %d name %s \n",k,unixfont->getFontKey()));
-		
-    		bool match = false;
-            const char * pName = unixfont->getFontKey();
-    		for(size_t i = 0; i < vec.getItemCount(); ++i)
-    		{
-				if(!strcmp(pName,reinterpret_cast<const char*>(vec.getNthItem(i))))
-    			{
-
-					UT_DEBUGMSG(("_ps: Font already emitted, forget it. \n"));
-			    	match=true;
-    				break;
-    			}
-    		}
-    		if(match)
-		    	continue;
-            
-            vec.addItem(reinterpret_cast<const void*>(pName));
-			UT_DEBUGMSG(("PS: Aboiut to embed font %s \n",pName));
-    		// Make sure the font file will open, maybe it disappeared...
-			UT_ASSERT(m_ps);
-			if (!unixfont->embedInto(*m_ps))
-    		{
-				UT_String message("Font data file [");
-				message += unixfont->getFontfile();
-				message += "cannot be opened for reading!\n"
-					"Did it disappear on us?  AbiWord can't print without\n"
-					"this file; your PostScript might be missing this resource.";
-
-    			// we don't have any frame info, so we use the non-parented dialog box
-    			messageBoxOK(message.c_str());
-    			return;
-    		}
-		
-    		// NOTE : here's an internationalization process step.  If the font
-    		// NOTE : encoding is NOT "iso8859", we do not emit this macro.
-    		// NOTE : this keeps fonts like Standard Symbols, and really
-    		// NOTE : any other encoding, from being mangled.  however, it's
-    		// NOTE : not intended to guarantee that these other encodings
-    		// NOTE : actually work.  that requires more design work.
-
-    		// write findfont
-			UT_String stName(psf->getUnixFont()->getPostscriptName());
-    		g_snprintf(buf, sizeof (buf), "/%s findfont\n", stName.c_str());
-    		m_ps->writeBytes(buf);
-
-    		// exec the swapper macro
-    		g_snprintf(buf, sizeof (buf), "/%s EXC\n", stName.c_str());
-    		m_ps->writeBytes(buf);
-
-    	}
-    }
-	else
+	for (k=0; k<kLimit; k++)
 	{
-		// when not embeding fonts, we have to issue %IncludeResource statement
-		bool bFontKeyword = true;
-		const char* pFResource[2] = {"font", NULL};
-		for (k=0; k<kLimit; k++)
+		char buf[128];
+
+		PSFont * psf = m_vecFontList.getNthItem(k);
+
+		// Instead of including the resources, we actually splat the fonts
+		// into the document.  This looks really slow... perhaps buffer line
+		// by line or in larger chunks the font data.
+		XAP_UnixFont * unixfont = psf->getUnixFont();
+
+		UT_DEBUGMSG(("ps: Look at Embedding font number %d name %s \n",k,unixfont->getFontKey()));
+	
+		bool match = false;
+		const char * pName = unixfont->getFontKey();
+		for(size_t i = 0; i < vec.getItemCount(); ++i)
 		{
-			PSFont * psf = static_cast<PSFont *>(m_vecFontList.getNthItem(k));
-			if(bFontKeyword)
+			if(!strcmp(pName,reinterpret_cast<const char*>(vec.getNthItem(i))))
 			{
-				vec.addItem(static_cast<void *>(UT_strdup("font")));
-				bFontKeyword = false;
-			}
-            
-			// only include each font name once
-			UT_String pName(psf->getUnixFont()->getPostscriptName());
-			bool bFound = false;
-			for(n = 0; n < vec.getItemCount(); n++)
-			{
-				if(!UT_strcmp(pName.c_str(), reinterpret_cast<const char *>(vec.getNthItem(n))))
-				{
-					bFound = true;
-					break;
-				}
-			}
-            
-			if(!bFound)
-			{
-				vec.addItem(reinterpret_cast<void*>(UT_strdup(pName.c_str())));
-				pFResource[1] = pName.c_str();
-				m_ps->formatComment("IncludeResource", pFResource, 2);
+
+				UT_DEBUGMSG(("_ps: Font already emitted, forget it. \n"));
+				match=true;
+				break;
 			}
 		}
+		if(match)
+			continue;
+		
+		vec.addItem(reinterpret_cast<const void*>(pName));
+		UT_DEBUGMSG(("PS: Aboiut to embed font %s \n",pName));
+		// Make sure the font file will open, maybe it disappeared...
+		UT_ASSERT(m_ps);
+		if (!unixfont->embedInto(*m_ps))
+		{
+			UT_String message("Font data file [");
+			message += unixfont->getFontfile();
+			message += "cannot be opened for reading!\n"
+				"Did it disappear on us?  AbiWord can't print without\n"
+				"this file; your PostScript might be missing this resource.";
+
+			// we don't have any frame info, so we use the non-parented dialog box
+			messageBoxOK(message.c_str());
+			return;
+		}
+	
+		// NOTE : here's an internationalization process step.  If the font
+		// NOTE : encoding is NOT "iso8859", we do not emit this macro.
+		// NOTE : this keeps fonts like Standard Symbols, and really
+		// NOTE : any other encoding, from being mangled.  however, it's
+		// NOTE : not intended to guarantee that these other encodings
+		// NOTE : actually work.  that requires more design work.
+
+		// write findfont
+		UT_String stName(psf->getUnixFont()->getPostscriptName());
+		g_snprintf(buf, sizeof (buf), "/%s findfont\n", stName.c_str());
+		m_ps->writeBytes(buf);
+
+		// exec the swapper macro
+		g_snprintf(buf, sizeof (buf), "/%s EXC\n", stName.c_str());
+		m_ps->writeBytes(buf);
 
-		UT_VECTOR_FREEALL(char*, vec);
 	}
         
 	// TODO add any other IncludeResource's here
@@ -1007,7 +955,7 @@ void PS_Graphics::_emit_FontMacros(void)
 
 	for (k=0; k<kLimit; k++)
 	{
-		PSFont * psf = static_cast<PSFont *>(m_vecFontList.getNthItem(k));
+		PSFont * psf = m_vecFontList.getNthItem(k);
 		UT_String stName(psf->getUnixFont()->getPostscriptName());
 		g_snprintf(buf,sizeof(buf),"  /F%d {%d /%s FSF setfont} bind def\n", k,
 				psf->getSize(), stName.c_str());
@@ -1040,36 +988,37 @@ void PS_Graphics::_emit_SetColor(void)
   // used for any averaging
   unsigned char newclr;
 
-	const char * old_locale = setlocale(LC_NUMERIC,"C");
-	switch(m_cs)
-	{
-	case GR_Graphics::GR_COLORSPACE_COLOR:
-        g_snprintf(buf, sizeof (buf), "%.8f %.8f %.8f setrgbcolor\n",
-                (static_cast<float>(m_currentColor.m_red) / static_cast<float>(255.0)),
-                (static_cast<float>(m_currentColor.m_grn) / static_cast<float>(255.0)),
-                (static_cast<float>(m_currentColor.m_blu) / static_cast<float>(255.0)));
-		break;
-	case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
-		newclr = static_cast<unsigned char>(( static_cast<float>(0.30) * m_currentColor.m_red +
-											  static_cast<float>(0.59) * m_currentColor.m_grn +
-											  static_cast<float>(0.11) * m_currentColor.m_blu ));
-		g_snprintf(buf, sizeof(buf), "%.8f setgray\n", static_cast<float>(newclr) / static_cast<float>(255.0));
-		break;
-	case GR_Graphics::GR_COLORSPACE_BW:
-		// Black & White is a special case of the Gray color space where
-		// all colors are 0 (black) and all absence of color is 1 (white)
-
-        if (m_currentColor.m_red + m_currentColor.m_grn + m_currentColor.m_blu > 3*250) // yay, arbitrary threshold!
-			g_snprintf(buf,sizeof(buf),"1 setgray\n");
-         else		
-			 g_snprintf(buf,sizeof(buf),"0 setgray\n");
-		break;
-	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	}
-	setlocale(LC_NUMERIC,old_locale); // restore original locale
+  {
+	  UT_LocaleTransactor(LC_NUMERIC, "C");
+	  switch(m_cs)
+	  {
+	  case GR_Graphics::GR_COLORSPACE_COLOR:
+		  g_snprintf(buf, sizeof (buf), "%.8f %.8f %.8f setrgbcolor\n",
+					 (static_cast<float>(m_currentColor.m_red) / static_cast<float>(255.0)),
+					 (static_cast<float>(m_currentColor.m_grn) / static_cast<float>(255.0)),
+					 (static_cast<float>(m_currentColor.m_blu) / static_cast<float>(255.0)));
+		  break;
+	  case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
+		  newclr = static_cast<unsigned char>(( static_cast<float>(0.30) * m_currentColor.m_red +
+												static_cast<float>(0.59) * m_currentColor.m_grn +
+												static_cast<float>(0.11) * m_currentColor.m_blu ));
+		  g_snprintf(buf, sizeof(buf), "%.8f setgray\n", static_cast<float>(newclr) / static_cast<float>(255.0));
+		  break;
+	  case GR_Graphics::GR_COLORSPACE_BW:
+		  // Black & White is a special case of the Gray color space where
+		  // all colors are 0 (black) and all absence of color is 1 (white)
+		  
+	  if (m_currentColor.m_red + m_currentColor.m_grn + m_currentColor.m_blu > 3*250) // yay, arbitrary threshold!
+		  g_snprintf(buf,sizeof(buf),"1 setgray\n");
+	  else		
+		  g_snprintf(buf,sizeof(buf),"0 setgray\n");
+	  break;
+	  default:
+		  UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	  }
+  }
 	
-	m_ps->writeBytes(buf);
+  m_ps->writeBytes(buf);
 }
 
 GR_Image* PS_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
@@ -1155,14 +1104,16 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 			image->width, image->height * -1, image->height);
 	m_ps->writeBytes(buf);
 	
-	g_snprintf(buf, sizeof (buf),"{currentfile\n  rowdata readhexstring pop}\nfalse 3\ncolorimage\n");
+	int byteWidth = (pPSImage->getHasAlpha() ? 4 : 3);
+
+	g_snprintf(buf, sizeof (buf),"{currentfile rowdata readhexstring pop}\nfalse %d colorimage\n", byteWidth);
 	m_ps->writeBytes(buf);
 	
 	// "image" is full of 24 bit data; throw the data specifications
 	// and then the raw data (in hexadeicmal) into the file.
 	UT_Byte * start = image->data;
 	UT_Byte * cursor = NULL;
-	UT_Byte * end = start + image->width * image->height * 3; // 3 bytes per pixel
+	UT_Byte * end = start + image->width * image->height * byteWidth;
 	UT_Byte hexbuf[3];
 	int col;
 	for (cursor = start, col = 0 ; cursor < end; cursor++, col++)
@@ -1174,8 +1125,7 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 		if (col == 40) // 2 chars per round, 80 columns total
 		{
 			col = -1;
-			g_snprintf(reinterpret_cast<char *>(&hexbuf[0]), sizeof(hexbuf), "\n");
-			m_ps->writeBytes(hexbuf, 1);
+			m_ps->writeBytes("\n", 1);
 		}
 	}
 	g_snprintf(buf, sizeof(buf), "\n");
@@ -1183,8 +1133,7 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 
 	// recall all that great info
 	g_snprintf(buf, sizeof(buf),"grestore\n");
-	m_ps->writeBytes(buf);
-	
+	m_ps->writeBytes(buf);	
 }
 
 void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
@@ -1225,15 +1174,13 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 			image->width, image->height * -1, image->height);
 	m_ps->writeBytes(buf);
 
-	g_snprintf(buf, sizeof(buf),"{currentfile\n  rowdata readhexstring pop}\nimage\n");
-	// color image does:
-	// sprintf(buf, "{currentfile\n  rowdata readhexstring pop}\nfalse 3\ncolorimage\n");
+	g_snprintf(buf, sizeof(buf),"{currentfile rowdata readhexstring pop}\nimage\n");
 	m_ps->writeBytes(buf);
 	
 	UT_Byte * start = image->data;
 	UT_Byte * cursor = NULL;
-	// 3 bytes per pixel in original RGB data
-	UT_Byte * end = start + image->width * image->height * 3;
+	// 1 byte per pixel in original gray data
+	UT_Byte * end = start + image->width * image->height;
 
 	UT_Byte hexbuf[3];
 	
@@ -1247,6 +1194,15 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 		  throwing that (in hex) to the output file.
 		*/
 
+		/*
+		  libpng says that we should do this:
+
+		  int rw = red_weight * 65536;
+		  int gw = green_weight * 65536;
+		  int bw = 65536 - (rw + gw);
+		  gray = (rw*red + gw*green + bw*blue)/65536;
+		*/
+
 		// TODO : Balance these colors!  I don't like the output
 		// TODO : I get from a simple average or adding the YIQ
 		// TODO : weights.  Look at Netscape for something better.
@@ -1257,7 +1213,7 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 										  static_cast<float>(3.0) )) );
 
 		m_ps->writeBytes(hexbuf, 2);
-		if (col == 38)
+		if (col == 40)
 		{
 			m_ps->writeByte(static_cast<UT_Byte>('\n'));
 			col = 0;
@@ -1339,18 +1295,16 @@ void PS_Graphics::setPageSize(char* pageSizeName, UT_uint32 iwidth, UT_uint32 ih
 	m_iHeight = iheight;
 }
 
-#ifdef WITH_PANGO
 
-/*!
-  This is a dummy function that should never be called, since the PS graphics ovewrited the
-  whole drawPangoGlyphString function (not implemented yet)
- */
-void PS_Graphics::_drawFT2Bitmap(UT_sint32 x, UT_sint32 y, FT_Bitmap * pBitmap) const
+GR_Graphics *   PS_Graphics::graphicsAllocator(GR_AllocInfo& info)
 {
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	PS_GraphicsAllocInfo &allocator = (PS_GraphicsAllocInfo&)info;
+
+	return new PS_Graphics(allocator.m_fileName, allocator.m_title,
+						   allocator.m_softwareName,
+						   allocator.m_fontManager, allocator.m_isFile, 
+						   allocator.m_app);
 }
 
-#endif
-
 /***************************************************************************/
 /***************************************************************************/
