@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 2001 AbiSource, Inc.
  * Copyright (C) 2001 Dom Lachowicz <cinamod@hotmail.com> 
@@ -23,10 +25,20 @@
 
 // Singleton class that will load/unload modules for us
 
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
+#endif
 
-class XAP_Module;
-class UT_Vector;
+#include "ut_vector.h"
+
+#include "xap_Module.h"
+
+
+class XAP_Spider;
 
 class ABI_EXPORT XAP_ModuleManager
 {
@@ -40,19 +52,31 @@ public:
 
 	static XAP_ModuleManager & instance ();
 
-	bool         loadBundle (const char * szFilename, const char * szBundlename);
 	bool         loadModule (const char * szFilename);
-	bool         unloadModule (XAP_Module * module);
 
-	const UT_Vector *  enumModules () const;
+	bool         loadPreloaded (XAP_Plugin_Registration fnRegister,
+								XAP_Plugin_Registration fnDeregister,
+								XAP_Plugin_VersionCheck fnSupportsVersion);
+
+	UT_uint32    registerPending ();
+
+	void         unloadModule (XAP_Module * module);
+private:
+	void         unloadModule (UT_sint32 ndx);
+public:
+	void         unloadUnregistered ();
+	void         unloadAllPlugins ();
+
+	const UT_GenericVector<XAP_Module*> * enumModules () const;
 
 private:
 
 	XAP_ModuleManager(const XAP_ModuleManager &);		// no impl
 	void operator=(const XAP_ModuleManager &);	        // no impl
 
-	UT_Vector *m_modules;
+	XAP_Spider * m_spider;
 
+	UT_GenericVector<XAP_Module*> * m_modules;
 };
 
 #endif /* XAP_MODULE_MANAGER_H */
