@@ -247,7 +247,9 @@ dcl(DLG_ULANG_LangTitle,				"Set Language")
 dcl(DLG_ULANG_LangLabel,				"Select Language:")
 dcl(DLG_ULANG_AvailableLanguages,			"Available Languages")
 dcl(DLG_ULANG_SetLangButton,				"&Set Language")
-
+dcl(DLG_ULANG_DefaultLangLabel,         "Default language: ")
+dcl(DLG_ULANG_DefaultLangChkbox,        "Make default for document")
+	
 /* ClipArt Dialog */
 dcl(DLG_CLIPART_Title, "Clip Art")
 
@@ -352,6 +354,7 @@ dcl(LANG_IT_IT, 							"Italian (Italy)")
 dcl(LANG_JA_JP, 							"Japanese")
 dcl(LANG_KN_IN,                             "Kannada")
 dcl(LANG_KO_KR, 							"Korean")
+dcl(LANG_KU, 								"Kurdish")
 dcl(LANG_LO_LA,                             "Lao")
 dcl(LANG_LA_IT, 							"Latin (Renaissance)")	// Is _IT the right thing here?
 dcl(LANG_LV_LV, 							"Latvian")
