@@ -24,10 +24,7 @@
 #include "ut_debugmsg.h"
 #include "ut_Language.h"
 
-#ifdef HAVE_PSPELL
-#include "pspell_checker.h"
-typedef PSpellChecker SpellCheckerClass;
-#elif HAVE_ENCHANT
+#ifdef HAVE_ENCHANT
 #include "enchant_checker.h"
 typedef EnchantChecker SpellCheckerClass;
 #else
@@ -66,7 +63,7 @@ SpellChecker::SpellCheckResult SpellChecker::checkWord(const UT_UCSChar* word, s
 
 	m_bIsBarbarism = false;
 	m_bIsDictionaryWord = false;
-
+	
     if (m_BarbarismChecker.checkWord (word, len))
 	{
 		UT_DEBUGMSG(("SPELL:  spell %lx %s barb \"%s\"\n", this, getLanguage().c_str(), UT_UTF8String (word, len).utf8_str()));
@@ -81,9 +78,9 @@ SpellChecker::SpellCheckResult SpellChecker::checkWord(const UT_UCSChar* word, s
 	return ret;
 }
 
-UT_Vector *SpellChecker::suggestWord(const UT_UCSChar* word, size_t len)
+UT_GenericVector<UT_UCSChar*> *SpellChecker::suggestWord(const UT_UCSChar* word, size_t len)
 {
-	UT_Vector *pvSugg = _suggestWord(word, len);
+	UT_GenericVector<UT_UCSChar*> *pvSugg = _suggestWord(word, len);
 
 	m_BarbarismChecker.suggestWord(word, len, pvSugg);
 
@@ -92,7 +89,6 @@ UT_Vector *SpellChecker::suggestWord(const UT_UCSChar* word, size_t len)
 
 bool SpellChecker::addToCustomDict (const UT_UCSChar *word, size_t len)
 {
-	// TODO: make this support language tags, subclass this for pspell
 	return XAP_App::getApp()->addWordToDict (word, len);
 }
 
@@ -187,7 +183,7 @@ SpellManager::requestDictionary (const char * szLang)
 	{
 		return static_cast<SpellCheckerClass *>(const_cast<void *>(m_map.pick (szLang)));
 	}
-
+	
 	// not found, so insert it
 	checker = new SpellCheckerClass ();
 
@@ -199,15 +195,16 @@ SpellManager::requestDictionary (const char * szLang)
 		m_lastDict = checker;
 		m_nLoadedDicts++;
 		checker->setDictionaryFound(true);
-		return checker;
     }
 	else
     {
 		checker->setDictionaryFound(false);
 		m_missingHashs += szLang;
 		delete checker;
-		return 0;
+		checker = NULL;
     }
+
+	return checker;
 }
 
 /*!
