@@ -34,12 +34,15 @@
 #include "gr_DrawArgs.h"
 #include "fv_View.h"
 #include "fp_FootnoteContainer.h"
+#include "fl_TOCLayout.h"
 #include "fl_FootnoteLayout.h"
+#include "fp_FrameContainer.h"
+#include "fl_FrameLayout.h"
 #include "fp_TableContainer.h"
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
 #include "ut_units.h"
-
+#include "gr_Painter.h"
 
 fp_Page::fp_Page(FL_DocLayout* pLayout,
 		 FV_View* pView,
@@ -53,7 +56,10 @@ fp_Page::fp_Page(FL_DocLayout* pLayout,
 		m_bNeedsRedraw(true),
 		m_pOwner(pOwner),
 		m_pFooter(0),
-		m_pHeader(0)
+		m_pHeader(0),
+		m_FillType(NULL,NULL,FG_FILL_TRANSPARENT),
+		m_pLastMappedTOC(NULL),
+		m_iCountWrapPasses(0)
 {
 	UT_ASSERT(pLayout);
 	UT_ASSERT(pOwner);
@@ -62,6 +68,10 @@ fp_Page::fp_Page(FL_DocLayout* pLayout,
 	UT_ASSERT(pG);
 	m_vecColumnLeaders.clear();
 	m_iResolution = pG->getResolution();
+	m_rDamageRect.left = 0;
+	m_rDamageRect.top = 0;
+	m_rDamageRect.width = 0;
+	m_rDamageRect.height = 0;
 	UT_DEBUGMSG(("!!!!!!!!!!!!!!!!!!!!!!!!!!Created Page %x \n",this));
 }
 
@@ -76,15 +86,42 @@ fp_Page::~fp_Page()
 	}
 }
 
+/*!
+ * FillType class for the page.
+ */
+fg_FillType * fp_Page::getFillType(void)
+{
+	return &m_FillType;
+}
+
 bool fp_Page::isEmpty(void) const
 {
-	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0))
+	if((m_vecColumnLeaders.getItemCount() == 0) && (m_vecFootnotes.getItemCount() == 0) && (m_vecFrames.getItemCount() == 0))
 	{
 		return true;
 	}
 	return false;
 }
 
+bool fp_Page::isOnScreen(void)
+{
+	if(!m_pView)
+	{
+	    return false;
+	}
+	UT_sint32 xoff,yoff;
+	m_pView->getPageScreenOffsets(this,xoff,yoff);
+	if(yoff+getHeight() < 0)
+	{
+		return false;
+	}
+	if(yoff > getHeight())
+	{
+		return false;
+	}
+	return true;
+}
+
 UT_sint32 fp_Page::getWidth(void) const
 {
 	return static_cast<UT_sint32>(m_iResolution * m_pageSize.Width(DIM_IN));
@@ -100,6 +137,394 @@ UT_sint32 fp_Page::getColumnGap(void) const
 	return getOwningSection()->getColumnGap();
 }
 
+/*!
+ * This method scans the page, looking for lines that overlap wrapped
+ * positioned frames. As it finds them it records the line and the block
+ * After scanning the page it calls a method in the block to rebreak
+ * the papargraph starting from the line supplied, but now making sure
+ * to not overlap any wrapped objects.
+ * If it does a re-break of any paragraph it returns the first container
+ * in the page. 
+ * and sets pNextColumn to the first column on the page.
+ * fp_ColumnBreaker then lays out the page again.
+ * If there are no rebreaks it returns NULL and fp_ColumnBreaker moves on to
+ * the next page.
+ * pNextCol is the next column fb_ColumnBreaker will evaluate it is used
+ * as an output..
+ */
+fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
+{
+	if(m_iCountWrapPasses > 199)
+	{
+		return NULL;
+	}
+	m_iCountWrapPasses++;
+#if DEBUG
+	UT_sint32 iPage = getDocLayout()->findPage(this);
+	UT_DEBUGMSG(("Wrap passes = %d page %x page number %d \n",m_iCountWrapPasses,this,iPage ));
+#endif
+	UT_sint32 i= 0;
+	UT_sint32 nWrapped = 0;
+	fp_Container * pFirst = NULL;
+	fl_BlockLayout * pFirstBL = NULL;
+	for(i=0; i < static_cast<UT_sint32>(countColumnLeaders()); i++)
+	{
+		fp_Column * pCol = getNthColumnLeader(i);
+		if(i == 0)
+		{
+			pFirst = static_cast<fp_Container *>(pCol->getNthCon(0));
+			if(pFirst == NULL)
+			{
+				return NULL;
+			}
+		}
+		while(pCol)
+		{
+			nWrapped += pCol->getNumWrapped();
+			pCol = static_cast<fp_Column *>(pCol->getNext());
+		}
+	}
+	UT_sint32 nWrappedObjs = 0;
+	for(i=0; i< static_cast<UT_sint32>(countFrameContainers()); i++)
+	{
+		fp_FrameContainer * pFrame = getNthFrameContainer(i);
+		if(pFrame->isWrappingSet())
+		{
+			nWrappedObjs++;
+		}
+	}
+	if((nWrapped == 0) && (nWrappedObjs == 0))
+	{
+		xxx_UT_DEBUGMSG(("page %x nWrapped %d nWrappedObjs %d does not need updating \n",this,nWrapped,nWrappedObjs));
+		return NULL;
+	}
+	bool bFormatAllWrapped = ((nWrapped > 0) && (nWrappedObjs == 0));
+	UT_GenericVector<_BL *> vecBL;
+	vecBL.clear();
+	for(i=0; i < static_cast<UT_sint32>(countColumnLeaders()); i++)
+	{
+		fp_Column * pCol = getNthColumnLeader(i);
+		while(pCol)
+		{
+			UT_sint32 j = 0;
+			for(j=0; j < static_cast<UT_sint32>(pCol->countCons()); j++)
+			{
+				fp_Container * pCon = static_cast<fp_Container *>(pCol->getNthCon(j));
+				xxx_UT_DEBUGMSG(("Look at con %x at j %d \n",pCon,j));
+				if(pCon->getContainerType() == FP_CONTAINER_LINE)
+				{
+					fp_Line * pLine = static_cast<fp_Line *>(pCon);
+					UT_Rect recLeft;
+					UT_Rect recRight;
+					pLine->genOverlapRects(recLeft,recRight);
+					if(recLeft.width == 0 && recRight.width == 0)
+					{
+						pLine->setWrapped(false);
+					}
+					else if((recLeft.width < 0) || (recRight.width <0))
+					{
+//
+// SOMETHING HAS GONE HORRIBALLY WRONG. Try to recover by collapsing the
+// content in this column and rebuilding
+//
+						UT_DEBUGMSG(("-ve width here!!!! %x left %d right %d \n",pLine,recLeft.width,recRight.width));
+						UT_VECTOR_PURGEALL(_BL *, vecBL);
+						fl_BlockLayout * pBL = pLine->getBlock();
+						fl_BlockLayout * pFirst = pBL;
+						fp_Column * pCol = static_cast<fp_Column *>(pLine->getColumn());
+						bool bLoop = true;
+						while(bLoop)
+						{
+							if(pBL && pBL->getContainerType() == FL_CONTAINER_BLOCK)
+							{
+								if(pBL->getFirstContainer() && (pBL->getFirstContainer()->getColumn() == pCol))
+								{
+									bLoop = true;
+									pFirst = pBL;
+								}
+								pBL = static_cast<fl_BlockLayout *>(pBL->getPrev());
+							}
+							else
+							{
+								bLoop = false;
+							}
+						}
+						fp_Column * pFirstCol = static_cast<fp_Column *>(pFirst->getFirstContainer()->getColumn());
+						pBL = pFirst;
+						UT_GenericVector<fl_BlockLayout *> vecCollapse;
+						bLoop = true;
+						while(bLoop)
+						{
+							if(pBL && pBL->getContainerType() == FL_CONTAINER_BLOCK)
+							{
+								if(pBL->getFirstContainer() && (pBL->getFirstContainer()->getColumn() == pCol))
+								{
+									bLoop = true;
+									vecCollapse.addItem(pBL);
+								}
+								pBL = static_cast<fl_BlockLayout *>(pBL->getNext());
+							}
+							else
+							{
+								bLoop = false;
+							}
+						}
+						UT_sint32 k = 0;
+						for(k=0; k<static_cast<UT_sint32>(vecCollapse.getItemCount());k++)
+						{
+							pBL = vecCollapse.getNthItem(k);
+							pBL->collapse();
+							pBL->format();
+						}
+						pNextCol = pFirstCol;
+						fp_Container * pNewFirstCon = static_cast<fp_Container *>(pNextCol->getNthCon(0));
+						return pNewFirstCon;
+					}
+					else
+					{
+						pLine->setWrapped(true);
+						if(pLine->getPrev() && !pLine->isSameYAsPrevious())
+						{
+							fp_Line * pPrev = static_cast<fp_Line *>(pLine->getPrev());
+							if(pPrev->getY() == pLine->getY())
+							{
+								pLine->setSameYAsPrevious(true);
+							}
+						}
+					}
+					if(bFormatAllWrapped)
+					{
+						if(pLine->isWrapped())
+						{
+							fl_BlockLayout * pBL = pLine->getBlock();
+							bool bPrev = false;
+							UT_sint32 k = 0;
+							for(k=0; k<static_cast<UT_sint32>(vecBL.getItemCount()); k++)
+							{
+								_BL * ppBL = vecBL.getNthItem(k);
+								if(ppBL->m_pBL == pBL)
+								{
+									bPrev = true;
+								}
+							}
+							if(!bPrev)
+							{
+								_BL * pBLine = new _BL(pBL,pLine);
+								vecBL.addItem(pBLine);
+							}
+							k =j;
+							while(pLine && pLine->getBlock() == pBL)
+							{
+								k++;
+								if(k >= static_cast<UT_sint32>(pCol->countCons()))
+								{
+									break;
+								}
+								pCon = static_cast<fp_Container *>(pCol->getNthCon(k));
+								if(pCon->getContainerType() == FP_CONTAINER_LINE)
+								{
+									pLine = static_cast<fp_Line *>(pCon);
+								}
+								else
+								{
+									break;
+								}
+							}
+							j = k-1;
+						}
+					}
+					else
+					{
+//
+// OK look to see if this line either overlaps a wrapped object or has
+// some space where a wrapped object should be
+//
+						bool bFoundOne = false;
+						fp_Line * pPrev = static_cast<fp_Line *>(pLine->getPrev());
+						if(pLine->isWrapped())
+						{
+							if(overlapsWrappedFrame(pLine))
+							{
+//
+// Wrapped line overlaps a wrapped frame
+//
+								xxx_UT_DEBUGMSG(("Found wrapped line %x that overlaps \n",pLine));
+								bFoundOne = true;
+							}
+							else if(pPrev && pLine->isSameYAsPrevious() && (pPrev->getY() != pLine->getY()))
+							{
+								pLine = pPrev;
+								j--;
+								bFoundOne = true;
+							}
+							else if(m_iCountWrapPasses < 101)
+							{
+//
+// Look to see if the wrapped line has no white space that overlaps
+// a wrapped object. We don't do this if we've tried more than 100
+// time to layout the page.
+//
+								
+								if(!overlapsWrappedFrame(recLeft) &&
+								   !overlapsWrappedFrame(recRight))
+								{
+									xxx_UT_DEBUGMSG(("Found wrapped line with extra  space %x  \n",pLine));
+									bFoundOne = true;
+								}
+							}
+							else
+							{
+							  xxx_UT_DEBUGMSG(("Not converging \n"));
+							}
+
+						}
+						else
+						{
+//
+// line overlaps a wrapped frame
+//
+							if(overlapsWrappedFrame(pLine))
+							{
+								xxx_UT_DEBUGMSG(("Found unwrapped line %x that overlaps \n",pLine));
+								bFoundOne = true;
+							}
+						}
+						if(bFoundOne)
+						{
+							fl_BlockLayout * pBL = pLine->getBlock();
+							bool bPrev = false;
+							UT_sint32 k = 0;
+							for(k=0; k<static_cast<UT_sint32>(vecBL.getItemCount()); k++)
+							{
+								_BL * ppBL = vecBL.getNthItem(k);
+								if(ppBL->m_pBL == pBL)
+								{
+									bPrev = true;
+								}
+							}
+							if(!bPrev)
+							{
+								_BL * pBLine = new _BL(pBL,pLine);
+								vecBL.addItem(pBLine);
+							}
+							k =j;
+							while(pLine && pLine->getBlock() == pBL)
+							{
+								k++;
+								if(k >= static_cast<UT_sint32>(pCol->countCons()))
+								{
+									break;
+								}
+								pCon = static_cast<fp_Container *>(pCol->getNthCon(k));
+								if(pCon->getContainerType() == FP_CONTAINER_LINE)
+								{
+									pLine = static_cast<fp_Line *>(pCon);
+								}
+								else
+								{
+									break;
+								}
+							}
+							j = k-1;
+						}
+					}
+				}
+			}
+			pCol = static_cast<fp_Column *>(pCol->getFollower());
+		}
+	}
+	if(vecBL.getItemCount() == 0)
+	{
+		return NULL;
+	}
+	_BL * pBLine = vecBL.getNthItem(0);
+	pFirstBL = pBLine->m_pBL;
+	for(i=0; i<static_cast<UT_sint32>(vecBL.getItemCount()); i++)
+	{
+		pBLine = vecBL.getNthItem(i);
+		xxx_UT_DEBUGMSG((" Doing line %x \n",pBLine->m_pL));
+#if DEBUG
+		if(m_iCountWrapPasses > 100)
+		{
+			xxx_UT_DEBUGMSG(("Not converging \n"));
+		}
+#endif
+		xxx_UT_DEBUGMSG(("Do regular rebreak \n"));
+		pBLine->m_pBL->formatWrappedFromHere(pBLine->m_pL,this);
+	}
+	UT_VECTOR_PURGEALL(_BL *, vecBL);
+	fp_Container * pNewFirstCon = NULL;
+	if(pFirstBL)
+	{
+		pNewFirstCon = pFirstBL->getFirstContainer();
+		pNextCol = static_cast<fp_Column *>(pNewFirstCon->getColumn());
+	}
+	else
+	{
+		return pNewFirstCon;
+	}
+	while(pNewFirstCon && pNewFirstCon->getPage() != NULL && pNewFirstCon->getPage() != this)
+	{
+		pNewFirstCon = static_cast<fp_Container *>(pNewFirstCon->getNext());
+	}
+	pNextCol = static_cast<fp_Column *>(pNewFirstCon->getColumn());
+	pNewFirstCon = static_cast<fp_Container *>(pNextCol->getNthCon(0));
+	if(pNewFirstCon->getContainerType() == FP_CONTAINER_LINE)
+	  {
+	    fp_Line * pFLine = static_cast<fp_Line *>(pNewFirstCon);
+	    UT_ASSERT(pFLine->getBlock() && (pFLine->getBlock()->findLineInBlock(pFLine) >= 0));
+	    //	    UT_ASSERT(!pFLine-isEmpty());
+	  }
+	return pNewFirstCon;
+}
+
+
+/*!
+ * Returns true if the supplied rectangle overlaps with one wrapped frame
+ * on the page.
+ */
+bool fp_Page::overlapsWrappedFrame(fp_Line * pLine)
+{
+	UT_Rect * pRec = pLine->getScreenRect();
+	if(pRec == NULL)
+	{
+		return false;
+	}
+	bool bRes = overlapsWrappedFrame(*pRec);
+	delete pRec;
+	return bRes;
+}
+/*!
+ * Returns true if the supplied rectangle overlaps with one wrapped frame
+ * on the page. The rectangle is relative to the screen.
+ */
+bool fp_Page::overlapsWrappedFrame(UT_Rect & rec)
+{
+	UT_sint32 i=0;
+	for(i=0; i<static_cast<UT_sint32>(countFrameContainers());i++)
+	{
+		fp_FrameContainer * pFC = getNthFrameContainer(i);
+		if(!pFC->isWrappingSet())
+		{
+			continue;
+		}
+		UT_Rect * pMyFrameRec = pFC->getScreenRect();
+		fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
+		UT_sint32 iextra = pFL->getBoundingSpace() -2;
+		pMyFrameRec->left -= iextra;
+		pMyFrameRec->top -= iextra;
+		pMyFrameRec->width += 2*iextra;
+		pMyFrameRec->height += 2*iextra;
+		if(rec.intersectsRect(pMyFrameRec))
+		{
+			delete pMyFrameRec;
+			return true;
+		}
+		delete pMyFrameRec;
+	}
+	return false;
+}
+
 /*!
  * This method scans the page for the table that contains the point given
  */
@@ -115,6 +540,11 @@ fp_TableContainer * fp_Page::getContainingTable(PT_DocPosition pos)
 		return NULL;
 	}
 	fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pCell->getContainer());
+	if(m_pView->isInFrame(pos))
+	{
+		return pTab;
+	}
+		
 	UT_sint32 i = 0;
 	UT_sint32 j =0;
 	bool bFound = false;
@@ -234,7 +664,7 @@ UT_sint32 fp_Page::getFilledHeight(fp_Container * prevContainer) const
 	for(i=0; !bstop && (i<  m_vecColumnLeaders.getItemCount()); i++)
 	{
 		maxHeight = 0;
-		pColumn = static_cast<fp_Column *>(m_vecColumnLeaders.getNthItem(i));
+		pColumn = m_vecColumnLeaders.getNthItem(i);
 		totalHeight += pColumn->getDocSectionLayout()->getSpaceAfter();
 		while(pColumn != NULL)
 		{
@@ -347,11 +777,12 @@ void fp_Page::_drawCropMarks(dg_DrawArgs* pDA)
 	   && pDA->pG->queryProperties(GR_Graphics::DGP_SCREEN)
 	   && countColumnLeaders() > 0)
 	{
+		GR_Painter painter(pDA->pG);
+
         fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
         fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
 		UT_ASSERT(m_pOwner == pFirstSectionLayout);
 
-
         UT_sint32 iLeftMargin = pFirstSectionLayout->getLeftMargin();
         UT_sint32 iRightMargin = pFirstSectionLayout->getRightMargin();
         UT_sint32 iTopMargin = pFirstSectionLayout->getTopMargin();
@@ -371,20 +802,20 @@ void fp_Page::_drawCropMarks(dg_DrawArgs* pDA)
 
 		pDA->pG->setLineProperties(pDA->pG->tluD(1.0),
 									 GR_Graphics::JOIN_MITER,
-									 GR_Graphics::CAP_BUTT,
+									 GR_Graphics::CAP_PROJECTING,
 									 GR_Graphics::LINE_SOLID);
 
-        pDA->pG->drawLine(xoffStart, yoffStart, xoffStart, yoffStart - iTopHeight);
-        pDA->pG->drawLine(xoffStart, yoffStart, xoffStart - iLeftWidth, yoffStart);
+        painter.drawLine(xoffStart, yoffStart, xoffStart, yoffStart - iTopHeight);
+        painter.drawLine(xoffStart, yoffStart, xoffStart - iLeftWidth, yoffStart);
 
-        pDA->pG->drawLine(xoffEnd, yoffStart - iTopHeight, xoffEnd, yoffStart);
-        pDA->pG->drawLine(xoffEnd, yoffStart, xoffEnd + iRightWidth, yoffStart);
+        painter.drawLine(xoffEnd, yoffStart - iTopHeight, xoffEnd, yoffStart);
+        painter.drawLine(xoffEnd, yoffStart, xoffEnd + iRightWidth, yoffStart);
 
-        pDA->pG->drawLine(xoffStart, yoffEnd, xoffStart, yoffEnd + iBottomHeight);
-        pDA->pG->drawLine(xoffStart - iLeftWidth, yoffEnd, xoffStart, yoffEnd);
+        painter.drawLine(xoffStart, yoffEnd, xoffStart, yoffEnd + iBottomHeight);
+        painter.drawLine(xoffStart - iLeftWidth, yoffEnd, xoffStart, yoffEnd);
 
-        pDA->pG->drawLine(xoffEnd, yoffEnd, xoffEnd, yoffEnd + iBottomHeight);
-        pDA->pG->drawLine(xoffEnd, yoffEnd, xoffEnd + iRightWidth, yoffEnd);
+        painter.drawLine(xoffEnd, yoffEnd, xoffEnd, yoffEnd + iBottomHeight);
+        painter.drawLine(xoffEnd, yoffEnd, xoffEnd + iRightWidth, yoffEnd);
     }
 }
 
@@ -396,31 +827,48 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 // only call this for printing and honour the option to not fill the paper with
 // color.
 //
+	xxx_UT_DEBUGMSG(("Draw wrap passes = %d \n",m_iCountWrapPasses));
+	m_iCountWrapPasses = 0;
 	int i=0;
-	if(!bAlwaysUseWhiteBackground && !pDA->pG->queryProperties(GR_Graphics::DGP_SCREEN))
+	if(!pDA->pG->queryProperties(GR_Graphics::DGP_SCREEN))
+	{
+		getOwningSection()->getDocLayout()->incrementGraphicTick();
+	}
+	getOwningSection()->checkGraphicTick(pDA->pG);
+	if(!pDA->pG->queryProperties(GR_Graphics::DGP_SCREEN))
+	{
+		getOwningSection()->getDocLayout()->incrementGraphicTick();
+	}
+	if(!pDA->bDirtyRunsOnly)
 	{
 		xxx_UT_DEBUGMSG(("Doing a rectangular color fill \n"));
-		UT_RGBColor * pClr = getOwningSection()->getPaperColor();
  		UT_sint32 xmin = pDA->xoff;
   		UT_sint32 ymin = pDA->yoff;
 
 		UT_sint32 height =getHeight();
 		UT_sint32 width = getWidth();
-		pDA->pG->fillRect(*pClr,xmin,ymin,width,height);
+		UT_sint32 srcX = 0;
+		UT_sint32 srcY = 0;
+		getFillType()->Fill(pDA->pG,srcX,srcY,xmin,ymin,width,height);
 	}
 
     _drawCropMarks(pDA);
 
+	UT_sint32 count = 0;
+
 	// draw each column on the page
-	int count = m_vecColumnLeaders.getItemCount();
+	count = m_vecColumnLeaders.getItemCount();
+
+	GR_Painter painter(pDA->pG);
+
 	for (i=0; i<count; i++)
 	{
-		fp_Column* pCol = static_cast<fp_Column*>(m_vecColumnLeaders.getNthItem(i));
+		fp_Column* pCol = m_vecColumnLeaders.getNthItem(i);
 		while (pCol)
 		{
 			dg_DrawArgs da = *pDA;
 			da.xoff += pCol->getX();
-			da.yoff += pCol->getY();
+			da.yoff += pCol->getY(pDA->pG);
 			pCol->draw(&da);
 
 			fp_Column *pNextCol = pCol->getFollower();
@@ -432,7 +880,7 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 				UT_sint32 x = pDA->xoff + (pCol->getX() + pCol->getWidth() + pNextCol->getX()) / 2;
 				UT_sint32 y = pDA->yoff + pCol->getY();
 				pDA->pG->setColor(m_pView->getColorColumnLine());
-				pDA->pG->drawLine(x, y, x, y + pCol->getHeight());
+				painter.drawLine(x, y, x, y + pCol->getHeight());
 			}
 
 			pCol = pNextCol;
@@ -440,7 +888,7 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 	}
 
 	// draw the page's headers and footers
-    if(m_pView->getViewMode() == VIEW_PRINT)
+    if(m_pView->getViewMode() == VIEW_PRINT  || pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
 	{
 		if (m_pHeader)
 		{
@@ -463,16 +911,106 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 	count = m_vecFootnotes.getItemCount();
 	for (i=0; i<count; i++)
 	{
-		fp_FootnoteContainer* pFC = static_cast<fp_FootnoteContainer*>(m_vecFootnotes.getNthItem(i));
+		fp_FootnoteContainer* pFC = m_vecFootnotes.getNthItem(i);
 		dg_DrawArgs da = *pDA;
+		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) && !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+		{
+			fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+			fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+			da.yoff -= pFirstSectionLayout->getTopMargin();
+		}
+		da.xoff += pFC->getX();
+		da.yoff += pFC->getY();
+		pFC->draw(&da);
+	}
+
+	// draw Frames
+	count = m_vecFrames.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
+		if(m_rDamageRect.intersectsRect(&r))
+		{
+			pFC->setOverWrote();
+		}
+		dg_DrawArgs da = *pDA;
+		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) && !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+		{
+			fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+			fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+			da.yoff -= pFirstSectionLayout->getTopMargin();
+		}
 		da.xoff += pFC->getX();
 		da.yoff += pFC->getY();
 		pFC->draw(&da);
 	}
 
 	m_bNeedsRedraw = false;
+	m_rDamageRect.left = 0;
+	m_rDamageRect.top = 0;
+	m_rDamageRect.width = 0;
+	m_rDamageRect.height = 0;
+
 }
 
+void   fp_Page::expandDamageRect(UT_sint32 x, UT_sint32 y, 
+										 UT_sint32 width, UT_sint32 height)
+{
+//
+// x and y and in screen offsets turn into page coords
+//
+	UT_sint32 xoff,yoff;
+	m_pView->getPageScreenOffsets(this, xoff, yoff);
+	x -= xoff;
+	y -= yoff;
+
+	if(m_rDamageRect.width == 0)
+	{
+		m_rDamageRect.left = x;
+		m_rDamageRect.top=y;
+		m_rDamageRect.width = width;
+		m_rDamageRect.height = height;
+		return;
+	}
+	UT_Rect r(x,y,width,height);
+	m_rDamageRect.unionRect(&r);
+	return;
+}
+
+void   fp_Page::redrawDamagedFrames(dg_DrawArgs* pDA)
+{
+	// draw Frames
+	UT_sint32 count = m_vecFrames.getItemCount();
+	UT_sint32 i = 0;
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
+		if(m_rDamageRect.intersectsRect(&r))
+		{
+			pFC->setOverWrote();
+		}
+		dg_DrawArgs da = *pDA;
+		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) && !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+		{
+			fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
+			fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
+			da.yoff -= pFirstSectionLayout->getTopMargin();
+		}
+		da.xoff += pFC->getX();
+		da.yoff += pFC->getY();
+		pFC->draw(&da);
+	}
+
+	m_rDamageRect.left = 0;
+	m_rDamageRect.top = 0;
+	m_rDamageRect.width = 0;
+	m_rDamageRect.height = 0;
+}
+
+
+
 bool fp_Page::needsRedraw(void) const
 {
 	return m_bNeedsRedraw;
@@ -485,9 +1023,38 @@ UT_uint32 fp_Page::countColumnLeaders(void) const
 
 fp_Column* fp_Page::getNthColumnLeader(UT_sint32 n) const
 {
-	return static_cast<fp_Column*>(m_vecColumnLeaders.getNthItem(n));
+	return m_vecColumnLeaders.getNthItem(n);
 }
 
+/*!
+ * This method is called following a notification of an increase in 
+ * HdrFtr size
+ */
+bool fp_Page::TopBotMarginChanged(void)
+{
+//
+// Adjust header/footer shadows first
+//
+	UT_sint32 iTopM = m_pOwner->getTopMargin();
+	UT_sint32 iBotM = m_pOwner->getBottomMargin();
+	clearScreenFrames();
+	if(m_pHeader)
+	{
+		m_pHeader->clearScreen();
+		m_pHeader->setMaxHeight(iTopM - m_pOwner->getHeaderMargin());
+		m_pHeader->layout();
+	}
+	if(m_pFooter)
+	{
+		m_pFooter->clearScreen();
+		m_pFooter->setMaxHeight(iBotM - m_pOwner->getFooterMargin());
+		m_pFooter->setY(getHeight() - iBotM);
+		m_pFooter->layout();
+	}
+	breakPage();
+	_reformat();
+	return true;
+}
 /*!
  * This method scans the current page to make sure that there is space to at least start
  * every column on the page. Columns with space are deleted and their contents redistributed.
@@ -744,10 +1311,11 @@ void fp_Page::_reformatColumns(void)
 
 	for (i = 0; i < count; i++)
 	{
+#if 0
 		if (iY >= (static_cast<UT_sint32>(getHeight() - iBottomMargin - iFootnoteHeight)))
 		{
-			UT_DEBUGMSG(("SEVIOR: Page incorrectly laid out iYlayoutuints= %d  \n",iY));
-			m_pOwner->markForRebuild();
+			xxx_UT_DEBUGMSG(("SEVIOR: Page incorrectly laid out iYlayoutuints= %d  \n",iY));
+//			m_pOwner->markForRebuild();
 //
 // FIXME see if this code works instead
 //
@@ -757,6 +1325,7 @@ void fp_Page::_reformatColumns(void)
 			return;
 //			break;
 		}
+#endif
 
 		fp_Column* pLeader = getNthColumnLeader(i);
 		UT_ASSERT(pLeader->getContainerType() == FP_CONTAINER_COLUMN);
@@ -790,7 +1359,7 @@ void fp_Page::_reformatColumns(void)
 			UT_ASSERT(pTmpCol->getContainerType() == FP_CONTAINER_COLUMN);
 			pTmpCol->setX(iX);
 			pTmpCol->setY(iY);
-			pTmpCol->setMaxHeight(getHeight() - iBottomMargin - iY);
+			pTmpCol->setMaxHeight(getHeight() - iBottomMargin - iY - iFootnoteHeight);
 			pTmpCol->setWidth(iColWidth);
 
 			if(pSL->getColumnOrder())
@@ -816,7 +1385,7 @@ void fp_Page::_reformatColumns(void)
 // Look for blank space to put more text
 //
 	fp_Column * pFirstOfNext = NULL;
-	fp_Page *pNext = getNext();
+	fp_Page * pNext = getNext();
 	if(pNext && pLastCol)
 	{
 		fp_Container * pLastContainer = static_cast<fp_Container *>(pLastCol->getLastContainer());
@@ -841,7 +1410,7 @@ void fp_Page::_reformatColumns(void)
 			bool bIsTable = (pFirstNextContainer->getContainerType() == FP_CONTAINER_TABLE)  || (countFootnoteContainers() > 0) || (pNext->countFootnoteContainers() > 0);
 			if( !bIsTable && (iY + 3*iYNext) < (getHeight() - getFootnoteHeight() - iBottomMargin))
 			{
-//		   		m_pOwner->markForRebuild();
+		   		m_pOwner->markForRebuild();
 //
 // FIXME see if this code works instead
 //
@@ -1110,13 +1679,23 @@ PT_DocPosition fp_Page::getFirstLastPos(bool bFirst) const
 	if (bFirst)
 	{
 		fp_Column* pColumn = getNthColumnLeader(0);
-		UT_ASSERT(pColumn);
+		UT_return_val_if_fail(pColumn, 2);
 		fp_Container* pFirstContainer = static_cast<fp_Container *>(pColumn->getFirstContainer());
-		UT_ASSERT(pFirstContainer);
-		while(pFirstContainer->getContainerType() != FP_CONTAINER_LINE)
+		while(pFirstContainer && pFirstContainer->getContainerType() != FP_CONTAINER_LINE)
 		{
-			pFirstContainer = static_cast<fp_Container *>(pFirstContainer->getNthCon(0));
+			if(pFirstContainer->getContainerType() == FP_CONTAINER_TABLE)
+			{
+				fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pFirstContainer);
+				pFirstContainer = static_cast<fp_Container *>(pTab->getFirstLineInColumn(pColumn));
+			}
+			else
+			{
+				pFirstContainer = static_cast<fp_Container *>(pFirstContainer->getNthCon(0));
+			}
 		}
+
+		UT_return_val_if_fail(pFirstContainer, 2);
+		
 		fp_Run* pFirstRun = static_cast<fp_Line *>(pFirstContainer)->getFirstRun();
 		fl_BlockLayout* pFirstBlock = static_cast<fp_Line *>(pFirstContainer)->getBlock(); // SEVIOR This needs fix me, FIXME
 
@@ -1125,17 +1704,33 @@ PT_DocPosition fp_Page::getFirstLastPos(bool bFirst) const
 	else
 	{
 		fp_Column* pColumn = getNthColumnLeader(cols-1);
-		UT_ASSERT(pColumn);
+		UT_return_val_if_fail(pColumn, 2);
 		fp_Container* pLastContainer = static_cast<fp_Container *>(pColumn->getLastContainer());
-		UT_ASSERT(pLastContainer);
+		UT_return_val_if_fail(pLastContainer, 2);
+		while(pLastContainer && pLastContainer->getContainerType() != FP_CONTAINER_LINE)
+		{
+			if(pLastContainer->getContainerType() == FP_CONTAINER_TABLE)
+			{
+				fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pLastContainer);
+				pLastContainer = static_cast<fp_Container *>(pTab->getLastLineInColumn(pColumn));
+			}
+			else
+			{
+				pLastContainer = static_cast<fp_Container *>(pLastContainer->getNthCon(0));
+			}
+		}
 
+		UT_return_val_if_fail(pLastContainer, 2);
+		
 		fp_Run* pLastRun = static_cast<fp_Line *>(pLastContainer)->getLastRun();
 		fl_BlockLayout* pLastBlock = static_cast<fp_Line *>(pLastContainer)->getBlock();
+		UT_return_val_if_fail(pLastRun && pLastBlock, 2);
 
-		while (!pLastRun->isFirstRunOnLine() && pLastRun->isForcedBreak())
+		while (pLastRun && !pLastRun->isFirstRunOnLine() && pLastRun->isForcedBreak())
 		{
-			pLastRun = pLastRun->getPrev();
+			pLastRun = pLastRun->getPrevRun();
 		}
+		UT_return_val_if_fail(pLastRun, 2);
 
 		if(pLastRun->isForcedBreak())
 		{
@@ -1150,11 +1745,26 @@ PT_DocPosition fp_Page::getFirstLastPos(bool bFirst) const
 	return pos;
 }
 
+void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow )
+{
+	fl_HdrFtrShadow * pShad = NULL;
+	if(pShadow == NULL)
+	{
+		mapXYToPosition(false,x,y,pos,bBOL,bEOL,isTOC, bUseHdrFtr, NULL);
+		return;
+	}
+	else
+	{
+		mapXYToPosition(false,x,y,pos,bBOL,bEOL,isTOC, bUseHdrFtr, &pShad);
+	}
+	*pShadow = pShad;
+}
 /*!
  * This method maps an x,y location on the page to the position in the
  * document of the corrsponding element.
  * This variation looks in the header/footer region and returns the
  * SectionLayout shadow of the
+ \param bNotFrames if true don't look inside frames
  \param x coordinate
  \param y coordinate
  \param bBOL
@@ -1162,7 +1772,7 @@ PT_DocPosition fp_Page::getFirstLastPos(bool bFirst) const
  \return pos The Document position corresponding the text at location x,y
  \return pShadow A pointer to the shadow corresponding to this header/footer
  */
-void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow)
+void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool &isTOC, bool bUseHdrFtr, fl_HdrFtrShadow ** pShadow )
 {
 	int count = m_vecColumnLeaders.getItemCount();
 	UT_uint32 iMinDist = 0xffffffff;
@@ -1173,6 +1783,81 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 	UT_uint32 iDist = 0;
 	fp_Column* pLeader = NULL;
 //
+// Start by looking in Frames for this point.
+//
+	UT_sint32 i =0;
+	fp_FrameContainer * pFrameC = NULL;
+	if(!bNotFrames)
+	{
+//
+// The iextra distance gives the space around the text box frame inside the
+// frame where the user can select the frame rather than the text inside the
+// frame. Without this distance clicking inside a text box would always place 
+// the caret in text rather than selecting the frame to drag/resize etc.
+//
+		UT_sint32 iextra = m_pLayout->getGraphics()->tlu(4);
+		// loop from high z to low z
+		// Because we draw from old to new, the new appears on top
+		// and because the new appears on top, it should be treated as such
+		for (i=(static_cast<UT_sint32>(countFrameContainers()-1)); i>=0; i--)
+		{
+			pFrameC = getNthFrameContainer(i);
+			bool isImage = false;
+			fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFrameC->getSectionLayout());
+			if(pFL->getFrameType() >= FL_FRAME_WRAPPER_IMAGE)
+			{
+				isImage = true;
+			}
+			if ((pFrameC->getFirstContainer()) || isImage )
+			{
+				if ((x >= (pFrameC->getFullX()- iextra))
+					&& (x < (pFrameC->getFullX() + pFrameC->getFullWidth()+iextra))
+					&& (y >= (pFrameC->getFullY() - iextra))
+					&& (y < (pFrameC->getFullY() + pFrameC->getFullHeight() + iextra))
+					)
+				{
+					if(isImage)
+					{
+						pos = pFL->getPosition(true);
+						return;
+					}
+					pFrameC->mapXYToPosition(x - pFrameC->getX(), y - pFrameC->getY(), pos, bBOL, bEOL,isTOC);
+					return;
+				}
+				
+				iDist = pFrameC->distanceFromPoint(x, y);
+//
+// The tlu(3) makes the distance of the mouse to the sensitive edge of the
+// text box 3 pixels. ie Move the mouse within 3 pixels of the textbox and it
+// change to show you can select the text box.
+//
+// If we're outside this distance make sure all other options are excluded
+// before placing hte point inside th text box
+//
+
+				if(static_cast<UT_sint32>(iDist) > m_pLayout->getGraphics()->tlu(3))
+				{
+					iDist += 200000;
+				}
+				if (iDist < iMinDist)
+				{
+					iMinDist = iDist;
+					pMinDist = static_cast<fp_VerticalContainer *>(pFrameC);
+				}
+				
+				if ( (y >= pFrameC->getY())
+					 && (y < (pFrameC->getY() + pFrameC->getHeight()))) 
+				{
+					if (iDist < iMinXDist)
+					{
+						iMinXDist = iDist;
+						pMinXDist = static_cast<fp_VerticalContainer *>(pFrameC);
+					}
+				}
+			}
+		}
+	}
+//
 // Look in header for insertion point
 //
 	if (bUseHdrFtr)
@@ -1189,12 +1874,9 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 				if(p == NULL || !p->getFirstContainer())
 					continue;
 
-				if ((x >= p->getX())
-					&& (x < (p->getX() + p->getWidth()))
-					&& (y >= p->getY())
-					&& (y < (p->getY() + p->getHeight())))
+				if ((y >= p->getY()) && (y < (p->getY() + p->getHeight())))
 				{
-					p->mapXYToPosition(x - p->getX(), y - p->getY(), pos, bBOL, bEOL);
+					p->mapXYToPosition(x - p->getX(), y - p->getY(), pos, bBOL, bEOL,isTOC);
 					if (pShadow)
 						*pShadow = p->getShadow();
 					return;
@@ -1206,10 +1888,9 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 //
 // Now look in page
 //
-	UT_sint32 i =0;
 	for (i=0; i<count; i++)
 	{
-		pLeader = static_cast<fp_Column*>(m_vecColumnLeaders.getNthItem(i));
+		pLeader = m_vecColumnLeaders.getNthItem(i);
 
 		pColumn = pLeader;
 		iMinXDist = 0xffffffff;
@@ -1225,7 +1906,7 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 					&& (y < (pColumn->getY() + pColumn->getHeight()))
 					)
 				{
-					pColumn->mapXYToPosition(x - pColumn->getX(), y - pColumn->getY(), pos, bBOL, bEOL);
+					pColumn->mapXYToPosition(x - pColumn->getX(), y - pColumn->getY(), pos, bBOL, bEOL,isTOC);
 					return;
 				}
 
@@ -1268,7 +1949,7 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 				&& (y < (pFC->getY() + pFC->getHeight()))
 				)
 			{
-				pFC->mapXYToPosition(x - pFC->getX(), y - pFC->getY(), pos, bBOL, bEOL);
+				pFC->mapXYToPosition(x - pFC->getX(), y - pFC->getY(), pos, bBOL, bEOL,isTOC);
 					return;
 			}
 
@@ -1292,13 +1973,13 @@ void fp_Page::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, boo
 	}
 	if (pMinXDist)
 	{
-		pMinXDist->mapXYToPosition(x - pMinXDist->getX(), y - pMinXDist->getY(), pos, bBOL, bEOL);
+		pMinXDist->mapXYToPosition(x - pMinXDist->getX(), y - pMinXDist->getY(), pos, bBOL, bEOL,isTOC);
 		return;
 	}
 
 	UT_ASSERT(pMinDist);
 
-	pMinDist->mapXYToPosition(x - pMinDist->getX(), y - pMinDist->getY(), pos, bBOL, bEOL);
+	pMinDist->mapXYToPosition(x - pMinDist->getX(), y - pMinDist->getY(), pos, bBOL, bEOL,isTOC);
 }
 
 void fp_Page::setView(FV_View* pView)
@@ -1412,6 +2093,128 @@ fp_ShadowContainer* fp_Page::getHdrFtrContainer(fl_HdrFtrSectionLayout* pHFSL)
 	}
 }
 
+// Frame methods
+
+void fp_Page::frameHeightChanged(void)
+{
+}
+
+void fp_Page::clearScreenFrames(void)
+{
+	UT_sint32 i =0;
+	for (i = 0; i < static_cast<UT_sint32>(countFrameContainers()); i++)
+	{
+		getNthFrameContainer(i)->clearScreen();
+	}
+}
+
+void fp_Page::markDirtyOverlappingRuns(fp_FrameContainer * pFrameC)
+{
+	UT_Rect * pMyFrameRect = pFrameC->getScreenRect();
+	if(pMyFrameRect == NULL)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+	// check each column for redraw
+
+	UT_sint32 count = m_vecColumnLeaders.getItemCount();
+	UT_sint32 i = 0;
+	for (i=0; i<count; i++)
+	{
+		fp_Column* pCol = m_vecColumnLeaders.getNthItem(i);
+		while (pCol)
+		{
+			pCol->markDirtyOverlappingRuns(*pMyFrameRect);
+			pCol = pCol->getFollower();
+		}
+	}
+
+	// Now do the headers and footers
+
+	if (m_pHeader)
+	{
+		m_pHeader->markDirtyOverlappingRuns(*pMyFrameRect);
+	}
+
+	if (m_pFooter)
+	{
+		m_pFooter->markDirtyOverlappingRuns(*pMyFrameRect);
+	}
+
+	// Now Footnotes
+
+	count = m_vecFootnotes.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FootnoteContainer* pFC = m_vecFootnotes.getNthItem(i);
+		pFC->markDirtyOverlappingRuns(*pMyFrameRect);
+	}
+
+	// Now Frames
+
+	count = m_vecFrames.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecFrames.getNthItem(i);
+		if(pFC != pFrameC)
+		{
+			pFC->markDirtyOverlappingRuns(*pMyFrameRect);
+		}
+	}
+	delete pMyFrameRect;
+}
+
+UT_uint32 fp_Page::countFrameContainers(void) const
+{
+	return m_vecFrames.getItemCount();
+}
+
+UT_sint32 fp_Page::findFrameContainer(fp_FrameContainer * pFC)
+{
+	UT_sint32 i = m_vecFrames.findItem(pFC);
+	return i;
+}
+
+fp_FrameContainer* fp_Page::getNthFrameContainer(UT_sint32 n) const 
+{
+	return m_vecFrames.getNthItem(n);
+} 
+
+bool fp_Page::insertFrameContainer(fp_FrameContainer * pFC)
+{
+	m_vecFrames.addItem(pFC);
+	if(pFC)
+	{
+		pFC->setPage(this);
+	}
+	_reformat();
+	return true;
+}
+
+void fp_Page::removeFrameContainer(fp_FrameContainer * pFC)
+{
+
+	markDirtyOverlappingRuns(pFC);
+	UT_sint32 ndx = m_vecFrames.findItem(pFC);
+	if(ndx>=0)
+	{
+		m_vecFrames.deleteNthItem(ndx);
+		for(ndx=0; ndx < static_cast<UT_sint32>(countFrameContainers());ndx++)
+		{			
+			fp_FrameContainer * pFC = getNthFrameContainer(ndx);
+			fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pFC->getSectionLayout());
+			pFC->clearScreen();
+			pFL->markAllRunsDirty();
+		}
+		_reformat();
+		return;
+	}
+	return;
+}
+
+// Footnote methods
+
 UT_uint32 fp_Page::countFootnoteContainers(void) const
 {
 	return m_vecFootnotes.getItemCount();
@@ -1419,24 +2222,29 @@ UT_uint32 fp_Page::countFootnoteContainers(void) const
 
 UT_sint32 fp_Page::findFootnoteContainer(fp_FootnoteContainer * pFC)
 {
-	UT_sint32 i = m_vecFootnotes.findItem(static_cast<void *>(pFC));
+	UT_sint32 i = m_vecFootnotes.findItem(pFC);
 	return i;
 }
 
 fp_FootnoteContainer* fp_Page::getNthFootnoteContainer(UT_sint32 n) const 
 {
-	return static_cast<fp_FootnoteContainer*>(m_vecFootnotes.getNthItem(n));
+	return m_vecFootnotes.getNthItem(n);
 } 
 
 bool fp_Page::insertFootnoteContainer(fp_FootnoteContainer * pFC)
 {
 	UT_uint32 i =0;
+	UT_sint32 j = findFootnoteContainer(pFC);
+	if(j >= 0)
+	{
+		return false;
+	}
 	UT_uint32 loc =0;
 	UT_sint32 fVal = pFC->getValue();
 	fp_FootnoteContainer * pFTemp = NULL;
 	for(i=0; i< m_vecFootnotes.getItemCount();i++)
 	{
-		pFTemp = static_cast<fp_FootnoteContainer *>(m_vecFootnotes.getNthItem(i));
+		pFTemp = m_vecFootnotes.getNthItem(i);
 		if(fVal < pFTemp->getValue())
 		{
 			loc = i;
@@ -1445,11 +2253,11 @@ bool fp_Page::insertFootnoteContainer(fp_FootnoteContainer * pFC)
 	}
 	if(pFTemp == NULL)
 	{
-		m_vecFootnotes.addItem(static_cast<void *>(pFC));
+		m_vecFootnotes.addItem(pFC);
 	}
 	else if( i>= m_vecFootnotes.getItemCount())
 	{
-		m_vecFootnotes.addItem(static_cast<void *>(pFC));
+		m_vecFootnotes.addItem(pFC);
 	}
 	else
 	{
