@@ -895,7 +895,8 @@ bool fp_TextRun::canMergeWithNext(void)
 	if (!getNextRun() ||
 		!getLine() ||
 		getNextRun()->getType() != FPRUN_TEXT ||
-		!getNextRun()->getLine())
+		!getNextRun()->getLine() ||
+		getLength()+ getNextRun()->getLength() > 32000) // sanity check, see bug 8542
 	{
 		return false;
 	}
@@ -914,11 +915,7 @@ bool fp_TextRun::canMergeWithNext(void)
 		|| (pNext->_getColorHL() != _getColorHL())
 		|| (pNext->_getColorHL().isTransparent() != _getColorHL().isTransparent())
 		|| (pNext->m_fPosition != m_fPosition)
-#if 0
 		|| (pNext->getVisDirection() != getVisDirection())
-#else
-		|| (pNext->getDirection() != getDirection())
-#endif
 		// we also want to test the override, because we do not want runs that have the same
 		// visual direction but different override merged
 		|| (pNext->m_iDirOverride != m_iDirOverride)
@@ -948,18 +945,10 @@ bool fp_TextRun::canMergeWithNext(void)
 		return false;
 	}
 	
-#if 0
 //
 // Don't coalese past word boundaries
 // This improves lots of flicker issues
 //
-	// I see; actually we went into lot (and I mean _lot_) of trouble to coalesce pass
-	// direction boundary changes (and that includes spaces) in order to reduce the number
-	// of text runs in the document. Clearly, this is not desirable, so I have made the
-	// necessary changes to the code above (test for direction rather than visual
-	// direction) and disabled the code in breakMeAtDirectionBoundaries() and
-	// breakNeigboursAtDirectionBoundaries(). This achieves the same as this code this but
-	// in much more efficient way.
 	PD_StruxIterator text(getBlock()->getStruxDocHandle(),
 						  getBlockOffset() + fl_BLOCK_STRUX_OFFSET);
 	text.setPosition(getLength()-1);
@@ -967,8 +956,8 @@ bool fp_TextRun::canMergeWithNext(void)
 	{
 		return false;
 	}
-#endif
-    return true;
+
+	return true;
 }
 
 void fp_TextRun::mergeWithNext(void)
@@ -1471,8 +1460,23 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	xxx_UT_DEBUGMSG(("fp_TextRun::_draw (0x%x): m_iVisDirection %d, _getDirection() %d\n",
 					 this, m_iVisDirection, _getDirection()));
 
+#if 0
+	// BAD, BAD HACK, please do not re-enable this.
+	//
+	// This causes bad pixed dirt with characters that fill the entire glyph box. We
+	// really cannot, under any circumstances, adjust the y coords -- the y coordinance is
+	// the base coordinace of the text, from which everything else derives (if we adust
+	// it, it means that the text gets drawn at wrong position !).
+	//
+	// Also, the comment that accompanies this hack makes no sense: how does adjusting y
+	// coordinance remove final character dirt?
+	// Tomas, Christmas Eve, 2004 (I know, I should get life)
 	UT_sint32 yTopOfRun = pDA->yoff - getAscent() - pG->tlu(1); // Hack to remove
 	UT_sint32 yTopOfSel = yTopOfRun + pG->tlu(1); // final character dirt
+#else
+	UT_sint32 yTopOfRun = pDA->yoff - getAscent();
+	UT_sint32 yTopOfSel = yTopOfRun;
+#endif
 	xxx_UT_DEBUGMSG(("_draw Text: yoff %d \n",pDA->yoff));
 	xxx_UT_DEBUGMSG(("_draw Text: getAscent %d fontAscent-1 %d fontAscent-2 %d \n",getAscent(),pG->getFontAscent(_getFont()),pG->getFontAscent()));
 	/*
@@ -2138,7 +2142,9 @@ void fp_TextRun::_drawFirstChar(bool bSelection)
 
 void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
 {
-	UT_sint32       iWidth = 0;
+	bool bRTL = getVisDirection() == UT_BIDI_RTL;
+	
+	UT_sint32       iWidth =  bRTL ? getWidth() : 0;
 	UT_uint32       iLen = getLength();
 	UT_sint32       iLineWidth = 1+ (UT_MAX(10,getAscent())-10)/8;
 	UT_sint32       iRectSize = iLineWidth * 3 / 2;
@@ -2161,11 +2167,21 @@ void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
 		
 		if(text.getChar() == UCS_SPACE)
 		{
-			painter.fillRect(pView->getColorShowPara(), xoff + iWidth + (iCharWidth - iRectSize) / 2,iY,iRectSize,iRectSize);
+			UT_uint32 x;
+			if(bRTL)
+				x = xoff + iWidth - (iCharWidth + iRectSize)/2;
+			else
+				x = xoff + iWidth + (iCharWidth - iRectSize)/2;
+			
+			painter.fillRect(pView->getColorShowPara(), x, iY, iRectSize, iRectSize);
 		}
 		UT_uint32 iCW = iCharWidth > 0 && iCharWidth < GR_OC_MAX_WIDTH ? iCharWidth : 0;
+
+		if(bRTL)
+			iWidth -= iCW;
+		else
+			iWidth += iCW;
 		
-		iWidth += iCW;
 		++iWidthOffset;
 	}
 }
@@ -2225,6 +2241,11 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 		points[nPoints-1].y = top + getGraphics()->tlu(1);
 	}
 
+	getGraphics()->setLineProperties(getGraphics()->tluD(1.0),
+									 GR_Graphics::JOIN_MITER,
+									 GR_Graphics::CAP_PROJECTING,
+									 GR_Graphics::LINE_SOLID);
+	
 	painter.polyLine(points, nPoints);
 
 	if (points != scratchpoints) delete[] points;
@@ -2258,7 +2279,7 @@ void fp_TextRun::drawSquiggle(UT_uint32 iOffset, UT_uint32 iLen)
 	UT_Rect r;
 	_getPartRect( &r, xoff, yoff, iOffset, iLen);
 
-	_drawSquiggle(r.top + iAscent + iGap, r.left, r.left + r.width);
+	_drawSquiggle(r.top + iAscent + iGap + getGraphics()->tlu(1), r.left, r.left + r.width);
 	xxx_UT_DEBUGMSG(("Done draw sqiggle for run in block %x \n",getBlock()));
 }
 
@@ -2535,10 +2556,33 @@ void fp_TextRun::setDirection(UT_BidiCharType dir, UT_BidiCharType dirOverride)
 		// only do this once
 		if(_getDirection() == UT_BIDI_UNSET)
 		{
-			UT_UCSChar firstChar;
-			getCharacter(0, firstChar);
+			// here we used to check the first character; we can no longer do that,
+			// because the latest versions of USP create items that are not homogenous and
+			// can contain strong chars prefixed by weak chars. So if the first char is
+			// not strong, we try the rest of the run to see if it might contain any
+			// strong chars
+			PD_StruxIterator text(getBlock()->getStruxDocHandle(),
+						  getBlockOffset() + fl_BLOCK_STRUX_OFFSET);
 
-			_setDirection(UT_bidiGetCharType(firstChar));
+			text.setUpperLimit(text.getPosition() + getLength() - 1);
+
+			UT_ASSERT_HARMLESS( text.getStatus() == UTIter_OK );
+			
+			UT_BidiCharType t = UT_BIDI_UNSET;
+			
+			while(text.getStatus() == UTIter_OK)
+			{
+				UT_UCS4Char c = text.getChar();
+
+				t = UT_bidiGetCharType(c);
+
+				if(UT_BIDI_IS_STRONG(t))
+					break;
+				
+				++text;
+			}
+			
+			_setDirection(t);
 		}
 	}
 	else //meaningfull value received
@@ -2657,8 +2701,6 @@ void fp_TextRun::setDirOverride(UT_BidiCharType dir)
 */
 void fp_TextRun::breakNeighborsAtDirBoundaries()
 {
-#if 0
-	// we no longer coalsce across dir boundaries, so this is not needed
 #ifndef NO_BIDI_SUPPORT
 	UT_BidiCharType iPrevType, iType = UT_BIDI_UNSET;
 	UT_BidiCharType iDirection = getDirection();
@@ -2794,13 +2836,10 @@ void fp_TextRun::breakNeighborsAtDirBoundaries()
 
 	}
 #endif
-#endif
 }
 
 void fp_TextRun::breakMeAtDirBoundaries(UT_BidiCharType iNewOverride)
 {
-#if 0
-	// we no longer coalesce past dir boundaries, so this is not needed
 #ifndef NO_BIDI_SUPPORT
 	// we cannot use the draw buffer here because in case of ligatures it might
 	// contain characters of misleading directional properties
@@ -2860,7 +2899,6 @@ void fp_TextRun::breakMeAtDirBoundaries(UT_BidiCharType iNewOverride)
 		iPrevType = iType;
 	}
 #endif
-#endif
 }
 
 
