@@ -55,6 +55,50 @@ bool pt_PieceTable::changeSectionAttsNoUpdate(pf_Frag_Strux * pfs,
 	return _realChangeSectionAttsNoUpdate(pfs, atts, attsValue);
 }
 
+
+/*!
+ * This method sends out change records to the layouts but it does put
+ * revision marks on them nor are they saved in the undo stack.
+ * It's used by the strux resizer for the hdrftr and maybe later for the frame.
+ */
+bool pt_PieceTable::changeStruxFmtNoUndo(PTChangeFmt ptc,
+										 pf_Frag_Strux * pfs,
+										 const XML_Char ** attributes,
+										 const XML_Char ** properties)
+{
+	PT_AttrPropIndex indexNewAP;
+	PTStruxType pts = pfs->getStruxType();
+
+	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
+	bool bMerged;
+	bMerged = m_varset.mergeAP(ptc,indexOldAP,attributes,properties,&indexNewAP,getDocument());
+	UT_ASSERT_HARMLESS(bMerged);
+	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
+	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
+		return true;
+
+	// convert this fragStrux into a doc position.  we add the length
+	// of the strux (in doc position coords) so that when undo looks
+	// it up by position it will be to the right of the beginning of
+	// the fragment and will find us -- rather than finding the end of
+	// the previous fragment.
+
+	PT_DocPosition dpos = getFragPosition(pfs) + pfs->getLength();
+
+	PX_ChangeRecord_StruxChange * pcr
+		= new PX_ChangeRecord_StruxChange(PX_ChangeRecord::PXT_ChangeStrux,
+										  dpos,
+										  indexOldAP,indexNewAP,pts);
+	UT_return_val_if_fail (pcr,false);
+
+	bool bResult;
+	bResult = _fmtChangeStrux(pfs,indexNewAP);
+	UT_return_val_if_fail (bResult,false);
+	m_pDocument->notifyListeners(pfs,pcr);
+
+	return true;
+}
+
 bool pt_PieceTable::changeStruxFmt(PTChangeFmt ptc,
 									  PT_DocPosition dpos1,
 									  PT_DocPosition dpos2,
@@ -62,15 +106,20 @@ bool pt_PieceTable::changeStruxFmt(PTChangeFmt ptc,
 									  const XML_Char ** properties,
 								   PTStruxType pts)
 {
+	bool bDoAll = (pts == PTX_StruxDummy);
 	if(m_pDocument->isMarkRevisions())
 	{
 		pf_Frag_Strux * pfs_First;
 		pf_Frag_Strux * pfs_End;
-
-		if(!_getStruxOfTypeFromPosition(dpos1,pts,&pfs_First))
+		PTStruxType ptsTemp = pts;
+		if(pts==PTX_StruxDummy)
+		{
+			ptsTemp = PTX_Block;
+		}
+		if(!_getStruxOfTypeFromPosition(dpos1,ptsTemp,&pfs_First))
 			return false;
 
-		if(!_getStruxOfTypeFromPosition(dpos2,pts,&pfs_End))
+		if(!_getStruxOfTypeFromPosition(dpos2,ptsTemp,&pfs_End))
 			return false;
 
 		// see if the change is exactly one block.  if so, we have
@@ -90,13 +139,13 @@ bool pt_PieceTable::changeStruxFmt(PTChangeFmt ptc,
 			{
 				case pf_Frag::PFT_EndOfDoc:
 				default:
-					UT_ASSERT(0);
+					UT_ASSERT_HARMLESS(0);
 					return false;
 
 				case pf_Frag::PFT_Strux:
 					{
 						pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
-						if (pfs->getStruxType() == pts)
+						if (bDoAll || (pfs->getStruxType() == pts))
 						{
 							bool bResult;
 							// get attributes for this fragement
@@ -109,17 +158,44 @@ bool pt_PieceTable::changeStruxFmt(PTChangeFmt ptc,
 								pAP->getAttribute(name, pRevision);
 							}
 
+							// if the request is for removal of fmt, in the revision mode, we still
+							// have to add these props (the removal is indicated by their emptiness)
+							// as we cannot rely on callers to set these correctly, we have to emtpy
+							// them ourselves
+							PTChangeFmt revPtc = ptc;
 							PP_RevisionAttr Revisions(pRevision);
-							Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_FMT_CHANGE,attributes,properties);
+							const XML_Char ** attrs = attributes;
+							const XML_Char ** props = properties;
+							
+							if(ptc == PTC_RemoveFmt)
+							{
+								revPtc = PTC_AddFmt;
 
+								// used to set these to NULL, but that causes difficulties
+								// for attributes, because the attribute value gets stored
+								// directly in the hash and the hash considers NULL values
+								// invalid, so we are not able to retrieve them (and any
+								// associated names
+								attrs = UT_setPropsToValue(attributes, "-/-");
+								props = UT_setPropsToValue(properties, "-/-");
+
+							}
+							
+							Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_FMT_CHANGE,
+												  attrs,props);
+
+							if(attrs != attributes)
+								delete[] attrs;
+							
+							if(props != properties)
+								delete[] props;
+							
 							const XML_Char * ppRevAttrib[3];
 							ppRevAttrib[0] = name;
 							ppRevAttrib[1] = Revisions.getXMLstring();
 							ppRevAttrib[2] = NULL;
-
-
-							bResult = _fmtChangeStruxWithNotify(ptc,pfs,ppRevAttrib,NULL);
-							UT_ASSERT(bResult);
+							bResult = _fmtChangeStruxWithNotify(revPtc,pfs,ppRevAttrib,NULL,false);
+							UT_return_val_if_fail (bResult,false);
 						}
 						if (pfs == pfs_End)
 							bFinished = true;
@@ -143,6 +219,25 @@ bool pt_PieceTable::changeStruxFmt(PTChangeFmt ptc,
 		return _realChangeStruxFmt(ptc, dpos1, dpos2, attributes, properties, pts);
 }
 
+bool pt_PieceTable::changeStruxFormatNoUpdate(PTChangeFmt ptc ,pf_Frag_Strux * pfs, const XML_Char ** attributes)
+{
+	PT_AttrPropIndex indexNewAP;
+	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
+	bool bMerged;
+	bMerged = m_varset.mergeAP(ptc,indexOldAP,attributes,NULL,&indexNewAP,getDocument());
+	UT_ASSERT_HARMLESS(bMerged);
+	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
+	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
+		return true;
+
+	bool bResult;
+	bResult = _fmtChangeStrux(pfs,indexNewAP);
+	UT_return_val_if_fail (bResult,false);
+
+	return true;
+
+}
+
 bool pt_PieceTable::_fmtChangeStrux(pf_Frag_Strux * pfs,
 									   PT_AttrPropIndex indexNewAP)
 {
@@ -156,10 +251,12 @@ bool pt_PieceTable::_fmtChangeStruxWithNotify(PTChangeFmt ptc,
 												 const XML_Char ** properties)
 {
 	PT_AttrPropIndex indexNewAP;
+	PTStruxType pts = pfs->getStruxType();
+
 	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
 	bool bMerged;
 	bMerged = m_varset.mergeAP(ptc,indexOldAP,attributes,properties,&indexNewAP,getDocument());
-	UT_ASSERT(bMerged);
+	UT_ASSERT_HARMLESS(bMerged);
 	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
 	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
 		return true;
@@ -175,12 +272,12 @@ bool pt_PieceTable::_fmtChangeStruxWithNotify(PTChangeFmt ptc,
 	PX_ChangeRecord_StruxChange * pcr
 		= new PX_ChangeRecord_StruxChange(PX_ChangeRecord::PXT_ChangeStrux,
 										  dpos,
-										  indexOldAP,indexNewAP);
-	UT_ASSERT(pcr);
+										  indexOldAP,indexNewAP,pts);
+	UT_return_val_if_fail (pcr,false);
 
 	bool bResult;
 	bResult = _fmtChangeStrux(pfs,indexNewAP);
-	UT_ASSERT(bResult);
+	UT_return_val_if_fail (bResult,false);
 
 	// add record to history.  we do not attempt to coalesce these.
 	m_history.addChangeRecord(pcr);
@@ -190,19 +287,22 @@ bool pt_PieceTable::_fmtChangeStruxWithNotify(PTChangeFmt ptc,
 }
 
 /*!
- * This Method implements the change strux we need to reparent lists.
+ * Don't broadcast changes to the endstruxs because will cause them to destroy
+ * the properties of the layout since they linked to the layout.
  */
-bool pt_PieceTable::_realChangeStruxForLists(PL_StruxDocHandle sdh,
-									   const char * pszParentID)
+bool pt_PieceTable::_fmtChangeStruxWithNotify(PTChangeFmt ptc,
+												 pf_Frag_Strux * pfs,
+												 const XML_Char ** attributes,
+												 const XML_Char ** properties,
+											  bool bDoAll)
 {
-	pf_Frag_Strux * pfs = (pf_Frag_Strux *) sdh;
-	const char * attributes[3] = {PT_PARENTID_ATTRIBUTE_NAME,pszParentID,NULL};
-
 	PT_AttrPropIndex indexNewAP;
+	PTStruxType pts = pfs->getStruxType();
+   
 	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
 	bool bMerged;
-	bMerged = m_varset.mergeAP( PTC_AddFmt ,indexOldAP,attributes,NULL,&indexNewAP,getDocument());
-	UT_ASSERT(bMerged);
+	bMerged = m_varset.mergeAP(ptc,indexOldAP,attributes,properties,&indexNewAP,getDocument());
+	UT_ASSERT_HARMLESS(bMerged);
 	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
 	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
 		return true;
@@ -218,12 +318,62 @@ bool pt_PieceTable::_realChangeStruxForLists(PL_StruxDocHandle sdh,
 	PX_ChangeRecord_StruxChange * pcr
 		= new PX_ChangeRecord_StruxChange(PX_ChangeRecord::PXT_ChangeStrux,
 										  dpos,
-										  indexOldAP,indexNewAP);
-	UT_ASSERT(pcr);
+										  indexOldAP,indexNewAP,pts);
+	UT_return_val_if_fail (pcr,false);
 
 	bool bResult;
 	bResult = _fmtChangeStrux(pfs,indexNewAP);
-	UT_ASSERT(bResult);
+	UT_return_val_if_fail (bResult,false);
+
+	// add record to history.  we do not attempt to coalesce these.
+	m_history.addChangeRecord(pcr);
+	if(bDoAll || ( (pts != PTX_EndTable) && (pts != PTX_EndCell)
+	   && (pts != PTX_EndFrame) && (pts != PTX_EndTOC) 
+	   && (pts != PTX_EndFootnote) && (pts != PTX_EndEndnote)))
+	{ 
+		m_pDocument->notifyListeners(pfs,pcr);
+	}
+	return true;
+}
+
+
+/*!
+ * This Method implements the change strux we need to reparent lists.
+ */
+bool pt_PieceTable::_realChangeStruxForLists(PL_StruxDocHandle sdh,
+									   const char * pszParentID)
+{
+	pf_Frag_Strux * pfs = (pf_Frag_Strux *) sdh;
+	PTStruxType pts = pfs->getStruxType();
+
+	const char * attributes[3] = {PT_PARENTID_ATTRIBUTE_NAME,pszParentID,NULL};
+
+	PT_AttrPropIndex indexNewAP;
+	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
+	bool bMerged;
+	bMerged = m_varset.mergeAP( PTC_AddFmt ,indexOldAP,attributes,NULL,&indexNewAP,getDocument());
+	UT_ASSERT_HARMLESS(bMerged);
+	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
+	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
+		return true;
+
+	// convert this fragStrux into a doc position.  we add the length
+	// of the strux (in doc position coords) so that when undo looks
+	// it up by position it will be to the right of the beginning of
+	// the fragment and will find us -- rather than finding the end of
+	// the previous fragment.
+
+	PT_DocPosition dpos = getFragPosition(pfs) + pfs->getLength();
+
+	PX_ChangeRecord_StruxChange * pcr
+		= new PX_ChangeRecord_StruxChange(PX_ChangeRecord::PXT_ChangeStrux,
+										  dpos,
+										  indexOldAP,indexNewAP,pts);
+	UT_return_val_if_fail (pcr,false);
+
+	bool bResult;
+	bResult = _fmtChangeStrux(pfs,indexNewAP);
+	UT_return_val_if_fail (bResult, false);
 
 	// add record to history.  So we can undo it later.
 
@@ -247,14 +397,14 @@ bool pt_PieceTable::_realChangeSectionAttsNoUpdate(pf_Frag_Strux * pfs,
 	PT_AttrPropIndex indexOldAP = pfs->getIndexAP();
 	bool bMerged;
 	bMerged = m_varset.mergeAP( PTC_AddFmt ,indexOldAP,attributes,NULL,&indexNewAP,getDocument());
-	UT_ASSERT(bMerged);
+	UT_ASSERT_HARMLESS(bMerged);
 	xxx_UT_DEBUGMSG(("Merging atts/props oldindex=%d , newindex =%d \n",indexOldAP,indexNewAP));
 	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
 		return true;
 
 	bool bResult;
 	bResult = _fmtChangeStrux(pfs,indexNewAP);
-	UT_ASSERT(bResult);
+	UT_return_val_if_fail (bResult,false);
 
 	return true;
 
@@ -267,15 +417,15 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 									  const XML_Char ** properties,
 									  PTStruxType pts)
 {
-	UT_ASSERT(m_pts==PTS_Editing);
-
+	UT_return_val_if_fail (m_pts==PTS_Editing,false);
+	bool bDoAll = (pts == PTX_StruxDummy);
 	// apply a strux-level formatting change to the given region.
 
-	UT_ASSERT(dpos1 <= dpos2);
+	UT_return_val_if_fail (dpos1 <= dpos2, false);
 	bool bHaveAttributes, bHaveProperties;
 	bHaveAttributes = (attributes && *attributes);
 	bHaveProperties = (properties && *properties);
-	UT_ASSERT(bHaveAttributes || bHaveProperties); // must have something to do
+	UT_return_val_if_fail (bHaveAttributes || bHaveProperties,false); // must have something to do
 
 	pf_Frag_Strux * pfs_First;
 	pf_Frag_Strux * pfs_End;
@@ -284,14 +434,29 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 	// type for both end points of the change.
 
 	bool bFoundFirst;
-	bFoundFirst = _getStruxOfTypeFromPosition(dpos1,pts,&pfs_First);
+	PTStruxType ptsTemp = pts;
+	if(bDoAll)
+	{
+		ptsTemp = PTX_Block;
+	}
+	bFoundFirst = _getStruxOfTypeFromPosition(dpos1,ptsTemp,&pfs_First);
 	bool bFoundEnd;
-	bFoundEnd = _getStruxOfTypeFromPosition(dpos2,pts,&pfs_End);
-	UT_ASSERT(bFoundFirst && bFoundEnd);
-	while(pfs_End->getPos() < pfs_First->getPos() && (dpos2 >= dpos1))
+	bFoundEnd = _getStruxOfTypeFromPosition(dpos2,ptsTemp,&pfs_End);
+	if(!(bFoundFirst && bFoundEnd))
+	{
+		UT_DEBUGMSG((" could not find bFoundFirst %d or Maybe bFoundEnd %d \n",
+					 bFoundFirst,bFoundEnd));
+		UT_DEBUGMSG(("Aborting attempted change. \n"));
+		return false;
+	}
+	while(pfs_End && (pfs_End->getPos() < pfs_First->getPos() && (dpos2 >= dpos1)))
 	{
 		dpos2--;
-		bFoundEnd = _getStruxOfTypeFromPosition(dpos2,pts,&pfs_End);
+		bFoundEnd = _getStruxOfTypeFromPosition(dpos2,ptsTemp,&pfs_End);
+	}
+	if(!pfs_End)
+	{
+		return false;
 	}
 	// see if the change is exactly one block.  if so, we have
 	// a simple change.  otherwise, we have a multistep change.
@@ -317,17 +482,17 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 			{
 			case pf_Frag::PFT_EndOfDoc:
 			default:
-				UT_ASSERT(0);
+				UT_ASSERT_HARMLESS(0);
 				return false;
 
 			case pf_Frag::PFT_Strux:
 				{
 					pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
-					if (pfs->getStruxType() == pts)
+					if (bDoAll || (pfs->getStruxType() == pts))
 					{
 						bool bResult;
-						bResult = _fmtChangeStruxWithNotify(ptc,pfs,attributes,properties);
-						UT_ASSERT(bResult);
+						bResult = _fmtChangeStruxWithNotify(ptc,pfs,attributes,properties,bDoAll);
+						UT_return_val_if_fail (bResult,false);
 					}
 					if (pfs == pfs_End)
 						bFinished = true;
@@ -372,7 +537,7 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 		PD_Style * pStyle = NULL;
 		PTChangeFmt ptcs = PTC_RemoveFmt;
 		getDocument()->getStyle(szStyle,&pStyle);
-		UT_ASSERT(pStyle);
+		UT_return_val_if_fail (pStyle,false);
 		UT_Vector vProps;
 //
 // Get the vector of properties
@@ -406,12 +571,12 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 			switch (pf->getType())
 			{
 			case pf_Frag::PFT_EndOfDoc:
-				UT_ASSERT(bEndSeen);
+				UT_ASSERT_HARMLESS(bEndSeen);
 				bFinished = true;
 				break;
 
 			default:
-				UT_ASSERT(0);
+				UT_ASSERT_HARMLESS(0);
 				return false;
 
 			case pf_Frag::PFT_Strux:
@@ -419,11 +584,11 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 					pfNewEnd = pf->getNext();
 					fragOffsetNewEnd = 0;
 					pfsContainer = static_cast<pf_Frag_Strux *> (pf);
-					if (!bEndSeen && (pfsContainer->getStruxType() == pts))
+					if (!bEndSeen && (bDoAll || (pfsContainer->getStruxType() == pts)))
 					{
 						bool bResult;
-						bResult = _fmtChangeStruxWithNotify(ptc,pfsContainer,attributes,sProps);
-						UT_ASSERT(bResult);
+						bResult = _fmtChangeStruxWithNotify(ptc,pfsContainer,attributes,sProps,false);
+						UT_return_val_if_fail (bResult,false);
 					}
 					if(!bEndSeen && isEndFootnote(static_cast<pf_Frag *>(pfsContainer)))
 					{
@@ -443,7 +608,7 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 												   0,dpos,lengthThisStep,
 													   attributes,sProps,
 												   pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-					UT_ASSERT(bResult);
+					UT_return_val_if_fail (bResult, false);
 					if (fragOffsetNewEnd > 0)
 					{
 						// skip over the rest of this frag since we've already
@@ -463,8 +628,8 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 													 0,dpos,lengthThisStep,
 														 attributes,sProps,
 													 pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-					UT_ASSERT(bResult);
-					UT_ASSERT(fragOffsetNewEnd == 0);
+					UT_return_val_if_fail (bResult, false);
+					UT_return_val_if_fail (fragOffsetNewEnd == 0,false);
 				}
 				break;
 
@@ -475,7 +640,7 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 														  dpos,
 														  attributes,sProps,
 													  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-					UT_ASSERT(bResult);
+					UT_return_val_if_fail (bResult,false);
 				}
 				break;
 			}
@@ -500,16 +665,70 @@ bool pt_PieceTable::_realChangeStruxFmt(PTChangeFmt ptc,
 	return true;
 }
 
+bool pt_PieceTable::changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pst,
+											 const XML_Char ** attrs, const XML_Char ** props,
+											 bool bSkipEmbededSections)
+{
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
 
+	pf_Frag * pf = m_fragments.findFirstFragBeforePos(dpos);;
 
+	UT_return_val_if_fail ( pf, false );
 
+	pf = _findLastStruxOfType(pf, pst, bSkipEmbededSections);
+	
+	UT_return_val_if_fail( pf, false );
+	
+	PT_AttrPropIndex currentAP = pf->getIndexAP();
 
+	const PP_AttrProp * pOldAP;
+    if(!getAttrProp(currentAP,&pOldAP))
+		return false;
 
+	PP_AttrProp * pNewAP = pOldAP->cloneWithReplacements(attrs,props,false);
+	pNewAP->markReadOnly();
 
+	PT_AttrPropIndex indexAP;
+	if (!m_varset.addIfUniqueAP(pNewAP,&indexAP))
+		return false;
 
+	pf->setIndexAP(indexAP);
 
+	return true;
+	
+}
 
+	
+bool pt_PieceTable::changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pst,
+											 const XML_Char ** attrs, const XML_Char * props,
+											 bool bSkipEmbededSections)
+{
+	if(props && *props)
+	{
+		// we parse the xml props string into separate field by simply duplicating it and then
+		// replacing ; and : with '0';
+	
+		// foolproofing
+		if(*props == ';')
+			props++;
+		
+		char * pProps = UT_strdup(props);
 
+		const XML_Char ** pPropsArray = UT_splitPropsToArray(pProps);
+		UT_return_val_if_fail( pPropsArray, false );
+		
+		bool bRet = changeLastStruxFmtNoUndo(dpos, pst, attrs, pPropsArray, bSkipEmbededSections);
 
+		delete [] pPropsArray;
+		FREEP(pProps);
+
+		return bRet;
+	}
+	else
+	{
+		const XML_Char ** pPropsArray = NULL;
+		return changeLastStruxFmtNoUndo(dpos, pst, attrs, pPropsArray, bSkipEmbededSections);
+	}
+}
 
 
