@@ -49,11 +49,12 @@
 /****************************************************************/
 
 #define DONE()  (bUndo ? m_history.didUndo() : m_history.didRedo());
+#define UNDO_return_val_if_fail(cond,val) if (!(cond)) { UT_ASSERT(cond); m_bDoingTheDo = false; return (val); }
 
 bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 {
 	// actually do the work of the undo or redo.
-
+	m_bDoingTheDo = true;
 	switch (pcr->getType())
 	{
 
@@ -62,6 +63,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 
 	case PX_ChangeRecord::PXT_GlobMarker:
 		DONE();
+		m_bDoingTheDo = false;
 		return true;
 		
 	//////////////////////////////////////////////////////////////////
@@ -73,11 +75,16 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrSpan->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound,false);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux,false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux,false);
+			}
 			if (!_insertSpan(pf,pcrSpan->getBufIndex(),fragOffset,
 							 pcrSpan->getLength(),pcrSpan->getIndexAP(),
                              pcrSpan->getField()))
@@ -86,6 +93,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 		
 	case PX_ChangeRecord::PXT_DeleteSpan:
@@ -100,22 +108,29 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrSpan->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_Text);
+			UNDO_return_val_if_fail (bFound,false);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_Text,false);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux, false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux, false);
+			}
+			UNDO_return_val_if_fail (bFoundStrux, false);
 
 			pf_Frag_Text * pft = static_cast<pf_Frag_Text *> (pf);
-			UT_ASSERT(pft->getIndexAP() == pcrSpan->getIndexAP());
-
+			UNDO_return_val_if_fail (pft->getIndexAP() == pcrSpan->getIndexAP(),false);
+			xxx_UT_DEBUGMSG(("deletespan in _doTheDo length %d \n",pcrSpan->getLength()));
 			_deleteSpan(pft,fragOffset,pcrSpan->getBufIndex(),pcrSpan->getLength(),NULL,NULL);
 
 			m_pDocument->notifyListeners(pfs,pcr);
 
 			DONE();
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	case PX_ChangeRecord::PXT_ChangeSpan:
@@ -129,14 +144,19 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrs->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_Text);
+			UNDO_return_val_if_fail (bFound, false);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_Text, false);
 
 			pf_Frag_Text * pft = static_cast<pf_Frag_Text *> (pf);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux, false);
+			}
+			UNDO_return_val_if_fail (bFoundStrux, false);
 
 			// we need to loop here, because even though we have a simple (atomic) change,
 			// the document may be fragmented slightly differently (or rather, it may not
@@ -156,7 +176,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 				if (length == 0)
 					break;
 
-				UT_ASSERT(pfEnd->getType() == pf_Frag::PFT_Text);
+				UNDO_return_val_if_fail (pfEnd->getType() == pf_Frag::PFT_Text, false);
 				pft = static_cast<pf_Frag_Text *> (pfEnd);
 				fragOffset = fragOffsetEnd;
 			}
@@ -164,6 +184,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 			
 	//////////////////////////////////////////////////////////////////
@@ -179,14 +200,19 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFoundFrag = getFragFromPosition(pcrStrux->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFoundFrag);
+			UNDO_return_val_if_fail (bFoundFrag, false);
 
 			// get the strux containing the given position.
 	
 // TODO see if we can avoid this call to _getStruxFromPosition ??
 			pf_Frag_Strux * pfsContainer = NULL;
 			bool bFoundContainer = _getStruxFromPosition(pcrStrux->getPosition(),&pfsContainer);
-			UT_ASSERT(bFoundContainer);
+			UNDO_return_val_if_fail (bFoundContainer,false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfsContainer)))
+			{
+				bool bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfsContainer),&pfsContainer);
+				UNDO_return_val_if_fail (bFoundStrux, false);
+			}
 			_insertStrux(pf,fragOffset,pfsNew);
 			DONE();
 			m_pDocument->notifyListeners(pfsContainer,pfsNew,pcr);
@@ -199,18 +225,19 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFoundFrag = getFragFromPosition(pcrStrux->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFoundFrag);
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
+			UNDO_return_val_if_fail (bFoundFrag,false);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_Strux,false);
 
 			pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *> (pf);
-			UT_ASSERT(pcrStrux->getStruxType() == pfs->getStruxType());
+			UNDO_return_val_if_fail (pcrStrux->getStruxType() == pfs->getStruxType(),false);
 			bool bResult = _unlinkStrux(pfs,NULL,NULL);
 			m_pDocument->notifyListeners(pfs,pcr);
-			UT_ASSERT(bResult);
+			UNDO_return_val_if_fail (bResult,false);
 			DONE();
 			
 			delete pfs;
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	case PX_ChangeRecord::PXT_ChangeStrux:
@@ -220,12 +247,13 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			const PX_ChangeRecord_StruxChange * pcrs = static_cast<const PX_ChangeRecord_StruxChange *>(pcr);
 			pf_Frag_Strux * pfs;
 			bool bFound = _getStruxFromPosition(pcrs->getPosition(),&pfs);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound,false);
 			bool bResult = _fmtChangeStrux(pfs,pcrs->getIndexAP());
-			UT_ASSERT(bResult);
+			UNDO_return_val_if_fail (bResult,false);
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	//////////////////////////////////////////////////////////////////
@@ -237,16 +265,21 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrObject->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound, false);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux, false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bool bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux,false);
+			}
             pf_Frag_Object * pfo = NULL;
 			if (!_insertObject(pf,fragOffset,pcrObject->getObjectType(),
                                pcrObject->getIndexAP(),pfo))
 				return false;
-            UT_ASSERT(pfo);
+            UNDO_return_val_if_fail (pfo,false);
             
             // need to set field pointers to values of new pointer
             // as old field doesn't exist
@@ -263,6 +296,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			m_pDocument->notifyListeners(pfs,pcr);
             // don't update field until all of changes have been made
 		}
+		m_bDoingTheDo = false;
 		return true;
 		
 	case PX_ChangeRecord::PXT_DeleteObject:
@@ -271,21 +305,27 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrObject->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_Object);
-			UT_ASSERT(fragOffset == 0);
+			UNDO_return_val_if_fail (bFound, false);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_Object,false);
+			UNDO_return_val_if_fail (fragOffset == 0,false);
 			
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux,false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bool bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux,false);
+			}
 
 			pf_Frag_Object * pfo = static_cast<pf_Frag_Object *> (pf);
-			UT_ASSERT(pfo->getIndexAP() == pcrObject->getIndexAP());
+			UNDO_return_val_if_fail (pfo->getIndexAP() == pcrObject->getIndexAP(),false);
 			_deleteObject(pfo,NULL,NULL);
 
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	case PX_ChangeRecord::PXT_ChangeObject:
@@ -297,13 +337,18 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcro->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_Object);
-			UT_ASSERT(fragOffset == 0);
+			UNDO_return_val_if_fail (bFound,false);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_Object,false);
+			UNDO_return_val_if_fail (fragOffset == 0, false);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux,false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bool bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux,false);
+			}
 
 			pf_Frag_Object * pfo = static_cast<pf_Frag_Object *> (pf);
 
@@ -312,6 +357,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	///////////////////////////////////////////////////////////////////
@@ -323,11 +369,16 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrFM->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound, false);
 
 			pf_Frag_Strux * pfs = NULL;
 			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UNDO_return_val_if_fail (bFoundStrux, false);
+			if(isEndFootnote(static_cast<pf_Frag *>(pfs)))
+			{
+				bool bFoundStrux = _getStruxFromFragSkip(static_cast<pf_Frag *>(pfs),&pfs);
+				UNDO_return_val_if_fail (bFoundStrux,false);
+			}
 
 			if (!_insertFmtMark(pf,fragOffset,pcrFM->getIndexAP()))
 				return false;
@@ -335,6 +386,7 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 	 
 	case PX_ChangeRecord::PXT_DeleteFmtMark:
@@ -343,26 +395,27 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrFM->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound, false);
 
 			// we backup one because we have zero length and getFragFromPosition()
 			// returns the right-most thing with this document position.
 			pf = pf->getPrev();
 
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_FmtMark);
-			UT_ASSERT(fragOffset == 0);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_FmtMark,false);
+			UNDO_return_val_if_fail (fragOffset == 0,false);
 			
 			pf_Frag_Strux * pfs = NULL;
-			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			bool bFoundStrux = _getStruxFromFragSkip(pf,&pfs);
+			UNDO_return_val_if_fail (bFoundStrux,false);
 
 			pf_Frag_FmtMark * pffm = static_cast<pf_Frag_FmtMark *> (pf);
-			UT_ASSERT(pffm->getIndexAP() == pcrFM->getIndexAP());
+			UNDO_return_val_if_fail (pffm->getIndexAP() == pcrFM->getIndexAP(),false);
 			_deleteFmtMark(pffm,NULL,NULL);
 
 			DONE();
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
+		m_bDoingTheDo = false;
 		return true;
 
 	case PX_ChangeRecord::PXT_ChangeFmtMark:
@@ -374,24 +427,25 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 			pf_Frag * pf = NULL;
 			PT_BlockOffset fragOffset = 0;
 			bool bFound = getFragFromPosition(pcrFMC->getPosition(),&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UNDO_return_val_if_fail (bFound,false);
 
 			// we backup one because we have zero length and getFragFromPosition()
 			// returns the right-most thing with this document position.
 			pf = pf->getPrev();
 
-			UT_ASSERT(pf->getType() == pf_Frag::PFT_FmtMark);
-			UT_ASSERT(fragOffset == 0);
+			UNDO_return_val_if_fail (pf->getType() == pf_Frag::PFT_FmtMark,false);
+			UNDO_return_val_if_fail (fragOffset == 0,false);
 
 			pf_Frag_Strux * pfs = NULL;
-			bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			bool bFoundStrux = _getStruxFromFragSkip(pf,&pfs);
+			UNDO_return_val_if_fail (bFoundStrux,false);
 
 			pf_Frag_FmtMark * pffm = static_cast<pf_Frag_FmtMark *> (pf);
 
 			_fmtChangeFmtMark(pffm,pcrFMC->getIndexAP(),NULL,NULL);
 
 			DONE();
+			m_bDoingTheDo = false;
 			m_pDocument->notifyListeners(pfs,pcr);
 		}
 		return true;
@@ -403,10 +457,12 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 		DONE();
 		m_pDocument->notifyListeners(NULL, pcr);
 
+		m_bDoingTheDo = false;
 		return true;
 
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
+		m_bDoingTheDo = false;
 		return false;
 	}
 }
@@ -414,12 +470,12 @@ bool pt_PieceTable::_doTheDo(const PX_ChangeRecord * pcr, bool bUndo)
 /*****************************************************************/
 /*****************************************************************/
 
-#define GETGLOBFLAGS(pcr)			(  (pcr->getType() == PX_ChangeRecord::PXT_GlobMarker)			\
-									 ? (static_cast<PX_ChangeRecord_Glob *>((pcr))->getFlags())		\
-									 : (UT_Byte)PX_ChangeRecord_Glob::PXF_Null)
-#define GETREVGLOBFLAGS(pcr)		(  (pcr->getType() == PX_ChangeRecord::PXT_GlobMarker)			\
-									 ? (static_cast<PX_ChangeRecord_Glob *>((pcr))->getRevFlags())	\
-									 : (UT_Byte)PX_ChangeRecord_Glob::PXF_Null)
+#define GETGLOBFLAGS(pcr)			(  (pcr->getType() == PX_ChangeRecord::PXT_GlobMarker)		\
+									 ? static_cast<PX_ChangeRecord_Glob *>(pcr)->getFlags()		\
+									 : static_cast<UT_Byte>(PX_ChangeRecord_Glob::PXF_Null))
+#define GETREVGLOBFLAGS(pcr)		(  (pcr->getType() == PX_ChangeRecord::PXT_GlobMarker)		\
+									 ? static_cast<PX_ChangeRecord_Glob *>(pcr)->getRevFlags()	\
+									 : static_cast<UT_Byte>(PX_ChangeRecord_Glob::PXF_Null))
 
 /*****************************************************************/
 /*****************************************************************/
@@ -444,7 +500,7 @@ bool pt_PieceTable::undoCmd(void)
 	PX_ChangeRecord * pcr;
 	if (!m_history.getUndo(&pcr))
 		return false;
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr,false);
 
 	// the first undo record tells us whether it is
 	// a simple change or a glob.  there are two kinds
@@ -458,11 +514,14 @@ bool pt_PieceTable::undoCmd(void)
 	// corresponding other end.
 
 	UT_Byte flagsFirst = GETGLOBFLAGS(pcr);
-
+	if(m_fragments.areFragsDirty())
+	{
+		m_fragments.cleanFrags();
+	}
 	do
 	{
 		PX_ChangeRecord * pcrRev = pcr->reverse(); // we must delete this.
-		UT_ASSERT(pcrRev);
+		UT_return_val_if_fail (pcrRev,false);
 		UT_Byte flagsRev = GETGLOBFLAGS(pcrRev);
 		bool bResult = _doTheDo(pcrRev, true);
 		delete pcrRev;
@@ -485,7 +544,7 @@ bool pt_PieceTable::redoCmd(void)
 	PX_ChangeRecord * pcr;
 	if (!m_history.getRedo(&pcr))
 		return false;
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr,false);
 
 	// the first undo record tells us whether it is
 	// a simple change or a glob.  there are two kinds
@@ -499,6 +558,10 @@ bool pt_PieceTable::redoCmd(void)
 	// corresponding other end.
 
 	UT_Byte flagsRevFirst = GETREVGLOBFLAGS(pcr);
+	if(m_fragments.areFragsDirty())
+	{
+		m_fragments.cleanFrags();
+	}
 
 	while (m_history.getRedo(&pcr))
 	{
