@@ -26,7 +26,8 @@
 
 #include "xap_App.h"
 #include "xap_QNXApp.h"
-#include "xap_QNXFrame.h"
+#include "xap_QNXFrameImpl.h"
+#include "xap_Frame.h"
 
 #include "ap_Strings.h"
 #include "ap_Dialog_Id.h"
@@ -100,13 +101,11 @@ AP_QNXDialog_ToggleCase::~AP_QNXDialog_ToggleCase(void)
 void AP_QNXDialog_ToggleCase::runModal(XAP_Frame * pFrame)
 {
 	// get top level window 
-	XAP_QNXFrame * frame = static_cast<XAP_QNXFrame *>(pFrame);
-	UT_ASSERT(frame);
-
-	PtWidget_t *parent = frame->getTopLevelWindow();
+	XAP_QNXFrameImpl * pQNXFrameImpl = (XAP_QNXFrameImpl*)pFrame->getFrameImpl();
+	PtWidget_t *parent =	pQNXFrameImpl->getTopLevelWindow();	
 	UT_ASSERT(parent);
-	PtSetParentWidget(parent);
 
+	PtSetParentWidget(parent);
 	// build the dialog
 	PtWidget_t * window = _constructWindow();
 	UT_ASSERT(window);
@@ -131,7 +130,7 @@ void AP_QNXDialog_ToggleCase::runModal(XAP_Frame * pFrame)
 }
 
 static ToggleCase *find_entry(ToggleCase match) {
-	int i;
+	unsigned int i;
 	for(i=0; i< sizeof(toggle_entries) / sizeof(*toggle_entries); i++) {
 		if(toggle_entries[i] == match) {
 			return &toggle_entries[i];
@@ -142,6 +141,7 @@ static ToggleCase *find_entry(ToggleCase match) {
 
 PtWidget_t * AP_QNXDialog_ToggleCase::_constructWindow(void)
 {
+#if 0 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	PtWidget_t *window, *widget;
 	PtWidget_t *vboxMain;
@@ -155,10 +155,12 @@ PtWidget_t * AP_QNXDialog_ToggleCase::_constructWindow(void)
 	int n;
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_Title), 0);
+PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, _(AP,DLG_ToggleCase_Title), 0);
     PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, 0, ABI_MODAL_WINDOW_RENDER_FLAGS);
     PtSetArg(&args[n++], Pt_ARG_WINDOW_MANAGED_FLAGS, 0, ABI_MODAL_WINDOW_MANAGE_FLAGS);
 	window = PtCreateWidget(PtWindow, NULL, n, args);
+	SetupContextHelp(window,this);
+	PtAddHotkeyHandler(window,Pk_F1,0,Pt_HOTKEY_SYM,this,OpenHelp);
 	PtAddCallback(window, Pt_CB_WINDOW_CLOSING, s_cancel_clicked, this);
 
 	n = 0;
@@ -177,10 +179,10 @@ PtWidget_t * AP_QNXDialog_ToggleCase::_constructWindow(void)
 				Pt_GROUP_STRETCH_VERTICAL | Pt_GROUP_STRETCH_HORIZONTAL | Pt_GROUP_EXCLUSIVE,
 				Pt_GROUP_STRETCH_VERTICAL | Pt_GROUP_STRETCH_HORIZONTAL | Pt_GROUP_EXCLUSIVE);
 	vboxMain =  PtCreateWidget(PtGroup, vboxOuter, n, args);
-  	pretty_group(vboxMain, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_Title));
+	pretty_group(vboxMain, _(AP,DLG_ToggleCase_Title));
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_SentenceCase), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(AP,DLG_ToggleCase_SentenceCase), 0);
 	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
 	PtSetArg(&args[n++], Pt_ARG_POINTER, find_entry(CASE_SENTENCE), 0);
 	PtSetArg(&args[n++], Pt_ARG_FLAGS, Pt_SET, Pt_SET);
@@ -188,28 +190,28 @@ PtWidget_t * AP_QNXDialog_ToggleCase::_constructWindow(void)
 	PtAddCallback(widget, Pt_CB_ACTIVATE, s_toggled, this);
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_LowerCase), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(AP,DLG_ToggleCase_LowerCase), 0);
 	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
 	PtSetArg(&args[n++], Pt_ARG_POINTER, find_entry(CASE_LOWER), 0);
 	widget = PtCreateWidget(PtToggleButton, vboxMain, n, args);
 	PtAddCallback(widget, Pt_CB_ACTIVATE, s_toggled, this);
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_UpperCase), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(AP,DLG_ToggleCase_UpperCase), 0);
 	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
 	PtSetArg(&args[n++], Pt_ARG_POINTER, find_entry(CASE_UPPER), 0);
 	widget = PtCreateWidget(PtToggleButton, vboxMain, n, args);
 	PtAddCallback(widget, Pt_CB_ACTIVATE, s_toggled, this);
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_FirstUpperCase), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(AP,DLG_ToggleCase_FirstUpperCase), 0);
 	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
 	PtSetArg(&args[n++], Pt_ARG_POINTER, find_entry(CASE_FIRST_CAPITAL), 0);
 	widget = PtCreateWidget(PtToggleButton, vboxMain, n, args);
 	PtAddCallback(widget, Pt_CB_ACTIVATE, s_toggled, this);
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pSS->getValue(AP_STRING_ID_DLG_ToggleCase_ToggleCase), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(AP,DLG_ToggleCase_ToggleCase), 0);
 	PtSetArg(&args[n++], Pt_ARG_INDICATOR_TYPE, Pt_TOGGLE_RADIO, 0);
 	PtSetArg(&args[n++], Pt_ARG_POINTER, find_entry(CASE_TOGGLE), 0);
 	widget = PtCreateWidget(PtToggleButton, vboxMain, n, args);
@@ -221,17 +223,18 @@ PtWidget_t * AP_QNXDialog_ToggleCase::_constructWindow(void)
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING,  pSS->getValue(XAP_STRING_ID_DLG_OK), 0);
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(XAP,DLG_OK), 0);
 	buttonOK = PtCreateWidget(PtButton, hboxBut, n, args);
 	PtAddCallback(buttonOK, Pt_CB_ACTIVATE, s_ok_clicked, this);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING,  pSS->getValue(XAP_STRING_ID_DLG_Cancel), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(XAP,DLG_Cancel), 0);
 	buttonCancel = PtCreateWidget(PtButton, hboxBut, n, args);
 	PtAddCallback(buttonCancel, Pt_CB_ACTIVATE, s_cancel_clicked, this);
 
 	return window;
+#endif
 }
 
 void AP_QNXDialog_ToggleCase::setFinalAnswer(AP_Dialog_ToggleCase::tAnswer ans) {
