@@ -17,6 +17,8 @@
  * 02111-1307, USA.
  */
 
+#undef GTK_DISABLE_DEPRECATED
+
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
@@ -73,6 +75,15 @@ static void s_customChanged (GtkWidget * widget, AP_UnixDialog_Lists * me)
 	me->customChanged ();
 }
 
+static void s_FoldCheck_changed(GtkWidget * widget, AP_UnixDialog_Lists * me)
+{
+	UT_DEBUGMSG(("Doing s_FoldCheck_changed \n"));
+	UT_UTF8String sLevel = static_cast<char *> (g_object_get_data(G_OBJECT(widget),"level"));
+	UT_sint32 iLevel = atoi(sLevel.utf8_str());
+	gboolean iVal =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	bool bSet = (iVal == TRUE);
+	me->setFoldLevel(iLevel,bSet);
+}
 
 static void s_styleChangedNone (GtkWidget * widget, AP_UnixDialog_Lists * me)
 {
@@ -154,12 +165,13 @@ static gboolean s_update (void)
 
 void AP_UnixDialog_Lists::closeClicked(void)
 {
+	setAnswer(AP_Dialog_Lists::a_QUIT);	
 	abiDestroyWidget(m_wMainWindow); // emit the correct signals
 }
 
 void AP_UnixDialog_Lists::runModal( XAP_Frame * pFrame)
 {
-	List_Type  savedListType;
+	FL_ListType  savedListType;
 	setModal();
 	
 	GtkWidget * mainWindow = _constructWindow();
@@ -184,7 +196,9 @@ void AP_UnixDialog_Lists::runModal( XAP_Frame * pFrame)
 	UT_ASSERT(m_wPreviewArea && m_wPreviewArea->window);
 
 	// make a new Unix GC
-	m_pPreviewWidget = new GR_UnixGraphics(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	//m_pPreviewWidget = new GR_UnixGraphics(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixAllocInfo ai(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	m_pPreviewWidget = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
 
 	// let the widget materialize
 	_createPreviewFromGC(m_pPreviewWidget,
@@ -195,9 +209,10 @@ void AP_UnixDialog_Lists::runModal( XAP_Frame * pFrame)
 	setNewListType(savedListType);
 	
 	abiRunModalDialog ( GTK_DIALOG(mainWindow), pFrame, this, BUTTON_CANCEL, false );
-	
+	AP_Dialog_Lists::tAnswer res = getAnswer();
 	g_list_free( m_glFonts);
 	abiDestroyWidget ( mainWindow ) ;
+	setAnswer(res);
 	DELETEP (m_pPreviewWidget);
 }
 
@@ -226,7 +241,8 @@ void AP_UnixDialog_Lists::runModeless (XAP_Frame * pFrame)
 	UT_ASSERT(m_wPreviewArea && m_wPreviewArea->window);
 
 	// make a new Unix GC
-	m_pPreviewWidget = new GR_UnixGraphics(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixAllocInfo ai(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	m_pPreviewWidget = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
 
 	// let the widget materialize
 
@@ -303,6 +319,73 @@ void AP_UnixDialog_Lists::destroy(void)
 	}
 }
 
+/*!
+ * Set the Fold level from the XP layer.
+ */
+void AP_UnixDialog_Lists::setFoldLevelInGUI(void)
+{
+	setFoldLevel(getCurrentFold(),true);
+}
+
+/*!
+ * Set the Fold Level in the current List structure.
+ */
+void AP_UnixDialog_Lists::setFoldLevel(UT_sint32 iLevel, bool bSet)
+{
+	UT_sint32 i = 0;
+	UT_sint32 count = static_cast<UT_sint32>(m_vecFoldCheck.getItemCount());
+	if(iLevel >= count)
+	{
+		return;
+	}
+	GtkWidget * wF = NULL;
+	UT_uint32 ID =0;
+	if(!bSet)
+	{
+		for(i=0; i< count;i++)
+		{
+			wF = m_vecFoldCheck.getNthItem(i);
+			ID = m_vecFoldID.getNthItem(i);
+			g_signal_handler_block(G_OBJECT(wF),ID);
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wF),FALSE);
+			g_signal_handler_unblock(G_OBJECT(wF),ID);
+		}
+		wF = m_vecFoldCheck.getNthItem(0);
+		ID = m_vecFoldID.getNthItem(0);
+		g_signal_handler_block(G_OBJECT(wF),ID);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wF),TRUE);
+		g_signal_handler_unblock(G_OBJECT(wF),ID);
+		setCurrentFold(0);
+	}
+	else
+	{
+		for(i=0; i< count;i++)
+		{
+			wF = m_vecFoldCheck.getNthItem(i);
+			ID = m_vecFoldID.getNthItem(i);
+			g_signal_handler_block(G_OBJECT(wF),ID);
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wF),FALSE);
+			g_signal_handler_unblock(G_OBJECT(wF),ID);
+		}
+		wF = m_vecFoldCheck.getNthItem(iLevel);
+		ID = m_vecFoldID.getNthItem(iLevel);
+		g_signal_handler_block(G_OBJECT(wF),ID);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wF),TRUE);
+		g_signal_handler_unblock(G_OBJECT(wF),ID);
+		setCurrentFold(iLevel);
+	}
+}
+
+bool AP_UnixDialog_Lists::isPageLists(void)
+{
+	if(isModal())
+	{
+		return true;
+	}
+	bool isPage =  (gtk_notebook_get_current_page(GTK_NOTEBOOK(m_wContents)) == m_iPageLists);
+	return isPage;
+}
+
 void AP_UnixDialog_Lists::activate (void)
 {
 	UT_ASSERT (m_wMainWindow);
@@ -406,7 +489,7 @@ void  AP_UnixDialog_Lists::styleChanged(gint type)
 void  AP_UnixDialog_Lists::setListTypeFromWidget(void)
 {
 	GtkWidget * wlisttype=gtk_menu_get_active(GTK_MENU(m_wListStyle_menu));
-	setNewListType(static_cast<List_Type>(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wlisttype), "user_data"))));
+	setNewListType(static_cast<FL_ListType>(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wlisttype), "user_data"))));
 }
 
 /*!
@@ -454,6 +537,7 @@ void  AP_UnixDialog_Lists::applyClicked(void)
 	if(isModal())
 	{
 		setAnswer(AP_Dialog_Lists::a_OK);
+		
 	}
 }
 
@@ -538,8 +622,9 @@ void AP_UnixDialog_Lists::_fillFontMenu(GtkWidget* menu)
 	// Sevior: This is not a valid font name. However if you select
         // this you get whatever font is in the document at the point the
         // list is inserted.
-
-	glade_menuitem = gtk_menu_item_new_with_label (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Current_Font).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Current_Font,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data (G_OBJECT (glade_menuitem), "user_data", GINT_TO_POINTER (0));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), glade_menuitem);
@@ -599,9 +684,128 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 	GtkWidget *preview_frame;
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	UT_UTF8String s;
+	GtkWidget * wNoteBook = NULL;
 
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
+	if(!isModal())
+	{
+
+// Note Book creation
+
+		wNoteBook = gtk_notebook_new ();
+		gtk_widget_show(wNoteBook);
+
+// Container for the lists
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_PageProperties,s);
+		GtkWidget * lbPageLists = gtk_label_new(s.utf8_str());
+		gtk_widget_show(lbPageLists);
+		gtk_notebook_append_page(GTK_NOTEBOOK(wNoteBook),vbox2,lbPageLists);
+
+		m_iPageLists = gtk_notebook_page_num(GTK_NOTEBOOK(wNoteBook),vbox2);
+
+// Container for Text Folding
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_PageFolding,s);
+		GtkWidget * lbPageFolding = gtk_label_new(s.utf8_str());
+		GtkWidget * wFoldingTable = gtk_table_new(6,3,FALSE);
+		gtk_widget_show(lbPageFolding);
+		gtk_widget_show(wFoldingTable);
+		gtk_notebook_append_page(GTK_NOTEBOOK(wNoteBook),wFoldingTable,lbPageFolding);
+
+		m_iPageFold = gtk_notebook_page_num(GTK_NOTEBOOK(wNoteBook),wFoldingTable);
+
+// Left Spacing Here
+
+		GtkWidget * lbLeftSpacer = gtk_label_new("");
+		gtk_misc_set_padding(GTK_MISC(lbLeftSpacer),8,0);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),lbLeftSpacer,0,1,0,6,GTK_SHRINK,GTK_FILL,0,0);
+		gtk_widget_show(lbLeftSpacer);
+
+// Bold markup
+		GtkWidget * lbFoldHeading = gtk_label_new("<b>%s</b>");
+		gtk_label_set_use_markup(GTK_LABEL(lbFoldHeading),TRUE);
+
+		localizeLabelMarkup(lbFoldHeading,pSS,AP_STRING_ID_DLG_Lists_FoldingLevelexp);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),lbFoldHeading,1,3,0,1,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(lbFoldHeading);
+
+// Mid Left Spacing Here
+
+		GtkWidget * lbMidLeftSpacer = gtk_label_new("");
+		gtk_misc_set_padding(GTK_MISC(lbMidLeftSpacer),8,0);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),lbMidLeftSpacer,1,2,1,6,GTK_SHRINK,GTK_FILL,0,0);
+		gtk_widget_show(lbMidLeftSpacer);
+
+		m_vecFoldCheck.clear();
+		m_vecFoldID.clear();
+		UT_uint32 ID =0;
+// CheckButtons
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_FoldingLevel0,s);
+		GtkWidget * wF = gtk_check_button_new_with_label(s.utf8_str());
+		g_object_set_data(G_OBJECT(wF),"level",(gpointer)"0");
+		ID = g_signal_connect(G_OBJECT(wF),
+						  "toggled",
+						 G_CALLBACK(s_FoldCheck_changed),
+						 (gpointer) this);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),wF,2,3,1,2,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(wF);
+		m_vecFoldCheck.addItem(wF);
+		m_vecFoldID.addItem(ID);
+
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_FoldingLevel1,s);
+		wF = gtk_check_button_new_with_label(s.utf8_str());
+		g_object_set_data(G_OBJECT(wF),"level",(gpointer)"1");
+		ID = g_signal_connect(G_OBJECT(wF),
+						  "toggled",
+						 G_CALLBACK(s_FoldCheck_changed),
+						 (gpointer) this);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),wF,2,3,2,3,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(wF);
+		m_vecFoldCheck.addItem(wF);
+		m_vecFoldID.addItem(ID);
+
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_FoldingLevel2,s);
+		wF = gtk_check_button_new_with_label(s.utf8_str());
+		g_object_set_data(G_OBJECT(wF),"level",(gpointer)"2");
+		ID = g_signal_connect(G_OBJECT(wF),
+						  "toggled",
+						 G_CALLBACK(s_FoldCheck_changed),
+						 (gpointer) this);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),wF,2,3,3,4,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(wF);
+		m_vecFoldCheck.addItem(wF);
+		m_vecFoldID.addItem(ID);
+
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_FoldingLevel3,s);
+		wF = gtk_check_button_new_with_label(s.utf8_str());
+		g_object_set_data(G_OBJECT(wF),"level",(gpointer)"3");
+		ID = g_signal_connect(G_OBJECT(wF),
+						  "toggled",
+						 G_CALLBACK(s_FoldCheck_changed),
+						 (gpointer) this);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),wF,2,3,4,5,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(wF);
+		m_vecFoldCheck.addItem(wF);
+		m_vecFoldID.addItem(ID);
+
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_FoldingLevel4,s);
+		wF = gtk_check_button_new_with_label(s.utf8_str());
+		g_object_set_data(G_OBJECT(wF),"level",(gpointer)"4");
+		ID = g_signal_connect(G_OBJECT(wF),
+						  "toggled",
+						 G_CALLBACK(s_FoldCheck_changed),
+						 (gpointer) this);
+		gtk_table_attach(GTK_TABLE(wFoldingTable),wF,2,3,5,6,GTK_FILL,GTK_EXPAND,0,0);
+		gtk_widget_show(wF);
+		m_vecFoldCheck.addItem(wF);
+		m_vecFoldID.addItem(ID);
+		gtk_widget_show(wFoldingTable);
+
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(wNoteBook),m_iPageLists);
+	}
+
+// List Page
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
 	hbox2 = gtk_hbox_new (FALSE, 8);
@@ -643,17 +847,20 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 					  (GtkAttachOptions) (0), 0, 0);
 
 	type_om_menu = gtk_menu_new ();
-	GtkWidget * menu_none = gtk_menu_item_new_with_label (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none,s);
+	GtkWidget * menu_none = gtk_menu_item_new_with_label (s.utf8_str());
 	m_wMenu_None = menu_none;
 	g_object_set_data(G_OBJECT(menu_none),"user_data",GINT_TO_POINTER(0));
 	gtk_widget_show (menu_none);
 	gtk_menu_shell_append (GTK_MENU_SHELL (type_om_menu), menu_none);
-	GtkWidget * menu_bull  = gtk_menu_item_new_with_label (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_bullet).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_bullet,s);
+	GtkWidget * menu_bull  = gtk_menu_item_new_with_label (s.utf8_str());
 	g_object_set_data(G_OBJECT(menu_bull),"user_data",GINT_TO_POINTER(1));
 	gtk_widget_show (menu_bull);
 	m_wMenu_Bull = menu_bull;
 	gtk_menu_shell_append (GTK_MENU_SHELL (type_om_menu), menu_bull);
-	GtkWidget * menu_num  = gtk_menu_item_new_with_label (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered,s);
+	GtkWidget * menu_num  = gtk_menu_item_new_with_label (s.utf8_str());
 	g_object_set_data(G_OBJECT(menu_num),"user_data",GINT_TO_POINTER(2));
 	gtk_widget_show (menu_num);
 	m_wMenu_Num = menu_num;
@@ -664,22 +871,22 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 	//
 	gtk_option_menu_set_history (GTK_OPTION_MENU (type_om), 2);
 	gtk_widget_set_events(type_om, GDK_ALL_EVENTS_MASK);
-
-	type_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type,s);
+	type_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (type_lb);
 	gtk_table_attach (GTK_TABLE (table1), type_lb, 0, 1, 0, 1,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (type_lb), 0, 0.5);
-
-	style_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Style).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Style,s);
+	style_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (style_lb);
 	gtk_table_attach (GTK_TABLE (table1), style_lb, 0, 1, 1, 2,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (style_lb), 0, 0.5);
-
-	customized_cb = gtk_button_new_with_label (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_SetDefault).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_SetDefault,s);
+	customized_cb = gtk_button_new_with_label (s.utf8_str());
 	gtk_widget_show (customized_cb);
 	gtk_table_attach (GTK_TABLE (table1), customized_cb, 0, 2, 2, 3,
 					  (GtkAttachOptions) (GTK_SHRINK),
@@ -749,42 +956,43 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (label_align_sb), TRUE);
 	gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (label_align_sb), TRUE);
 
-	format_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Format).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Format,s);
+	format_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (format_lb);
 	gtk_table_attach (GTK_TABLE (table2), format_lb, 0, 1, 0, 1,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (format_lb), 0.0, 0.5);
-
-	font_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Font).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Font,s);
+	font_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (font_lb);
 	gtk_table_attach (GTK_TABLE (table2), font_lb, 0, 1, 1, 2,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (font_lb), 0.0, 0.5);
-
-	delimiter_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_DelimiterString).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_DelimiterString,s);
+	delimiter_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (delimiter_lb);
 	gtk_table_attach (GTK_TABLE (table2), delimiter_lb, 0, 1, 2, 3,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (delimiter_lb), 0.0, 0.5);
-
-	start_at_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start,s);
+	start_at_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (start_at_lb);
 	gtk_table_attach (GTK_TABLE (table2), start_at_lb, 0, 1, 3, 4,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (start_at_lb), 0.0, 0.5);
-
-	text_align_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Align).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Align,s);
+	text_align_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (text_align_lb);
 	gtk_table_attach (GTK_TABLE (table2), text_align_lb, 0, 1, 4, 5,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (text_align_lb), 0.0, 0.5);
-
-	label_align_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Indent).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Indent,s);
+	label_align_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (label_align_lb);
 	gtk_table_attach (GTK_TABLE (table2), label_align_lb, 0, 1, 5, 6,
 					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -794,8 +1002,8 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 	vbox3 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox3);
 	gtk_box_pack_start (GTK_BOX (hbox2), vbox3, TRUE, TRUE, 0);
-
-	preview_lb = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Preview).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Preview,s);
+	preview_lb = gtk_label_new (s.utf8_str());
 	gtk_widget_show (preview_lb);
 	gtk_box_pack_start (GTK_BOX (vbox3), preview_lb, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (preview_lb), 0.0, 0.5);
@@ -815,28 +1023,35 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 	if(!isModal())
 		gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
-
-	start_list_rb = gtk_radio_button_new_with_label (action_group, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New,s);
+	start_list_rb = gtk_radio_button_new_with_label (action_group, s.utf8_str());
 	action_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (start_list_rb));
 	if(!isModal())
 		gtk_widget_show (start_list_rb);
 	gtk_box_pack_start (GTK_BOX (hbox1), start_list_rb, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (start_list_rb), TRUE);
-
-	apply_list_rb = gtk_radio_button_new_with_label (action_group, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Apply_Current).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Apply_Current,s);
+	apply_list_rb = gtk_radio_button_new_with_label (action_group, s.utf8_str());
 	action_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (apply_list_rb));
 	if(!isModal())
 		gtk_widget_show (apply_list_rb);
 	gtk_box_pack_start (GTK_BOX (hbox1), apply_list_rb, FALSE, FALSE, 0);
-
-	resume_list_rb = gtk_radio_button_new_with_label (action_group, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume,s);
+	resume_list_rb = gtk_radio_button_new_with_label (action_group, s.utf8_str());
 	action_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resume_list_rb));
 	if(!isModal())
 		gtk_widget_show (resume_list_rb);
 	gtk_box_pack_start (GTK_BOX (hbox1), resume_list_rb, FALSE, FALSE, 0);
 
 	// Save useful widgets in member variables
-	m_wContents = vbox2;
+	if(isModal())
+	{
+		m_wContents = vbox2;
+	}
+	else
+	{
+		m_wContents = wNoteBook;
+	}
 	m_wStartNewList = start_list_rb;
 	m_wStartNew_label = GTK_BIN(start_list_rb)->child;
 	m_wApplyCurrent = apply_list_rb;
@@ -884,7 +1099,7 @@ GtkWidget *AP_UnixDialog_Lists::_constructWindowContents (void)
 GList *  AP_UnixDialog_Lists::_getGlistFonts (void)
 {
 	XAP_UnixApp * unixapp = static_cast<XAP_UnixApp *> (m_pApp);
-	UT_Vector * list = unixapp->getFontManager()->getAllFonts();
+	UT_GenericVector<XAP_UnixFont*>* list = unixapp->getFontManager()->getAllFonts();
 	UT_uint32 count = list->size();
 
 	GList *glFonts = NULL;
@@ -893,7 +1108,7 @@ GList *  AP_UnixDialog_Lists::_getGlistFonts (void)
 
 	for (UT_uint32 i = 0; i < count; i++)
 	{
-		XAP_UnixFont * pFont = static_cast<XAP_UnixFont *>(list->getNthItem(i));
+		XAP_UnixFont * pFont = list->getNthItem(i);
 		const gchar * lgn  = reinterpret_cast<const gchar *>(pFont->getName());
 		if((strstr(currentfont,lgn)==NULL) || (strlen(currentfont)!=strlen(lgn)) )
 		{
@@ -913,9 +1128,9 @@ void AP_UnixDialog_Lists::_fillNoneStyleMenu( GtkWidget *listmenu)
 {
 	GtkWidget *glade_menuitem;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data", GINT_TO_POINTER(
 		static_cast<gint>(NOT_A_LIST) ));
@@ -928,9 +1143,9 @@ void AP_UnixDialog_Lists::_fillNumberedStyleMenu( GtkWidget *listmenu)
 {
 	GtkWidget *glade_menuitem;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Numbered_List).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Numbered_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(NUMBERED_LIST) ));
@@ -938,9 +1153,8 @@ void AP_UnixDialog_Lists::_fillNumberedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Case_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Case_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(LOWERCASE_LIST)));
@@ -948,8 +1162,8 @@ void AP_UnixDialog_Lists::_fillNumberedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Case_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Case_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(UPPERCASE_LIST)));
@@ -957,17 +1171,16 @@ void AP_UnixDialog_Lists::_fillNumberedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Roman_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Roman_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(LOWERROMAN_LIST)));
 	gtk_menu_shell_append (GTK_MENU_SHELL (listmenu), glade_menuitem);
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Roman_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Roman_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(UPPERROMAN_LIST)));
@@ -975,16 +1188,17 @@ void AP_UnixDialog_Lists::_fillNumberedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Arabic_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Arabic_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(ARABICNUMBERED_LIST)));
 	gtk_menu_shell_append (GTK_MENU_SHELL (listmenu), glade_menuitem);
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hebrew_List).utf8_str());
+
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hebrew_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(HEBREW_LIST)));
@@ -998,9 +1212,9 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 {
 	GtkWidget *glade_menuitem;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Bullet_List).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Bullet_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(BULLETED_LIST)));
@@ -1008,8 +1222,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Dashed_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Dashed_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(DASHED_LIST)));
@@ -1017,9 +1231,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Square_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Square_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(SQUARE_LIST)));
@@ -1027,9 +1240,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Triangle_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Triangle_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(TRIANGLE_LIST)));
@@ -1037,8 +1249,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Diamond_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Diamond_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(DIAMOND_LIST)));
@@ -1046,9 +1258,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Star_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Star_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(STAR_LIST)));
@@ -1056,9 +1267,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Implies_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Implies_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(IMPLIES_LIST)));
@@ -1066,9 +1276,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Tick_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Tick_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(TICK_LIST)));
@@ -1076,9 +1285,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Box_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Box_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(BOX_LIST)));
@@ -1087,9 +1295,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 						G_CALLBACK (s_typeChanged), this);
 
 
-
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hand_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hand_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(HAND_LIST)));
@@ -1097,8 +1304,8 @@ void AP_UnixDialog_Lists::_fillBulletedStyleMenu( GtkWidget *listmenu)
 	g_signal_connect (G_OBJECT (glade_menuitem), "activate",
 						G_CALLBACK (s_typeChanged), this);
 
-	glade_menuitem = gtk_menu_item_new_with_label (
-		pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Heart_List).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Heart_List,s);
+	glade_menuitem = gtk_menu_item_new_with_label (s.utf8_str());
 	gtk_widget_show (glade_menuitem);
 	g_object_set_data(G_OBJECT(glade_menuitem),"user_data",GINT_TO_POINTER(
 		static_cast<gint>(HEART_LIST)));
@@ -1112,16 +1319,20 @@ void AP_UnixDialog_Lists::_setRadioButtonLabels(void)
 {
 	//	char *tmp;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	UT_UTF8String s;
 	PopulateDialogData();
 	// Button 0 is Start New List, button 2 is resume list
-	gtk_label_set_text( GTK_LABEL(m_wStartNew_label), pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New).utf8_str());
-	gtk_label_set_text( GTK_LABEL(m_wStartSub_label), pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New,s);
+	gtk_label_set_text( GTK_LABEL(m_wStartNew_label), s.utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume,s);
+	gtk_label_set_text( GTK_LABEL(m_wStartSub_label), s.utf8_str());
 }
 
 static void s_destroy_clicked(GtkWidget * /* widget */,
 			      AP_UnixDialog_Lists * dlg)
 {
 	UT_ASSERT(dlg);
+	dlg->setAnswer(AP_Dialog_Lists::a_QUIT);
 	dlg->destroy();
 }
 
@@ -1237,7 +1448,7 @@ void AP_UnixDialog_Lists::loadXPDataIntoLocal(void)
 
 	//
 	// Now set the list type and style
-	List_Type save = getNewListType();
+	FL_ListType save = getNewListType();
 	if(getNewListType() == NOT_A_LIST)
 	{
 		styleChanged(0);
