@@ -29,7 +29,7 @@
 #include "xap_App.h"
 #include "xap_Frame.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Win32FrameImpl.h"
 
 #include "ap_Strings.h"
 #include "ap_Dialog_Id.h"
@@ -40,7 +40,7 @@
 #include "ie_imp.h"
 #include "ie_types.h"
 #include "ut_string_class.h"
-
+#include "xap_Win32DialogHelper.h"
 #include "ap_Win32Resources.rc2"
 
 /*****************************************************************/
@@ -69,7 +69,7 @@ AP_Win32Dialog_New::~AP_Win32Dialog_New(void)
 void AP_Win32Dialog_New::runModal(XAP_Frame * pFrame)
 {
 
-	UT_ASSERT(pFrame);
+	UT_ASSERT_HARMLESS(pFrame);
 	m_pFrame = pFrame;
 
 	_win32Dialog.runModal( pFrame, 
@@ -86,7 +86,7 @@ BOOL AP_Win32Dialog_New::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	m_hThisDlg = hWnd;
 
 	XAP_Win32App * app = static_cast<XAP_Win32App *> (m_pApp);
-	UT_ASSERT(app);
+	UT_return_val_if_fail (app,1);
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
@@ -95,7 +95,6 @@ BOOL AP_Win32Dialog_New::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	// localize controls
 	_DSX(NEW_BTN_OK,		DLG_OK);
 	_DSX(NEW_BTN_CANCEL,	DLG_Cancel);
-	_DS(NEW_RDO_BLANK,		DLG_NEW_StartEmpty);
 	_DS(NEW_RDO_TEMPLATE,	DLG_NEW_Create);
 	_DS(NEW_RDO_EXISTING,	DLG_NEW_Open);
     _DS(NEW_BTN_EXISTING,	DLG_NEW_Choose);
@@ -145,6 +144,7 @@ BOOL AP_Win32Dialog_New::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	}
 	_findclose( findtag );
 
+	XAP_Win32DialogHelper::s_centerDialog(hWnd);	
 	_updateControls();
 	return 1;	// 1 == we did not call SetFocus()
 }
@@ -182,11 +182,6 @@ BOOL AP_Win32Dialog_New::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		_doChoose();
 		return 1;
 
-	case AP_RID_DIALOG_NEW_RDO_BLANK:
-		setOpenType(AP_Dialog_New::open_New);
-		_updateControls();
-		return 1;
-
 	case AP_RID_DIALOG_NEW_RDO_TEMPLATE:
 		setOpenType(AP_Dialog_New::open_Template);
 		{
@@ -228,7 +223,7 @@ void AP_Win32Dialog_New::_doChoose()
 
 	XAP_Dialog_FileOpenSaveAs * pDialog
 		= (XAP_Dialog_FileOpenSaveAs *)(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 	pDialog->setCurrentPathname(0);
 	pDialog->setSuggestFilename(false);
@@ -279,7 +274,6 @@ void AP_Win32Dialog_New::_updateControls()
 		_win32Dialog.enableControl( AP_RID_DIALOG_NEW_LBX_TEMPLATE, false );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_EXISTING, false );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_TEMPLATE, false );
-		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_BLANK, true );
 		break;
 	case AP_Dialog_New::open_Template:
 		_win32Dialog.enableControl( AP_RID_DIALOG_NEW_EBX_EXISTING, false );
@@ -287,7 +281,6 @@ void AP_Win32Dialog_New::_updateControls()
 		_win32Dialog.enableControl( AP_RID_DIALOG_NEW_LBX_TEMPLATE, true );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_EXISTING, false );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_TEMPLATE, true );
-		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_BLANK, false );
 		break;
 	case AP_Dialog_New::open_Existing:
 		_win32Dialog.enableControl( AP_RID_DIALOG_NEW_EBX_EXISTING, true );
@@ -295,7 +288,6 @@ void AP_Win32Dialog_New::_updateControls()
 		_win32Dialog.enableControl( AP_RID_DIALOG_NEW_LBX_TEMPLATE, false );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_EXISTING, true );
 		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_TEMPLATE, false );
-		_win32Dialog.checkButton( AP_RID_DIALOG_NEW_RDO_BLANK, false );
 		break;
 	}
 }
@@ -317,7 +309,7 @@ void AP_Win32Dialog_New::_setFileName( UT_sint32 nIndex )
 			templateName = XAP_App::getApp()->getAbiSuiteLibDir();
 			break;
 		default:
-			UT_ASSERT( UT_SHOULD_NOT_HAPPEN );
+			UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
 			break;
 		}
 		templateName += "\\templates\\";
