@@ -37,7 +37,7 @@ AP_Win32Toolbar_ZoomCombo::AP_Win32Toolbar_ZoomCombo(EV_Toolbar * pToolbar,
 													 XAP_Toolbar_Id id)
 	: EV_Toolbar_Control(pToolbar/*,id*/)
 {
-	UT_ASSERT(id==AP_TOOLBAR_ID_ZOOM);
+	UT_return_if_fail (id==AP_TOOLBAR_ID_ZOOM);
 
 	m_nPixels = 90;	// TODO: do a better calculation
 	m_nLimit = 6;
@@ -56,18 +56,18 @@ bool AP_Win32Toolbar_ZoomCombo::populate(void)
 	m_vecContents.clear();
 
 	// populate the vector
-	m_vecContents.addItem((void*)"200%");
-	m_vecContents.addItem((void*)"150%");
-	m_vecContents.addItem((void*)"100%");
-	m_vecContents.addItem((void*)"75%");
-	m_vecContents.addItem((void*)"50%");
-	m_vecContents.addItem((void*)"25%");
+	m_vecContents.addItem("200%");
+	m_vecContents.addItem("150%");
+	m_vecContents.addItem("100%");
+	m_vecContents.addItem("75%");
+	m_vecContents.addItem("50%");
+	m_vecContents.addItem("25%");
 
 	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
 	
-	m_vecContents.addItem((void *)pSS->getValue(XAP_STRING_ID_TB_Zoom_PageWidth));
-	m_vecContents.addItem((void *)pSS->getValue(XAP_STRING_ID_TB_Zoom_WholePage));
-	m_vecContents.addItem((void *)pSS->getValue(XAP_STRING_ID_TB_Zoom_Percent));
+	m_vecContents.addItem(pSS->getValue(XAP_STRING_ID_TB_Zoom_PageWidth));
+	m_vecContents.addItem(pSS->getValue(XAP_STRING_ID_TB_Zoom_WholePage));
+	m_vecContents.addItem(pSS->getValue(XAP_STRING_ID_TB_Zoom_Percent));
 
 	return true;
 }
