@@ -26,7 +26,7 @@
 #include "ap_Win32TopRuler.h"
 #include "gr_Win32Graphics.h"
 #include "xap_Win32App.h"
-#include "ap_Win32Frame.h"
+#include "ap_Win32FrameImpl.h"
 
 #include "fv_View.h"
 
@@ -35,7 +35,7 @@
 #define GWL(hwnd)		(AP_Win32TopRuler*)GetWindowLong((hwnd), GWL_USERDATA)
 #define SWL(hwnd, f)	(AP_Win32TopRuler*)SetWindowLong((hwnd), GWL_USERDATA,(LONG)(f))
 
-#define ENSUREP(p)		do { UT_ASSERT(p); if (!p) goto Cleanup; } while (0)
+#define ENSUREP(p)		do { UT_ASSERT_HARMLESS(p); if (!p) goto Cleanup; } while (0)
 
 static char s_TopRulerWndClassName[256];
 
@@ -50,7 +50,14 @@ AP_Win32TopRuler::AP_Win32TopRuler(XAP_Frame * pFrame)
 
 AP_Win32TopRuler::~AP_Win32TopRuler(void)
 {
+	reinterpret_cast<FV_View *>(getView())->setTopRuler(NULL);
 	DELETEP(m_pG);
+
+	if (m_hwndTopRuler)
+	{
+		if (IsWindow(m_hwndTopRuler))
+			DestroyWindow(m_hwndTopRuler);
+	}
 }
 
 void AP_Win32TopRuler::setView(AV_View * pView)
@@ -58,11 +65,15 @@ void AP_Win32TopRuler::setView(AV_View * pView)
 	AP_TopRuler::setView(pView);
 
 	DELETEP(m_pG);
-	GR_Win32Graphics * pG = new GR_Win32Graphics(GetDC(m_hwndTopRuler), m_hwndTopRuler, m_pFrame->getApp());
-	m_pG = pG;
-	UT_ASSERT(m_pG);
+	GR_Win32AllocInfo ai(GetDC(m_hwndTopRuler), m_hwndTopRuler, m_pFrame->getApp());
+	GR_Win32Graphics * pG = (GR_Win32Graphics *)XAP_App::getApp()->newGraphics(ai);
 
+	m_pG = pG;
+	UT_return_if_fail (m_pG);		
 	pG->init3dColors();
+
+	if (IsWindow(m_hwndTopRuler))
+		SendMessage(m_hwndTopRuler, WM_ERASEBKGND, (WPARAM)GetDC(m_hwndTopRuler), 0);
 }
 
 /*****************************************************************/
@@ -90,7 +101,7 @@ bool AP_Win32TopRuler::RegisterClass(XAP_Win32App * app)
 	wndclass.hIconSm       = NULL;
 
 	a = RegisterClassEx(&wndclass);
-	UT_ASSERT(a);
+	UT_ASSERT_HARMLESS(a);
 
 	return true;
 }
@@ -99,19 +110,32 @@ HWND AP_Win32TopRuler::createWindow(HWND hwndContainer,
 									UT_uint32 left, UT_uint32 top,
 									UT_uint32 width)
 {
+
+	UT_DEBUGMSG(("Jordi->AP_Win32TopRuler::createWindow->getGraphics() %x\n", getGraphics()));
+	
+	
+
 	XAP_Win32App * app = static_cast<XAP_Win32App *>(m_pFrame->getApp());
 	m_hwndTopRuler = CreateWindowEx(0, s_TopRulerWndClassName, NULL,
 									WS_CHILD | WS_VISIBLE,
 									left, top, width, s_iFixedHeight,
 									hwndContainer, NULL, app->getInstance(), NULL);
-	UT_ASSERT(m_hwndTopRuler);
+	UT_return_val_if_fail (m_hwndTopRuler,0);
 	SWL(m_hwndTopRuler, this);
 
+	
+	DELETEP(m_pG);
+	GR_Win32AllocInfo ai(GetDC(m_hwndTopRuler), m_hwndTopRuler, m_pFrame->getApp());
+	GR_Win32Graphics * pG = (GR_Win32Graphics *)XAP_App::getApp()->newGraphics(ai);
+	m_pG = pG;
+	UT_return_val_if_fail (m_pG, 0);
+	pG->init3dColors();
+
 	RECT rSize;
 	GetClientRect(m_hwndTopRuler,&rSize);
 	setHeight(rSize.bottom);
 	setWidth(rSize.right);
-
+	
 	return m_hwndTopRuler;
 }
 
@@ -137,67 +161,67 @@ LRESULT CALLBACK AP_Win32TopRuler::_TopRulerWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	if (!pRuler)
 		return DefWindowProc(hwnd, iMsg, wParam, lParam);
+
+	GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
 		
 	switch (iMsg)
 	{
 	case WM_LBUTTONDOWN:
 		SetCapture(hwnd);
-		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON1,LOWORD(lParam),HIWORD(lParam));
-		{
-			GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
+		
+		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON1,pG->tlu(LOWORD(lParam)),pG->tlu(HIWORD(lParam)));
+		{				
 			pG->handleSetCursorMessage();
 		}
 		return 0;
 		
 	case WM_MBUTTONDOWN:
-		SetCapture(hwnd);
-		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON2,LOWORD(lParam),HIWORD(lParam));
-		{
-			GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
+		SetCapture(hwnd);		
+		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON2,pG->tlu(LOWORD(lParam)),pG->tlu(HIWORD(lParam)));
+		{				
 			pG->handleSetCursorMessage();
 		}
 		return 0;
 		
-	case WM_RBUTTONDOWN:
+	case WM_RBUTTONDOWN:		
 		SetCapture(hwnd);
-		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON3,LOWORD(lParam),HIWORD(lParam));
-		{
-			GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
+		pRuler->mousePress(s_GetEMS(wParam),EV_EMB_BUTTON3,
+			pG->tlu(LOWORD(lParam)),pG->tlu(HIWORD(lParam)));
+		{			
 			pG->handleSetCursorMessage();
 		}
 		return 0;
 		
 	case WM_MOUSEMOVE:
-		pRuler->mouseMotion(s_GetEMS(wParam),signedLoWord(lParam),signedHiWord(lParam));
-		pRuler->isMouseOverTab(signedLoWord(lParam),signedHiWord(lParam));
+		{
+			// HACK for not updating Ruler for incremental Loading
+			FV_View * pView = (FV_View *) pRuler->getView();
+			if(pView && (pView->getPoint() == 0))
+			{
+				return 0;
+			}
+		}		
+		pRuler->mouseMotion(s_GetEMS(wParam),pG->tlu(signedLoWord(lParam)),pG->tlu(signedHiWord(lParam)));
+		pRuler->isMouseOverTab(pG->tlu(signedLoWord(lParam)),pG->tlu(signedHiWord(lParam)));
 		return 0;
 
-	case WM_LBUTTONUP:
-		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON1,signedLoWord(lParam),signedHiWord(lParam));
+	case WM_LBUTTONUP:		
+		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON1,pG->tlu(signedLoWord(lParam)),pG->tlu(signedHiWord(lParam)));
 		ReleaseCapture();
-		return 0;
+		return 0;		 
 
-	case WM_MBUTTONUP:
-		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON2,signedLoWord(lParam),signedHiWord(lParam));
+	case WM_MBUTTONUP:		
+		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON2,pG->tlu(signedLoWord(lParam)),pG->tlu(signedHiWord(lParam)));
 		ReleaseCapture();
 		return 0;
 		
-	case WM_RBUTTONUP:
-		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON3,signedLoWord(lParam),signedHiWord(lParam));
+	case WM_RBUTTONUP:		
+		pRuler->mouseRelease(s_GetEMS(wParam),EV_EMB_BUTTON3,pG->tlu(signedLoWord(lParam)),pG->tlu(signedHiWord(lParam)));
 		ReleaseCapture();
 		return 0;
-
-	case WM_SIZE:
-		{
-			int nWidth = LOWORD(lParam);
-			int nHeight = HIWORD(lParam);
-			pRuler->setHeight(nHeight);
-			pRuler->setWidth(nWidth);
-			return 0;
-		}
 	
 	case WM_PAINT:
-		{
+		{	
 			// HACK for not updating Ruler for incremental Loading
 			FV_View * pView = (FV_View *) pRuler->getView();
 			if(pView && (pView->getPoint() == 0))
@@ -206,9 +230,12 @@ LRESULT CALLBACK AP_Win32TopRuler::_TopRulerWndProc(HWND hwnd, UINT iMsg, WPARAM
 			}
 			PAINTSTRUCT ps;
 			HDC hdc = BeginPaint(hwnd, &ps);
-			UT_Rect r(ps.rcPaint.left,ps.rcPaint.top,
-					  ps.rcPaint.right-ps.rcPaint.left,
-					  ps.rcPaint.bottom-ps.rcPaint.top);
+
+			UT_Rect r(pG->tlu(ps.rcPaint.left),
+					  pG->tlu(ps.rcPaint.top),
+					  pG->tlu(ps.rcPaint.right-ps.rcPaint.left+1),
+					  pG->tlu(ps.rcPaint.bottom-ps.rcPaint.top+1));
+
 			pRuler->draw(&r);
 			EndPaint(hwnd,&ps);
 			return 0;
@@ -216,14 +243,12 @@ LRESULT CALLBACK AP_Win32TopRuler::_TopRulerWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	case WM_SYSCOLORCHANGE:
 		{
-			GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
 			pG->init3dColors();
 			return 0;
 		}
 
 	case WM_SETCURSOR:
 		{
-			GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
 			pG->handleSetCursorMessage();
 			return 0;
 		}
