@@ -1,19 +1,19 @@
 /* AbiWord
- * Copyright (C) 1998-2001 AbiSource, Inc.
- * 
+ * Copyright (C) 1998-2000 AbiSource, Inc.
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -31,7 +31,7 @@
 // The following are the set of application-dependent preference keys and the
 // set of default values for them.  Each item must have the AP_PREF_KEY_ prefix
 // and each value must have the XAP_PREF_DEFAULT_ prefix.  Default values *must* obey
-// XML encoding rules if they contain any double quote (&quot;), ampersand (&amp;), 
+// XML encoding rules if they contain any double quote (&quot;), ampersand (&amp;),
 // or angle bracket (&lt; and &gt;) characters.
 //
 // ***FOR EACH PAIR DEFINED, ADD A 'dcl(basename)' TO THE BOTTOM HALF OF THIS FILE***
@@ -55,12 +55,6 @@
 #define AP_PREF_KEY_SpellCheckInternet				"SpellCheckInternet"		/* enable spell checking internet names {0,1} */
 #define AP_PREF_DEFAULT_SpellCheckInternet			"1"
 
-#define AP_PREF_KEY_SpellCheckIgnoredWordsSave		"SpellCheckIgnoredWordsSave"	/* save ignored words list with doc */
-#define AP_PREF_DEFAULT_SpellCheckIgnoredWordsSave	"1"
-
-#define AP_PREF_KEY_SpellCheckIgnoredWordsLoad		"SpellCheckIgnoredWordsLoad"	/* load ignored words list with doc */
-#define AP_PREF_DEFAULT_SpellCheckIgnoredWordsLoad	"1"
-
 #define AP_PREF_KEY_SpellAutoReplace                "SpellAutoReplace" /* automatically substitute "incorrect" words with replacements */
 #define AP_PREF_DEFAULT_SpellAutoReplace            "0"
 
@@ -79,6 +73,8 @@
 #define AP_PREF_DEFAULT_FormatBarVisible			"1"
 #define AP_PREF_KEY_ExtraBarVisible					"ExtraBarVisible"			/* is the extra toolbar visible? {0,1} */
 #define AP_PREF_DEFAULT_ExtraBarVisible				"0"
+#define AP_PREF_KEY_TableBarVisible					"TableBarVisible"			/* is the table toolbar visible? {0,1} */
+#define AP_PREF_DEFAULT_TableBarVisible				"0"
 
 #define AP_PREF_KEY_StatusBarVisible				"StatusBarVisible"			/* is the status bar visible? {0,1} */
 #define AP_PREF_DEFAULT_StatusBarVisible			"1"
@@ -105,14 +101,9 @@
 #define AP_PREF_KEY_MenuLayout						"MenuLayouts"
 #define AP_PREF_DEFAULT_MenuLayout					"Main"						/* value in BeginLayout() */
 
-#define AP_PREF_KEY_MenuLabelSet					"MenuLabelSet"
-#define AP_PREF_DEFAULT_MenuLabelSet				"en-US"						/* value in BeginSet() */
-
-#define AP_PREF_KEY_ToolbarLabelSet					"ToolbarLabelSet"
-#define AP_PREF_DEFAULT_ToolbarLabelSet				"en-US"						/* value in BeginSet() */
 
 #define AP_PREF_KEY_ToolbarLayouts					"ToolbarLayouts"
-#define AP_PREF_DEFAULT_ToolbarLayouts				"FileEditOps FormatOps ExtraOps"		/* values in BeginLayout() */
+#define AP_PREF_DEFAULT_ToolbarLayouts				"FileEditOps FormatOps TableOps ExtraOps"		/* values in BeginLayout() */
 
 #define AP_PREF_KEY_SpellDirectory					"SpellCheckDirectory"		/* where we find hash files */
 #define AP_PREF_DEFAULT_SpellDirectory				"dictionary" 				/* if relative, use prefix "getAbiSuiteLibDir()" */
@@ -126,14 +117,12 @@
 #define AP_PREF_KEY_AlwaysPromptEncoding			"AlwaysPromptEncoding"		/* if true, always show encoding dialog when importing/exporting text */
 #define AP_PREF_DEFAULT_AlwaysPromptEncoding		"0"
 
-#ifdef BIDI_ENABLED
 #define AP_PREF_KEY_DefaultDirectionRtl             "DefaultDirectionRtl"       /* the deafault direction of text is rtl */
 #ifndef BIDI_RTL_DOMINANT
 #define AP_PREF_DEFAULT_DefaultDirectionRtl         "0"
 #else
 #define AP_PREF_DEFAULT_DefaultDirectionRtl         "1"
 #endif
-#endif /*BIDI_ENABLED*/
 
 #define AP_PREF_KEY_ShowSplash     "ShowSplash"
 #define AP_PREF_DEFAULT_ShowSplash "1"
@@ -141,13 +130,10 @@
 #define AP_PREF_KEY_DefaultSaveFormat "DefaultSaveFormat"
 #define AP_PREF_DEFAULT_DefaultSaveFormat ".abw"
 #define AP_PREF_KEY_CloseOnLastDoc "CloseOnLastDoc"
-
-#ifdef DEBUG
 #define AP_PREF_DEFAULT_CloseOnLastDoc "0"
-#else
-// TODO: turn on after 0.9.3
-#define AP_PREF_DEFAULT_CloseOnLastDoc "1"
-#endif
+
+#define AP_PREF_KEY_LockStyles "LockStyles"
+#define AP_PREF_DEFAULT_LockStyles "0"
 
 #else /* AP_PREFS_SCHEMEIDS_H */
 #ifdef dcl
@@ -159,8 +145,6 @@ dcl(AutoSpellCheck)
 dcl(SpellCheckCaps)
 dcl(SpellCheckNumbers)
 dcl(SpellCheckInternet)
-dcl(SpellCheckIgnoredWordsSave)
-dcl(SpellCheckIgnoredWordsLoad)
 dcl(SpellAutoReplace)
 dcl(OptionsTabNumber)
 dcl(RulerUnits)
@@ -168,6 +152,7 @@ dcl(RulerVisible)
 dcl(StandardBarVisible)
 dcl(FormatBarVisible)
 dcl(ExtraBarVisible)
+dcl(TableBarVisible)
 dcl(StatusBarVisible)
 dcl(ParaVisible)
 dcl(StringSet)
@@ -176,8 +161,6 @@ dcl(KeyBindingsCycle)
 dcl(InsertMode)
 dcl(InsertModeToggle)
 dcl(MenuLayout)
-dcl(MenuLabelSet)
-dcl(ToolbarLabelSet)
 dcl(ToolbarLayouts)
 dcl(LayoutMode)
 dcl(AlwaysPromptEncoding)
@@ -186,9 +169,8 @@ dcl(SpellDirectory)
 dcl(StringSetDirectory)
 dcl(ShowSplash)
 
-#ifdef BIDI_ENABLED
 dcl(DefaultDirectionRtl)
-#endif
-	
+dcl(LockStyles)
+
 #endif /* dcl */
 #endif /* AP_PREFS_SCHEMEIDS_H */
