@@ -1,54 +1,85 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
-#include <locale.h>
+#include <time.h>
 
+#include "ut_locale.h"
 #include "ut_string.h"
 #include "ut_types.h"
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
 #include "ut_debugmsg.h"
-#include "pt_Types.h"
 #include "ut_set.h"
-#include "ie_exp_AbiWord_1.h"
+#include "ut_string_class.h"
+#include "ut_uuid.h"
+
+#ifdef ENABLE_RESOURCE_MANAGER
+#include "xap_ResourceManager.h"
+#endif
+
+#include "xap_App.h"
+
+#include "pt_Types.h"
+
 #include "pd_Document.h"
+#include "pd_Style.h"
+
 #include "pp_AttrProp.h"
+
 #include "px_ChangeRecord.h"
 #include "px_CR_Object.h"
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
-#include "xap_App.h"
-#include "ap_Prefs.h"
-#include "pd_Style.h"
+
 #include "fd_Field.h"
-#include "xap_EncodingManager.h"
+
 #include "fl_AutoNum.h"
+
 #include "fp_PageSize.h"
 
-#include "ut_string_class.h"
+#include "ie_exp_AbiWord_1.h"
 
-// our currently used DTD
-#define ABIWORD_FILEFORMAT "fileformat=\"1.0\""
+#include "ap_Prefs.h"
+
+// the fileformat that used to be defined here is now defined at the
+// top of pd_Document.cpp
 
 /*****************************************************************/
 /*****************************************************************/
 
+IE_Exp_AbiWord_1_Sniffer::IE_Exp_AbiWord_1_Sniffer ()
+	: IE_ExpSniffer(IE_IMPEXPNAME_AWML11)
+{
+	// 
+}
+
+UT_Confidence_t IE_Exp_AbiWord_1_Sniffer::supportsMIME (const char * szMIME)
+{
+	if (UT_strcmp (szMIME, IE_MIME_AbiWord) == 0)
+		{
+			return UT_CONFIDENCE_GOOD;
+		}
+	return UT_CONFIDENCE_ZILCH;
+}
+
 bool IE_Exp_AbiWord_1_Sniffer::recognizeSuffix(const char * szSuffix)
 {
 	return (!UT_stricmp(szSuffix,".abw"));
@@ -84,11 +115,15 @@ IE_Exp_AbiWord_1::IE_Exp_AbiWord_1(PD_Document * pDocument, bool isTemplate)
 IE_Exp_AbiWord_1::~IE_Exp_AbiWord_1()
 {
 }
-	  
+
 /*****************************************************************/
 /*****************************************************************/
 
+#ifdef ENABLE_RESOURCE_MANAGER
+class s_AbiWord_1_Listener : public PL_Listener, XAP_ResourceManager::Writer
+#else
 class s_AbiWord_1_Listener : public PL_Listener
+#endif
 {
 public:
 	s_AbiWord_1_Listener(PD_Document * pDocument,
@@ -115,29 +150,46 @@ public:
 
 	virtual bool		signal(UT_uint32 iSignal);
 
+	/* implementation of XAP_[Internal]Resource[Manager]::Writer
+	 */
+	UT_Error write_base64 (void * context, const char * base64, UT_uint32 length, bool final);
+
+	UT_Error write_xml (void * context, const char * name, const char * const * atts);
+	UT_Error write_xml (void * context, const char * name);
+
 protected:
+	void                _closeTable(void);
+	void                _closeCell(void);
 	void				_closeSection(void);
 	void				_closeBlock(void);
 	void				_closeSpan(void);
 	void				_closeField(void);
+	void				_closeHyperlink(void);
 	void				_closeTag(void);
 	void				_openSpan(PT_AttrPropIndex apiSpan);
 	void				_openTag(const char * szPrefix, const char * szSuffix,
-								 bool bNewLineAfter, PT_AttrPropIndex api);
+								 bool bNewLineAfter, PT_AttrPropIndex api,
+								 bool bIgnoreProperties = false);
 	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
+	void				_outputXMLChar(const XML_Char * data, UT_uint32 length);
 	void				_handleStyles(void);
-	void				_handleIgnoredWords(void);
 	void				_handleLists(void);
 	void				_handlePageSize(void);
 	void				_handleDataItems(void);
-	
+    void                _handleMetaData(void);
+	void                _handleRevisions(void);
+	void                _handleHistory(void);
+
 	PD_Document *		m_pDocument;
 	IE_Exp_AbiWord_1 *	m_pie;
 	bool                m_bIsTemplate;
 	bool				m_bInSection;
 	bool				m_bInBlock;
 	bool				m_bInSpan;
-	bool				m_bInTag;
+	//	bool				m_bInTag;
+	bool				m_bInHyperlink;
+	UT_sint32           m_iInTable;
+	UT_sint32           m_iInCell;
 	PT_AttrPropIndex	m_apiLastSpan;
     fd_Field *          m_pCurrentField;
 	bool                m_bOpenChar;
@@ -145,19 +197,40 @@ protected:
 private:
 	UT_Set				m_pUsedImages;
 	const XML_Char*		getObjectKey(const PT_AttrPropIndex& api, const XML_Char* key);
-
 };
 
 void s_AbiWord_1_Listener::_closeSection(void)
 {
 	if (!m_bInSection)
 		return;
-	
+
 	m_pie->write("</section>\n");
 	m_bInSection = false;
 	return;
 }
 
+
+void s_AbiWord_1_Listener::_closeTable(void)
+{
+	if (m_iInTable == 0)
+		return;
+
+	m_pie->write("</table>\n");
+	m_iInTable--;
+	return;
+}
+
+
+void s_AbiWord_1_Listener::_closeCell(void)
+{
+	if (m_iInCell == 0)
+		return;
+
+	m_pie->write("</cell>\n");
+	m_iInCell--;
+	return;
+}
+
 void s_AbiWord_1_Listener::_closeBlock(void)
 {
 	if (!m_bInBlock)
@@ -180,12 +253,8 @@ void s_AbiWord_1_Listener::_closeSpan(void)
 
 void s_AbiWord_1_Listener::_closeTag(void)
 {
-	if (!m_bInTag)
-		return;
-	m_pie->write("</c>");
-	m_bInTag = false;
+	if (m_bOpenChar) m_pie->write("</c>");
 	m_bOpenChar = false;
-	return;
 }
 
 void s_AbiWord_1_Listener::_closeField(void)
@@ -198,6 +267,16 @@ void s_AbiWord_1_Listener::_closeField(void)
 	return;
 }
 
+void s_AbiWord_1_Listener::_closeHyperlink(void)
+{
+	if (!m_bInHyperlink)
+		return;
+    _closeSpan();
+	m_pie->write("</a>");
+    m_bInHyperlink = false;
+	return;
+}
+
 void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 {
 	if (m_bInSpan)
@@ -209,7 +288,7 @@ void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 
 	if (!apiSpan)				// don't write tag for empty A/P
 		return;
-	
+
 	_openTag("c","",false,apiSpan);
 	m_bInSpan = true;
 	m_apiLastSpan = apiSpan;
@@ -217,13 +296,79 @@ void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 }
 
 void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix,
-								   bool bNewLineAfter, PT_AttrPropIndex api)
+								   bool bNewLineAfter, PT_AttrPropIndex api,
+								   bool bIgnoreProperties)
 {
+#ifdef ENABLE_RESOURCE_MANAGER
+	UT_ASSERT_HARMLESS (!m_bOpenChar);
+
+	UT_UTF8String tag("<");
+	tag += szPrefix;
+
+	const PP_AttrProp * pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp (api, &pAP);
+	if (bHaveProp && pAP)
+		{
+			const XML_Char * szName = 0;
+			const XML_Char * szValue = 0;
+
+			UT_uint32 k = 0;
+			while (pAP->getNthAttribute (k++, szName, szValue))
+				{
+					tag += " ";
+					tag += szName;
+					tag += "=\"";
+
+					if ((*szValue == '/') && ((strcmp (szName, "href") == 0) || (strcmp (szName, "xlink:href") == 0)))
+						{
+							XAP_ResourceManager & RM = m_pDocument->resourceManager ();
+							XAP_ExternalResource * re = dynamic_cast<XAP_ExternalResource *>(RM.resource (szValue, false));
+							if (re) tag += re->URL ();
+						}
+					else _outputXMLChar (szValue, strlen (szValue));
+
+					tag += "\"";
+				}
+			if (!bIgnoreProperties)
+				{
+					k = 0;
+					while (pAP->getNthProperty (k++, szName, szValue))
+						{
+							if (k == 1)
+								{
+									tag += " ";
+									tag += PT_PROPS_ATTRIBUTE_NAME;
+									tag += "=\"";
+								}
+							else tag += "; ";
+
+							tag += szName;
+							tag += ":";
+
+							_outputXMLChar (szValue, strlen (szValue));
+						}
+					if (k > 1) tag += "\"";
+				}
+		}
+
+	if (szSuffix)
+		if (*szSuffix == '/')
+			tag += "/";
+	tag += ">";
+	if (bNewLineAfter) tag += "\n";
+
+	m_pie->write (tag.utf8_str (), tag.byteLength());
+
+	if (strcmp (szPrefix, "c") == 0) m_bOpenChar = true;
+
+#else /* ENABLE_RESOURCE_MANAGER */
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	
+	xxx_UT_DEBUGMSG(("_openTag: api %d, bHaveProp %d\n",api, bHaveProp));
 	m_pie->write("<");
-	UT_ASSERT(szPrefix && *szPrefix);
+
+	UT_return_if_fail(szPrefix && *szPrefix);
+
 	if(strcmp(szPrefix,"c")== 0)
 		m_bOpenChar = true;
 	m_pie->write(szPrefix);
@@ -238,21 +383,22 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 			// TODO we force double-quotes on all values.
 			// TODO consider scanning the value to see if it has one
 			// TODO in it and escaping it or using single-quotes.
-			
+			// Let's also escape ampersands and other goodies.
+
 			m_pie->write(" ");
-			m_pie->write((char*)szName);
+			m_pie->write(static_cast<const char*>(szName));
 			m_pie->write("=\"");
-			m_pie->write((char*)szValue);
+			_outputXMLChar(szValue, strlen(szValue));
 			m_pie->write("\"");
 		}
-		if (pAP->getNthProperty(0,szName,szValue))
+		if (!bIgnoreProperties && pAP->getNthProperty(0,szName,szValue))
 		{
 			m_pie->write(" ");
-			m_pie->write((char*)PT_PROPS_ATTRIBUTE_NAME);
+			m_pie->write(static_cast<const char*>(PT_PROPS_ATTRIBUTE_NAME));
 			m_pie->write("=\"");
-			m_pie->write((char*)szName);
+			m_pie->write(static_cast<const char*>(szName));
 			m_pie->write(":");
-			m_pie->write((char*)szValue);
+			_outputXMLChar(szValue, strlen(szValue));
 			UT_uint32 j = 1;
 			while (pAP->getNthProperty(j++,szName,szValue))
 			{
@@ -262,9 +408,9 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 				if (*szValue)
 				{
 					m_pie->write("; ");
-					m_pie->write((char*)szName);
+					m_pie->write(static_cast<const char*>(szName));
 					m_pie->write(":");
-					m_pie->write((char*)szValue);
+					_outputXMLChar(szValue, strlen(szValue));
 				}
 			}
 			m_pie->write("\"");
@@ -277,15 +423,17 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 	if (bNewLineAfter)
 		m_pie->write("\n");
 
-	m_bInTag = true;
+	//	m_bInTag = true;
+#endif /* ENABLE_RESOURCE_MANAGER */
 }
 
-void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
+// This method is very much like _outputData but uses XML_Chars instead of UT_UCS4_Char's.
+void s_AbiWord_1_Listener::_outputXMLChar(const XML_Char * data, UT_uint32 length)
 {
 	UT_String sBuf;
-	const UT_UCSChar * pData;
+	const XML_Char * pData;
 
-	UT_ASSERT(sizeof(UT_Byte) == sizeof(char));
+	UT_return_if_fail(sizeof(UT_Byte) == sizeof(char));
 
 	for (pData=data; (pData<data+length); /**/)
 	{
@@ -295,12 +443,47 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 			sBuf += "&lt;";
 			pData++;
 			break;
-			
+
 		case '>':
 			sBuf += "&gt;";
 			pData++;
 			break;
-			
+
+		case '&':
+			sBuf += "&amp;";
+			pData++;
+			break;
+
+		default:
+			sBuf += (char)*pData++;
+			break;
+		}
+	}
+
+	m_pie->write(sBuf.c_str(),sBuf.size());
+}
+
+void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
+{
+	UT_UTF8String sBuf;
+	const UT_UCSChar * pData;
+
+	UT_return_if_fail(sizeof(UT_Byte) == sizeof(char));
+
+	for (pData=data; (pData<data+length); /**/)
+	{
+		switch (*pData)
+		{
+		case '<':
+			sBuf += "&lt;";
+			pData++;
+			break;
+
+		case '>':
+			sBuf += "&gt;";
+			pData++;
+			break;
+
 		case '&':
 			sBuf += "&amp;";
 			pData++;
@@ -310,67 +493,34 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 			sBuf += "<br/>";
 			pData++;
 			break;
-			
+
 		case UCS_VTAB:					// VTAB -- representing a Forced-Column-Break
 			sBuf += "<cbr/>";
 			pData++;
 			break;
-			
+
+		case UCS_TAB:
+			sBuf += "\t";
+			pData++;
+			break;
+
 		case UCS_FF:					// FF -- representing a Forced-Page-Break
 			sBuf += "<pbr/>";
 			pData++;
 			break;
-			
-		default:
-			if (*pData > 0x007f)
-			{
-				if(XAP_EncodingManager::get_instance()->isUnicodeLocale() || 
-				   (XAP_EncodingManager::get_instance()->try_nativeToU(0xa1) == 0xa1))
 
-				{
-					XML_Char * pszUTF8 = UT_encodeUTF8char(*pData++);
-					while (*pszUTF8)
-					{
-						sBuf += (char)*pszUTF8;
-						pszUTF8++;
-					}
-				}
-				else
-				{
-					/*
-					Try to convert to native encoding and if
-					character fits into byte, output raw byte. This 
-					is somewhat essential for single-byte non-latin
-					languages like russian or polish - since
-					tools like grep and sed can be used then for
-					these files without any problem.
-					Networks and mail transfers are 8bit clean
-					these days.  - VH
-					*/
-					UT_UCSChar c = XAP_EncodingManager::get_instance()->try_UToNative(*pData);
-					if (c==0 || c>255)
-					{
-						char localBuf[20];
-						char * plocal = localBuf;
-						sprintf(localBuf,"&#x%x;",*pData++);
-						sBuf += plocal;
-					}
-					else
-					{
-						sBuf += (char)c;
-						pData++;
-					}
-				}
-			}
+		default:
+			if (*pData < 0x20)         // Silently eat these characters.
+				pData++;
 			else
-			{
-				sBuf += (char)*pData++;
-			}
-			break;
+				{
+					sBuf.appendUCS4 (pData, 1);
+					pData++;
+				}
 		}
 	}
 
-	m_pie->write(sBuf.c_str(),sBuf.size());
+	m_pie->write(sBuf.utf8_str(),sBuf.byteLength());
 }
 
 s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
@@ -384,67 +534,50 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	m_bInSection = false;
 	m_bInBlock = false;
 	m_bInSpan = false;
-	m_bInTag = false;
+	//	m_bInTag = false;
+	m_bInHyperlink = false;
 	m_bOpenChar = false;
 	m_apiLastSpan = 0;
 	m_pCurrentField = 0;
+	m_iInTable = 0;
+	m_iInCell = 0;
 
-	// Be nice to XML apps.  See the notes in _outputData() for more 
-	// details on the charset used in our documents.  By not declaring 
-	// any encoding, XML assumes we're using UTF-8.  Note that US-ASCII 
-	// is a strict subset of UTF-8. 
-
-	if (!XAP_EncodingManager::get_instance()->cjk_locale() &&
-	    (XAP_EncodingManager::get_instance()->try_nativeToU(0xa1) != 0xa1)) {
-	    // use utf8 for CJK locales and latin1 locales and unicode locales
-	    m_pie->write("<?xml version=\"1.0\" encoding=\"");
-	    m_pie->write(XAP_EncodingManager::get_instance()->getNativeEncodingName());
-	    m_pie->write("\"?>\n");
-	} else {
-	    m_pie->write("<?xml version=\"1.0\"?>\n");
-	}
-
-	// We write this first so that the sniffer can detect AbiWord 
-	// documents more easily.   
-
-	// TODO: write out a DOCTYPE description after we update the DTD
+	m_pie->write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	m_pie->write ("<!DOCTYPE abiword PUBLIC \"-//ABISOURCE//DTD AWML 1.0 Strict//EN\" \"http://www.abisource.com/awml.dtd\">\n");
 
-	m_pie->write("<abiword xmlns:awml=\"http://www.abisource.com/awml.dtd\"");
-	m_pie->write(" version=\"");
-	if (XAP_App::s_szBuild_Version && XAP_App::s_szBuild_Version[0])
-	{
-		m_pie->write(XAP_App::s_szBuild_Version);
-	}
-	m_pie->write("\" ");
-	m_pie->write(ABIWORD_FILEFORMAT);
+	/***********************************************************************************
 
-	if (m_bIsTemplate) {
-		m_pie->write (" template=\"true\"");
-	}
+        The fixed attributes of the <abiword> token that used to reside
+        here are now found in PD_Document::setAttrProp()
 
-	m_pie->write(">\n");
+	************************************************************************************/
 
-	// TODO add a file-format name/value pair to this tag.
+	const XML_Char *attr[3];
+	attr[0] = "template";
+	attr[1] = m_bIsTemplate ? "true" : "false";
+	attr[2] = NULL;
+	
+	pDocument->setAttributes(attr);
 
+	_openTag("abiword", NULL, true, pDocument->getAttrPropIndex(),false);
 
 	// NOTE we output the following preamble in XML comments.
 	// NOTE this information is for human viewing only.
-	// TODO should this preamble have a DTD reference in it ??
-	
-	m_pie->write("<!-- =====================================================================  -->\n");
-	m_pie->write("<!-- This file is an AbiWord document.                                      -->\n");
-	m_pie->write("<!-- AbiWord is a free, Open Source word processor.                         -->\n");
-	m_pie->write("<!-- You may obtain more information about AbiWord at www.abisource.com     -->\n");
-	m_pie->write("<!-- You should not edit this file by hand.                                 -->\n");
-	m_pie->write("<!-- =====================================================================  -->\n");
-	m_pie->write("\n");
+
+	m_pie->write("<!-- ======================================================================== -->\n");
+	m_pie->write("<!-- This file is an AbiWord document.                                        -->\n");
+	m_pie->write("<!-- AbiWord is a free, Open Source word processor.                           -->\n");
+	m_pie->write("<!-- More information about AbiWord is available at http://www.abisource.com/ -->\n");
+	m_pie->write("<!-- You should not edit this file by hand.                                   -->\n");
+	m_pie->write("<!-- ======================================================================== -->\n\n");
 
 	// end of preamble.
 	// now we begin the actual document.
-		
+
+	_handleMetaData();
+	_handleHistory();
+	_handleRevisions();
 	_handleStyles();
-	_handleIgnoredWords();
 	_handleLists();
 	_handlePageSize();
 }
@@ -453,10 +586,11 @@ s_AbiWord_1_Listener::~s_AbiWord_1_Listener()
 {
 	_closeSpan();
 	_closeField();
+	_closeHyperlink();
 	_closeBlock();
 	_closeSection();
 	_handleDataItems();
-	
+
 	m_pie->write("</abiword>\n");
 }
 
@@ -491,7 +625,7 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
             }
 			PT_AttrPropIndex api = pcr->getIndexAP();
 			_openSpan(api);
-			
+
 			PT_BufIndex bi = pcrs->getBufIndex();
 			_outputData(m_pDocument->getPointer(bi),pcrs->getLength());
 
@@ -508,11 +642,11 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				{
 				_closeSpan();
                 _closeField();
-
-				const XML_Char* image_name = getObjectKey(api, (const XML_Char*) "dataid");
+#ifndef ENABLE_RESOURCE_MANAGER
+				const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>("dataid"));
 				if (image_name)
 					m_pUsedImages.insert(image_name);
-
+#endif
 				_openTag("image","/",false,api);
 
 				return true;
@@ -522,25 +656,72 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
                     _closeSpan();
                     _closeField();
                     _openTag("field","",false,api);
-                    m_pCurrentField = pcro->getField(); 
-                    UT_ASSERT(m_pCurrentField);
+                    m_pCurrentField = pcro->getField();
+                    UT_ASSERT_HARMLESS(m_pCurrentField);
                     return true;
                 }
+   			case PTO_Bookmark:
+   				{
+   					_closeSpan();
+   					_closeField();
+   					_openTag("bookmark", "/",false, api,true);
+   					return true;
+   				}
+
+   			case PTO_Hyperlink:
+   				{
+   					_closeSpan();
+   					_closeField();
+					const PP_AttrProp * pAP = NULL;
+					m_pDocument->getAttrProp(api,&pAP);
+					const XML_Char * pName;
+					const XML_Char * pValue;
+					bool bFound = false;
+					UT_uint32 k = 0;
+
+					while(pAP->getNthAttribute(k++, pName, pValue))
+					{
+						bFound = (0 == UT_XML_strnicmp(pName,"xlink:href",10));
+						if(bFound)
+							break;
+					}
+
+					if(bFound)
+					{
+						//this is the start of the hyperlink
+   						_openTag("a", "",false, api,true);
+   						m_bInHyperlink = true;
+   					}
+   					else
+   					{
+   						_closeHyperlink();
+   					}
+
+
+   					return true;
+
+   				}
+
+
 			default:
-				UT_ASSERT(0);
+				UT_ASSERT_NOT_REACHED();
 				return false;
 			}
 		}
 
 	case PX_ChangeRecord::PXT_InsertFmtMark:
+#if 0
+		// fmt marks are temporary placeholders for props and
+		// attributes and should not be saved
 		if(m_bOpenChar)
 			_closeTag();
 		_openTag("c","",false,pcr->getIndexAP());
 		_closeTag();
+#endif
 		return true;
-		
+
 	default:
-		UT_ASSERT(0);
+	  UT_ASSERT_NOT_REACHED();
 		return false;
 	}
 }
@@ -549,29 +730,165 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 										   const PX_ChangeRecord * pcr,
 										   PL_StruxFmtHandle * psfh)
 {
-	UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
+	UT_return_val_if_fail(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux, false);
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
 	*psfh = 0;							// we don't need it.
+#ifndef ENABLE_RESOURCE_MANAGER
+	PT_AttrPropIndex api = pcr->getIndexAP();
+	const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>(PT_STRUX_IMAGE_DATAID));
+	if (image_name)
+		m_pUsedImages.insert(image_name);
+#endif
 
 	switch (pcrx->getStruxType())
 	{
 	case PTX_Section:
 	case PTX_SectionHdrFtr:
-	case PTX_SectionEndnote:
 		{
 			_closeSpan();
             _closeField();
+            _closeHyperlink();
 			_closeBlock();
 			_closeSection();
 			_openTag("section","",true,pcr->getIndexAP());
 			m_bInSection = true;
 			return true;
 		}
-
+	case PTX_SectionTable:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_openTag("table","",true,pcr->getIndexAP());
+			m_iInTable++;
+			return true;
+		}
+	case PTX_SectionCell:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_openTag("cell","",true,pcr->getIndexAP());
+			m_iInCell++;
+			return true;
+		}
+	case PTX_SectionFootnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			m_bInBlock = false;
+			_openTag("foot","",true,pcr->getIndexAP());
+			return true;
+		}
+	case PTX_SectionEndnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			m_bInBlock = false;
+			_openTag("endnote","",true,pcr->getIndexAP());
+			return true;
+		}
+	case PTX_SectionTOC:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_bInBlock = false;
+			_openTag("toc","",true,pcr->getIndexAP());
+			return true;
+		}
+	case PTX_SectionMarginnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_openTag("margin","",true,pcr->getIndexAP());
+			return true;
+		}
+	case PTX_SectionFrame:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_openTag("frame","",true,pcr->getIndexAP());
+			return true;
+		}
+	case PTX_EndTable:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_closeTable();
+			return true;
+		}
+	case PTX_EndCell:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			_closeCell();
+			return true;
+		}
+	case PTX_EndFootnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</foot>");
+			m_bInBlock = true;
+			return true;
+		}
+	case PTX_EndEndnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</endnote>");
+			m_bInBlock = true;
+			return true;
+		}
+	case PTX_EndTOC:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</toc>");
+			return true;
+		}
+	case PTX_EndMarginnote:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			return true;
+		}
+	case PTX_EndFrame:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</frame>");
+			return true;
+		}
 	case PTX_Block:
 		{
 			_closeSpan();
             _closeField();
+			_closeHyperlink();
 			_closeBlock();
 			_openTag("p","",false,pcr->getIndexAP());
 			m_bInBlock = true;
@@ -579,7 +896,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 		}
 
 	default:
-		UT_ASSERT(0);
+	  UT_ASSERT_NOT_REACHED();
 		return false;
 	}
 }
@@ -587,7 +904,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 bool s_AbiWord_1_Listener::change(PL_StruxFmtHandle /*sfh*/,
 									const PX_ChangeRecord * /*pcr*/)
 {
-	UT_ASSERT(0);						// this function is not used.
+  UT_ASSERT_NOT_REACHED();
 	return false;
 }
 
@@ -599,16 +916,64 @@ bool s_AbiWord_1_Listener::insertStrux(PL_StruxFmtHandle /*sfh*/,
 																	  PL_ListenerId /* lid */,
 																	  PL_StruxFmtHandle /* sfhNew */))
 {
-	UT_ASSERT(0);						// this function is not used.
+	UT_ASSERT_NOT_REACHED();						// this function is not used.
 	return false;
 }
 
 bool s_AbiWord_1_Listener::signal(UT_uint32 /* iSignal */)
 {
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+  UT_ASSERT_NOT_REACHED();
 	return false;
 }
 
+/* base64-encoded object data
+ */
+UT_Error s_AbiWord_1_Listener::write_base64 (void * /*context*/, const char * base64, UT_uint32 length, bool /*final*/)
+{
+	m_pie->write (base64, length);
+	m_pie->write ("\n");
+
+	return UT_OK;
+}
+
+/* start tag & attributes
+ */
+UT_Error s_AbiWord_1_Listener::write_xml (void * /*context*/, const char * name, const char * const * atts)
+{
+	UT_UTF8String tag(" <");
+
+	tag += name;
+
+	const char * const * attr = atts;
+	while (*attr)
+		{
+			tag += " ";
+			tag += *attr++;
+			tag += "=\"";
+			tag += *attr++;
+			tag += "\"";
+		}
+	tag += ">\n";
+
+	m_pie->write (tag.utf8_str (), tag.byteLength());
+
+	return UT_OK;
+}
+
+/* end tag
+ */
+UT_Error s_AbiWord_1_Listener::write_xml (void * /*context*/, const char * name)
+{
+	UT_UTF8String tag(" </");
+
+	tag += name;
+	tag += ">\n";
+
+	m_pie->write (tag.utf8_str (), tag.byteLength());
+
+	return UT_OK;
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -617,8 +982,18 @@ UT_Error IE_Exp_AbiWord_1::_writeDocument(void)
 	m_pListener = new s_AbiWord_1_Listener(getDoc(),this, m_bIsTemplate);
 	if (!m_pListener)
 		return UT_IE_NOMEMORY;
-	if (!getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)))
-		return UT_ERROR;
+
+	if (getDocRange())
+	{
+		if(!getDoc()->tellListenerSubset(static_cast<PL_Listener *>(m_pListener),getDocRange()))
+			return UT_ERROR;
+	}
+	else
+	{
+		if (!getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)))
+			return UT_ERROR;
+	}
+	
 	delete m_pListener;
 
 	m_pListener = NULL;
@@ -635,12 +1010,12 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 
 	const char * szName = NULL;
 	const PD_Style * pStyle=NULL;
-	UT_Vector vecStyles;
+	UT_GenericVector<PD_Style *> vecStyles;
 	m_pDocument->getAllUsedStyles(&vecStyles);
 	UT_uint32 k = 0;
 	for (k=0; k < vecStyles.getItemCount(); k++)
 	{
-		pStyle = (PD_Style *) vecStyles.getNthItem(k);
+		pStyle = vecStyles.getNthItem(k);
 		if (!bWroteOpenStyleSection)
 		{
 			m_pie->write("<styles>\n");
@@ -653,7 +1028,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 
 	for (k=0; m_pDocument->enumStyles(k, &szName, &pStyle); k++)
 	{
-		if (!pStyle->isUserDefined() || (vecStyles.findItem((void *) pStyle) >= 0))
+		if (!pStyle->isUserDefined() || (vecStyles.findItem(const_cast<PD_Style*>(pStyle))) >= 0)
 			continue;
 
 		if (!bWroteOpenStyleSection)
@@ -672,103 +1047,111 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 	return;
 }
 
-void s_AbiWord_1_Listener::_handleIgnoredWords(void)
-{
-	UT_ASSERT(m_pDocument);
-
-	XAP_App *pApp = m_pDocument->getApp();
-	UT_ASSERT(pApp);
-
-	XAP_Prefs *pPrefs = pApp->getPrefs();
-	UT_ASSERT(pPrefs);
-	
-	bool saveIgnores = false;
-	pPrefs->getPrefsValueBool((XML_Char *)AP_PREF_KEY_SpellCheckIgnoredWordsSave, &saveIgnores);
-
-	bool bWroteIgnoredWords = false;
-
-	if (!saveIgnores) 
-		return;  // don't bother
-
-	const UT_UCSChar *word;
-	for (UT_uint32 i = 0; m_pDocument->enumIgnores(i, &word); i++)
-	{
-
-		if (!bWroteIgnoredWords)
-		{
-			bWroteIgnoredWords = true;
-			m_pie->write("<ignoredwords>\n");
-		}
-		
-		m_pie->write("<iw>");
-		_outputData (word, UT_UCS_strlen (word));
-		m_pie->write("</iw>\n");
-	}
-
-	if (bWroteIgnoredWords)
-		m_pie->write("</ignoredwords>\n");
-
-	return;
-}
-
 void s_AbiWord_1_Listener::_handleLists(void)
 {
 	bool bWroteOpenListSection = false;
-	
+
  	//const char * szID;
 	//const char * szPid;
 	//const char * szProps;
 
-	fl_AutoNum * pAutoNum;
-	const char ** attr;
-
 #define LCheck(str) (0 == UT_strcmp(attr[0], str))
 
+	fl_AutoNum * pAutoNum;
 	for (UT_uint32 k = 0; (m_pDocument->enumLists(k, &pAutoNum )); k++)
-	{	
+	{
+		const char ** attr = NULL, ** attr0 = NULL;
+
 		if (pAutoNum->isEmpty() == true)
 			continue;
-		
+
 		if (!bWroteOpenListSection)
 		{
 			m_pie->write("<lists>\n");
 			bWroteOpenListSection = true;
 		}
 		m_pie->write("<l");
-		for (attr = pAutoNum->getAttributes(); (*attr); attr++)
+		for (attr0 = attr = pAutoNum->getAttributes(); (*attr); attr++)
 		{
 			if (LCheck("id") || LCheck("parentid") || LCheck("type") || LCheck("start-value") || LCheck("list-delim") || LCheck("list-decimal"))
 			{
 				m_pie->write(" ");
 				m_pie->write(attr[0]);
-				m_pie->write("=\""); 
+				m_pie->write("=\"");
 				m_pie->write(attr[1]);
 				m_pie->write("\"");
 			}
-			//attr++;
 		}
 		m_pie->write("/>\n");
+		// No, no, you can't free attr and expect good things to happen.
+		FREEP(attr0);
 	}
 
-#undef LCheck			
-	
+
+#undef LCheck
+
 	if (bWroteOpenListSection)
 		m_pie->write("</lists>\n");
-	
+
 	return;
 }
 
+void s_AbiWord_1_Listener::_handleMetaData(void)
+{
+	if (m_pie->isCopying ())
+		return;
+
+  // set all of the important meta-data props
+
+  m_pDocument->setMetaDataProp ( PD_META_KEY_GENERATOR, UT_UTF8String("AbiWord") ) ;
+  m_pDocument->setMetaDataProp ( PD_META_KEY_FORMAT,    UT_UTF8String(IE_MIME_AbiWord) ) ;
+
+#if 0
+  // get the saved time, remove trailing newline
+  time_t now = time ( NULL ) ;
+  UT_String now_str ( ctime(&now) ) ;
+  now_str = now_str.substr ( 0, now_str.size() -1 ) ;
+  m_pDocument->setMetaDataProp ( PD_META_KEY_DATE_LAST_CHANGED, UT_UTF8String(now_str.c_str()) ) ;
+#endif
+
+  // TODO: set dc.date and abiword.date_created if document is new (i.e. first save)
+
+  UT_GenericStringMap<UT_UTF8String*> & ref = m_pDocument->getMetaData() ;
+
+  // don't print out a thing
+  if ( ref.size () == 0 )
+    return ;
+
+  m_pie->write("<metadata>\n");
+
+  UT_GenericStringMap<UT_UTF8String*>::UT_Cursor cursor ( &ref ) ;
+
+  const UT_UTF8String * val = NULL ;
+  for ( val = cursor.first(); cursor.is_valid(); val = cursor.next () )
+    {
+      if ( val )
+	{
+	  const UT_UTF8String *stringval = val;
+	  if( stringval->size () > 0 )
+	    {
+	      m_pie->write( "<m key=\"" ) ;
+	      _outputXMLChar ( cursor.key().c_str(), cursor.key().size() ) ;
+	      m_pie->write ( "\">" ) ;
+	      _outputXMLChar ( stringval->utf8_str(), stringval->byteLength() ) ;
+	      m_pie->write ( "</m>\n" ) ;
+	    }
+	}
+    }
+
+  m_pie->write("</metadata>\n");
+}
+
 void s_AbiWord_1_Listener::_handlePageSize(void)
 {
   //
   // Code to write out the PageSize Definitions to disk
-  // 
-	char *old_locale;
-
-	char buf[20];
-
-	old_locale = setlocale (LC_NUMERIC, "C");
-        m_pie->write("<pagesize pagetype=\"");
+  //
+	m_pie->write("<pagesize pagetype=\"");
 	m_pie->write(m_pDocument->m_docPageSize.getPredefinedName());
 	m_pie->write("\"");
 
@@ -777,37 +1160,22 @@ void s_AbiWord_1_Listener::_handlePageSize(void)
 	        m_pie->write("portrait\"");
 	else
 	        m_pie->write("landscape\"");
-	m_pie->write( " width=\"");
-	fp_PageSize::Unit  docUnit = m_pDocument->m_docPageSize.getUnit(); 
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Width(docUnit));
-	m_pie->write( (char *) buf);
-	m_pie->write("\"");
-	m_pie->write(" height=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Height(docUnit));
-	m_pie->write( (char *) buf);
-	m_pie->write("\"");
-	m_pie->write(" units=\"");
-	if(docUnit == fp_PageSize::cm)
-	        m_pie->write("cm");
-	else if(docUnit == fp_PageSize::mm)
-	        m_pie->write("mm");
-	else if(docUnit == fp_PageSize::inch)
-	        m_pie->write("inch");
-	else
-	        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	m_pie->write("\"");
-	m_pie->write(" page-scale=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.getScale());
-	m_pie->write( (char *) buf);
-	m_pie->write("\"");
+	UT_Dimension docUnit = m_pDocument->m_docPageSize.getDims();
 
-	m_pie->write("/>\n");
-	setlocale (LC_NUMERIC, old_locale);
-	return;
+	UT_LocaleTransactor(LC_NUMERIC, "C");
+	m_pie->write( UT_String_sprintf(" width=\"%f\"", m_pDocument->m_docPageSize.Width(docUnit)).c_str() );
+	m_pie->write( UT_String_sprintf(" height=\"%f\"",m_pDocument->m_docPageSize.Height(docUnit)).c_str() );
+	m_pie->write(" units=\"");
+	m_pie->write(UT_dimensionName(docUnit));
+	m_pie->write("\"");
+	m_pie->write( UT_String_sprintf(" page-scale=\"%f\"/>\n",m_pDocument->m_docPageSize.getScale()).c_str() );
 }
 
 void s_AbiWord_1_Listener::_handleDataItems(void)
 {
+#ifdef ENABLE_RESOURCE_MANAGER
+	m_pDocument->resourceManager().write_xml (0, *this);
+#else
 	bool bWroteOpenDataSection = false;
 
 	const char * szName;
@@ -817,7 +1185,9 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 	UT_ByteBuf bbEncoded(1024);
 	UT_Set::Iterator end(m_pUsedImages.end());
 
-	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,(void**)&szMimeType)); k++)
+	for (UT_uint32 k=0;
+		 (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,reinterpret_cast<const void**>(&szMimeType)));
+		 k++)
 	{
 		UT_Set::Iterator it(m_pUsedImages.find_if(szName, ut_lexico_equal));
 		if (it == end)
@@ -840,18 +1210,18 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 
 	   	bool status = false;
 	   	bool encoded = true;
-	   
+
 		if (szMimeType && (UT_strcmp(szMimeType, "image/svg-xml") == 0 || UT_strcmp(szMimeType, "text/mathml") == 0))
 	    {
 		   bbEncoded.truncate(0);
-		   bbEncoded.append((UT_Byte*)"<![CDATA[", 9);
+		   bbEncoded.append(reinterpret_cast<const UT_Byte*>("<![CDATA["), 9);
 		   UT_uint32 off = 0;
 		   UT_uint32 len = pByteBuf->getLength();
 		   const UT_Byte * buf = pByteBuf->getPointer(0);
 		   while (off < len) {
 		      if (buf[off] == ']' && buf[off+1] == ']' && buf[off+2] == '>') {
 			 bbEncoded.append(buf, off-1);
-			 bbEncoded.append((UT_Byte*)"]]&gt;", 6);
+			 bbEncoded.append(reinterpret_cast<const UT_Byte*>("]]&gt;"), 6);
 			 off += 3;
 			 len -= off;
 			 buf = pByteBuf->getPointer(off);
@@ -861,11 +1231,11 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 		      off++;
 		   }
 		   bbEncoded.append(buf, off);
-		   bbEncoded.append((UT_Byte*)"]]>\n", 4);
+		   bbEncoded.append(reinterpret_cast<const UT_Byte*>("]]>\n"), 4);
 		   status = true;
 		   encoded = false;
 		}
-	   	else  
+	   	else
 		{
 		   status = UT_Base64Encode(&bbEncoded, pByteBuf);
 		   encoded = true;
@@ -874,13 +1244,16 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 	   	if (status)
 	    {
 	   		m_pie->write("<d name=\"");
-			m_pie->write(szName);
+			// We assume that UT_XML_Char is equivalent to char.
+			// That's not really a good assumption, but, hey.
+			// TODO: make szName, szMimeType be UT_XML_Chars.
+			_outputXMLChar(szName, strlen(szName));
 		   	if (szMimeType)
 			{
 			   m_pie->write("\" mime-type=\"");
-			   m_pie->write(szMimeType);
+			   _outputXMLChar(szMimeType, strlen(szMimeType));
 			}
-		   	if (encoded) 
+		   	if (encoded)
 		    {
 			   	m_pie->write("\" base64=\"yes\">\n");
 				// break up the Base64 blob as a series lines
@@ -891,15 +1264,15 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 				UT_uint32 j;
 				for (j=0; j<jLimit; j+=72)
 				{
-					jSize = MyMin(72,(jLimit-j));
-					m_pie->write((const char *)bbEncoded.getPointer(j),jSize);
+					jSize = UT_MIN(72,(jLimit-j));
+					m_pie->write(reinterpret_cast<const char *>(bbEncoded.getPointer(j)),jSize);
 					m_pie->write("\n");
 				}
 			}
 		   	else
 		    {
 		     	m_pie->write("\" base64=\"no\">\n");
-			   	m_pie->write((const char*)bbEncoded.getPointer(0), bbEncoded.getLength());
+			   	m_pie->write(reinterpret_cast<const char*>(bbEncoded.getPointer(0)), bbEncoded.getLength());
 			}
 			m_pie->write("</d>\n");
 		}
@@ -907,6 +1280,98 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 
 	if (bWroteOpenDataSection)
 		m_pie->write("</data>\n");
+#endif
+}
+
+void s_AbiWord_1_Listener::_handleRevisions(void)
+{
+	bool bWroteOpenRevisionsSection = false;
+
+	const AD_Revision * pRev=NULL;
+
+	const UT_GenericVector<AD_Revision*> & vRevisions = m_pDocument->getRevisions();
+
+	UT_uint32 k = 0;
+	for (k=0; k < vRevisions.getItemCount(); k++)
+	{
+		pRev = vRevisions.getNthItem(k);
+		UT_return_if_fail(pRev);
+		
+		UT_String s;
+		
+		if (!bWroteOpenRevisionsSection)
+		{
+			UT_String_sprintf(s, "<revisions show=\"%d\" mark=\"%d\" show-level=\"%d\" auto=\"%d\">\n",
+							  m_pDocument->isShowRevisions(),
+							  m_pDocument->isMarkRevisions(),
+							  m_pDocument->getShowRevisionId(),
+							  m_pDocument->isAutoRevisioning());
+			
+			m_pie->write(s.c_str());
+			bWroteOpenRevisionsSection = true;
+		}
+
+		UT_String_sprintf(s, "<r id=\"%d\" time-started=\"%d\" version=\"%d\">",
+						  pRev->getId(),
+						  pRev->getStartTime(),
+						  pRev->getVersion());
+		
+		m_pie->write(s.c_str());
+
+		if(pRev->getDescription())
+		{
+			_outputData(pRev->getDescription(), UT_UCS4_strlen(pRev->getDescription()));
+		}
+		
+
+		m_pie->write("</r>\n");
+	}
+
+	if (bWroteOpenRevisionsSection)
+		m_pie->write("</revisions>\n");
+
+	return;
+}
+
+void s_AbiWord_1_Listener::_handleHistory(void)
+{
+	bool bWroteOpenSection = false;
+
+	UT_uint32 k = 0;
+	const UT_uint32 iCount = m_pDocument->getHistoryCount();
+	
+	for (k=0; k < iCount; k++)
+	{
+		UT_uint32 iVersion  =  m_pDocument->getHistoryNthId(k);
+		const UT_UUID& UID  =  m_pDocument->getHistoryNthUID(k);
+		time_t tStarted     =  m_pDocument->getHistoryNthTimeStarted(k);
+		bool bAuto          =  m_pDocument->getHistoryNthAutoRevisioned(k);
+		
+		UT_UTF8String s, hUid;
+		UID.toString(hUid);
+		
+		
+		if (!bWroteOpenSection)
+		{
+			UT_UTF8String_sprintf(s, "<history version=\"%d\" edit-time=\"%d\" last-saved=\"%d\" "
+							     "uid=\"%s\">\n",
+							  m_pDocument->getDocVersion(),
+							  m_pDocument->getEditTime(),
+							  m_pDocument->getLastSavedTime(),
+							  m_pDocument->getDocUUIDString());
+			
+			m_pie->write(s.utf8_str());
+			bWroteOpenSection = true;
+		}
+
+		UT_UTF8String_sprintf(s, "<version id=\"%d\" started=\"%d\" uid=\"%s\" auto=\"%d\"/>\n",
+						  iVersion, tStarted, hUid.utf8_str(),bAuto);
+		
+		m_pie->write(s.utf8_str());
+	}
+
+	if (bWroteOpenSection)
+		m_pie->write("</history>\n");
 
 	return;
 }
