@@ -31,7 +31,7 @@ class IE_Exp_GZipAbiWord_Sniffer : public IE_ExpSniffer
 	friend class IE_Exp;
 
 public:
-	IE_Exp_GZipAbiWord_Sniffer () {}
+	IE_Exp_GZipAbiWord_Sniffer ();
 	virtual ~IE_Exp_GZipAbiWord_Sniffer () {}
 
 	virtual bool recognizeSuffix (const char * szSuffix);
@@ -50,9 +50,9 @@ public:
 	virtual ~IE_Exp_GZipAbiWord();
 
 protected:
-	bool				_openFile(const char * szFilename);
-	UT_uint32			_writeBytes(const UT_Byte * pBytes, UT_uint32 length);
-	bool				_closeFile(void);
+	virtual bool				_openFile(const char * szFilename);
+	virtual UT_uint32			_writeBytes(const UT_Byte * pBytes, UT_uint32 length);
+	virtual bool				_closeFile(void);
 
 private:
     
