@@ -43,7 +43,6 @@
 #ifdef WIN32
   #include "ut_Win32OS.h"
 #endif
-#include <fribidi/fribidi.h>
 
 
 /*****************************************************************/
@@ -97,15 +96,23 @@ IE_Exp_Text::IE_Exp_Text(PD_Document * pDocument, const char * encoding)
     }
 }
 
+IE_Exp_Text::~IE_Exp_Text ()
+{
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
 IE_Exp_Text_Sniffer::IE_Exp_Text_Sniffer ()
-	: IE_ExpSniffer(IE_IMPEXPNAME_TEXT)
+	: IE_ExpSniffer(IE_IMPEXPNAME_TEXT, true)
 {
 	// 
 }
 
+IE_Exp_Text_Sniffer::~IE_Exp_Text_Sniffer ()
+{
+}
+
 UT_Confidence_t IE_Exp_Text_Sniffer::supportsMIME (const char * szMIME)
 {
 	if (UT_strcmp (szMIME, IE_MIME_Text) == 0)
@@ -150,11 +157,15 @@ bool IE_Exp_Text_Sniffer::getDlgLabels(const char ** pszDesc,
 /*****************************************************************/
 
 IE_Exp_EncodedText_Sniffer::IE_Exp_EncodedText_Sniffer ()
-	: IE_ExpSniffer(IE_IMPEXPNAME_TEXTENC)
+	: IE_ExpSniffer(IE_IMPEXPNAME_TEXTENC, false)
 {
 	// 
 }
 
+IE_Exp_EncodedText_Sniffer::~IE_Exp_EncodedText_Sniffer ()
+{
+}
+
 /*!
   Check filename extension for filetypes we support
  \param szSuffix Filename extension
@@ -543,7 +554,7 @@ void Text_Listener::_handleDirMarker(PT_AttrPropIndex api)
 			{
 				UT_DEBUGMSG(("Text_Listener::_handleDirMarker: dir-override value '%s'\n",
 							 szValue));
-				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			}
 		}
 		else
@@ -559,7 +570,7 @@ void Text_Listener::_handleDirMarker(PT_AttrPropIndex api)
 	{
 		UT_DEBUGMSG(("Text_Listener::_handleDirMarker: no props! (bHaveProp %d, pAP 0x%x)\n",
 					 bHaveProp, pAP));
-		UT_ASSERT( UT_SHOULD_NOT_HAPPEN );
+		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
 	}
 
 	// first see if there is a pending marker (for block direction)
@@ -680,25 +691,25 @@ bool Text_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				UT_UCS4Char cRLM = UCS_RLM;
 				UT_UCS4Char cLRM = UCS_LRM;
 
-				FriBidiCharType type = fribidi_get_type(static_cast<FriBidiChar>(*pData));
+				UT_BidiCharType type = UT_bidiGetCharType(*pData);
 				
-				if(m_eDirMarkerPending == DO_RTL && type == FRIBIDI_TYPE_RTL)
+				if(m_eDirMarkerPending == DO_RTL && type == UT_BIDI_RTL)
 				{
 					//the override corresponds to the marker, no marker needed
 					m_eDirMarkerPending = DO_UNSET;
 				}
-				else if(m_eDirMarkerPending == DO_RTL && type == FRIBIDI_TYPE_LTR)
+				else if(m_eDirMarkerPending == DO_RTL && type == UT_BIDI_LTR)
 				{
 					//need to issue marker
 					_outputData(&cRLM, 1);
 					m_eDirMarkerPending = DO_UNSET;
 				}
-				else if(m_eDirMarkerPending == DO_LTR && type == FRIBIDI_TYPE_LTR)
+				else if(m_eDirMarkerPending == DO_LTR && type == UT_BIDI_LTR)
 				{
 					//the override corresponds to the marker, no marker needed
 					m_eDirMarkerPending = DO_UNSET;
 				}
-				else if(m_eDirMarkerPending == DO_LTR && type == FRIBIDI_TYPE_RTL)
+				else if(m_eDirMarkerPending == DO_LTR && type == UT_BIDI_RTL)
 				{
 					//need to issue marker
 					_outputData(&cLRM, 1);
@@ -752,7 +763,7 @@ bool Text_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		return true;
 
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 	}
 }
@@ -761,7 +772,7 @@ bool Text_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 									   const PX_ChangeRecord * pcr,
 									   PL_StruxFmtHandle * psfh)
 {
-	UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
+	UT_return_val_if_fail(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux, false);
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *>(pcr);
 	*psfh = 0;							// we don't need it.
 
@@ -877,11 +888,13 @@ bool Text_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_SectionMarginnote:
 	case PTX_SectionFootnote:
 	case PTX_EndEndnote:
+	case PTX_SectionTOC:
+	case PTX_EndTOC:
 	    return true ;
 
 	default:
 		UT_ASSERT_NOT_REACHED();
-		return false;
+		return true;
 	}
 }
 
@@ -909,3 +922,8 @@ bool Text_Listener::signal(UT_uint32 /* iSignal */)
   UT_ASSERT_NOT_REACHED();
 	return false;
 }
+
+Text_Listener::~Text_Listener ()
+{
+}
+
