@@ -1,5 +1,7 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,25 +24,36 @@
 #define IE_IMP_ABIWORD_1_H
 
 #include <stdio.h>
-#include "ut_vector.h"
-#include "ut_stack.h"
+
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
+#include "ut_types.h"
+#endif
+#include "ut_hash.h"
+
 #include "ie_imp_XML.h"
-#include "ut_bytebuf.h"
+
+
 class PD_Document;
 
 // The importer/reader for AbiWord file format version 1.
 
-class IE_Imp_AbiWord_1_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_AbiWord_1_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 
 public:
-	IE_Imp_AbiWord_1_Sniffer() {}
+	IE_Imp_AbiWord_1_Sniffer();
 	virtual ~IE_Imp_AbiWord_1_Sniffer() {}
 
-	virtual bool recognizeContents (const char * szBuf, 
+	UT_Confidence_t supportsMIME (const char * szMIME);
+
+	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
 									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -49,31 +62,37 @@ public:
 
 };
 
-class IE_Imp_AbiWord_1 : public IE_Imp_XML
+class ABI_EXPORT IE_Imp_AbiWord_1 : public IE_Imp_XML
 {
 public:
     IE_Imp_AbiWord_1(PD_Document * pDocument);
+
     virtual ~IE_Imp_AbiWord_1();
 
-    void				_startElement(const XML_Char *name, const XML_Char **atts);
-    void				_endElement(const XML_Char *name);
+    void				startElement(const XML_Char *name, const XML_Char **atts);
+    void				endElement(const XML_Char *name);
 
     virtual UT_Error	importFile(const char * szFilename);
+	virtual bool        supportsLoadStylesOnly() {return true;}
 
 protected:
     const XML_Char *	_getDataItemName(const XML_Char ** atts);
     const XML_Char *	_getDataItemMimeType(const XML_Char ** atts);
-    bool		_getDataItemEncoded(const XML_Char ** atts);
+    bool				_getDataItemEncoded(const XML_Char ** atts);
+
+    bool				_handleImage(const XML_Char ** atts);
+    bool				_handleResource(const XML_Char ** atts, bool isResource);
 
  private:
-    bool			m_bDocHasLists;
-    bool			m_bDocHasPageSize;
-    UT_uint32		m_iInlineStart;
+    bool				m_bWroteSection;
+    bool				m_bWroteParagraph;
+    bool				m_bDocHasLists;
+    bool				m_bDocHasPageSize;
+
+    UT_uint32			m_iInlineStart;
+
+	UT_GenericStringMap<UT_UTF8String *> *	m_refMap;
+	bool                m_bAutoRevisioning;
 };
 
 #endif /* IE_IMP_ABIWORD_1_H */
-
-
-
-
-
