@@ -20,16 +20,14 @@
 #ifndef ABI_OPT_WIDGET
 
 #include "ap_UnixApp.h"
-#include <unistd.h>
 
-int main(int argc, char ** argv)
+int main(int argc, const char ** argv)
 {
-	if (getenv("TELLPID")) 
-	{	
-		printf("pid is %d\n",getpid());
-		getc(stdin);
-	}
 	return AP_UnixApp::main(ABIWORD_APP_NAME, argc, argv);
 }
 
 #endif
+
+#ifdef ABI_PLUGIN_BUILTIN
+#include "abi_plugin_builtin.h"
+#endif
