@@ -41,6 +41,8 @@
 #include "ie_impGraphic.h"
 #include "ie_exp.h"
 #include "ie_types.h"
+#include "ap_Convert.h"
+#include "ap_UnixApp.h"
 
 #include "ut_sleep.h"
 #include <unistd.h>
@@ -53,14 +55,16 @@
 #endif
 #include <readline/readline.h>
 #include <readline/history.h>
-
+#include <stdlib.h>
+#include <sys/stat.h>
 #include "AbiCommand.h"
 
 #include "xap_UnixNullGraphics.h"
 #include "xap_UnixPSGraphics.h"
 #include "ap_UnixFrame.h"
+#include "gr_DrawArgs.h"
 
-ABI_PLUGIN_DECLARE(AbiCommand);
+ABI_PLUGIN_DECLARE(AbiCommand)
 
 static bool AbiCommand_invoke(AV_View* v, EV_EditMethodCallData *d);
 
@@ -177,11 +181,15 @@ static bool  AbiCommand_invoke(AV_View* v, EV_EditMethodCallData *d)
 
 AbiCommand::AbiCommand(void) :
 	m_pCurDoc(NULL),
+	m_pCurFile(new UT_UTF8String),
 	m_pCurFrame(NULL),
 	m_pCurView(NULL),
 	m_pG(NULL),
 	m_pLayout(NULL),
-	m_bViewDoc(false)
+	m_bViewDoc(false),
+	m_bRunAsServer(false),
+	m_iPID(0),
+	m_sErrorFile("")
 {
 	m_pApp = XAP_App::getApp();
 }
@@ -189,6 +197,7 @@ AbiCommand::AbiCommand(void) :
 AbiCommand::~AbiCommand(void)
 {
 	deleteCurrentDoc();
+	DELETEP(m_pCurFile);
 }
 
 void AbiCommand::deleteCurrentDoc(void)
@@ -241,12 +250,12 @@ void AbiCommand::doCommands(void)
 // break it into tokens
 //
 		tokenizeString(toks,pCom);
-		FREEP(pCom);
 		if(toks.getItemCount() > 0)
 		{
-			UT_String * pTok = static_cast<UT_String *>(toks.getNthItem(0));
+			UT_String * pTok = const_cast<UT_String *>(static_cast<const UT_String *>(toks.getNthItem(0)));
 			if(pTok && UT_strcmp(pTok->c_str(),"quit") == 0)
 			{
+				FREEP(pCom);
 				bQuit = true;
 			}
 			else
@@ -258,9 +267,17 @@ void AbiCommand::doCommands(void)
 				}
 				else
 				{
+					if(m_bRunAsServer)
+					{
+						FILE * errF = fopen(m_sErrorFile.c_str(),"w");
+						fprintf(errF,"Error in command %s number %d \n",pCom,bres);
+						fflush(errF);
+						fclose(errF);
+					}
 					printf("error %d \n",bres);
 				}
 			}
+			FREEP(pCom);
 			clearTokenVector(toks);
 		}
 	}
@@ -400,7 +417,7 @@ void AbiCommand::clearTokenVector(UT_Vector & vecToks)
 	UT_uint32 i =0;
 	for(i=0; i< vecToks.getItemCount(); i++)
 	{
-		UT_String * pComm = static_cast<UT_String *>(vecToks.getNthItem(i));
+		UT_String * pComm = const_cast<UT_String *>(static_cast<const UT_String *>(vecToks.getNthItem(i)));
 		delete pComm;
 	}
 	vecToks.clear();
@@ -420,11 +437,11 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 	printf("Number of tokens here is %d \n",count);
 	for(i=0; i< count; i++)
 	{
-		UT_String * pstr = static_cast<UT_String *>(pToks->getNthItem(i));
+		UT_String * pstr = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(i)));
 		printf("Token %d string is %s \n",i,pstr->c_str());
 	}
 
-	UT_String * pCom0 = static_cast<UT_String *>(pToks->getNthItem(0));
+	UT_String * pCom0 = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(0)));
 
 	printf("first command is %s \n",pCom0->c_str());
 
@@ -443,6 +460,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 			return static_cast<UT_sint32>(error);
 		}
 		replaceDocument(pDoc);
+		m_pCurFile->assign("");
 		return 0;
 	}
 //
@@ -453,7 +471,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 		printf("Attempting to load a document \n");
 		if(count >= 2)
 		{
-			UT_String * pCom1 =  static_cast<UT_String *>(pToks->getNthItem(1));
+			UT_String * pCom1 =  const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 			PD_Document * pDoc = new PD_Document(m_pApp);
 			UT_Error error = pDoc->readFromFile(pCom1->c_str(),IEFT_Unknown);
 			if(error != UT_OK)
@@ -463,6 +481,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 				return static_cast<UT_sint32>(error);
 			}
 			replaceDocument(pDoc);
+			m_pCurFile->assign(pCom1->c_str());
 			return 0;
 //
 // We're finished a Load!
@@ -611,7 +630,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 		if(m_pCurView && (pToks->getItemCount()> 1))
 		{
 			bool bEOD = false;
-			UT_String * pFind = static_cast<UT_String *>(pToks->getNthItem(1));
+			UT_String * pFind = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 			const UT_UCSChar * pUCSFind = reinterpret_cast<UT_UCSChar *>(UT_calloc(pFind->size()+1,sizeof(UT_UCSChar)));
 	              static_cast<FV_View *>(m_pCurView)->findSetMatchCase(true);
 			static_cast<FV_View *>(m_pCurView)->findNext(pUCSFind,bEOD);
@@ -638,7 +657,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 			IEFileType ieft = 0;
 			if(pToks->getItemCount() > 1)
 			{
-				UT_String * pCom1 = static_cast<UT_String *>(pToks->getNthItem(1));
+				UT_String * pCom1 = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 				char* suffix = rindex(pCom1->c_str(), '.');
 				if(i != 0)
 				{
@@ -652,20 +671,128 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 				m_pCurDoc->saveAs(pCom1->c_str(),ieft);
 				return 0;
 			}
-			UT_String sFile = m_pCurDoc->getFileName();
 			ieft = static_cast<IEFileType>(m_pCurDoc->getLastOpenedType());
-			m_pCurDoc->saveAs(sFile.c_str(),ieft);
+			m_pCurDoc->saveAs(m_pCurFile->utf8_str(),ieft);
 			return 0;
 		}
 		return -1;
 	}
 //
+// Convert to Text
+//
+	else if(UT_strcmp(pCom0->c_str(),"converttotext")==0)
+	{
+		AP_Convert APConvert;
+		UT_String src;
+		UT_String dest;
+		IEFileType ieftTxt = IE_Exp::fileTypeForSuffix(".txt");
+		UT_uint32 i;
+		if(pToks->getItemCount() < 2)
+		{
+			return -1;
+		}
+		if(pToks->getItemCount() == 2)
+		{
+			src = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+			i = UT_String_findCh(src,'.');
+			if(i < (size_t) -1 )
+			{
+				dest = src.substr(0,i);
+				dest += ".txt";
+			}
+			else
+			{
+				dest = src;
+				dest += ".txt";
+			}
+			APConvert.convertTo(src.c_str(),IEFT_Unknown,dest.c_str(),ieftTxt);
+			struct stat myNameStat;
+			int iMyStat =stat(dest.c_str(),&myNameStat); 
+			if(iMyStat == 0)
+			{
+				return 0;
+			}
+			return -1;
+		}
+		src = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+		dest = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(2)));
+		APConvert.convertTo(src.c_str(),IEFT_Unknown,dest.c_str(),ieftTxt);
+		struct stat myNameStat;
+		int iMyStat =stat(dest.c_str(),&myNameStat); 
+		if(iMyStat == 0)
+		{
+			return 0;
+		}
+		return -1;
+	}
+//
+// Write PID to file
+//
+	else if(UT_strcmp(pCom0->c_str(),"writepid")==0)
+	{
+		if(pToks->getItemCount() < 2)
+		{
+			return -1;
+		}
+		UT_String pidFile;
+		pidFile =  *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+		FILE * pidF = fopen(pidFile.c_str(),"w");
+		fprintf(pidF,"%d",getpid());
+		fflush(pidF);
+		fclose(pidF);
+		return 0;
+	}
+//
+// server
+//	
+	else if(UT_strcmp(pCom0->c_str(),"server")==0)
+	{
+		m_bRunAsServer = true;
+		m_iPID = getpid();
+		m_sErrorFile = "/tmp/AbiCommandError";
+		UT_String spid("");
+		UT_String_sprintf(spid, "%d",m_iPID);
+		m_sErrorFile += spid;
+		m_sErrorFile +=".txt";
+			
+		return 0;
+	}
+//
+// PNG preview
+//
+	else if(UT_strcmp(pCom0->c_str(),"previewpng")==0)
+	{
+		UT_String src,destPNG,sWidth,sHeight;
+		if(pToks->getItemCount() < 5)
+		{
+			return -1;
+		}
+		src = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+		destPNG = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(2)));
+		sWidth = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(3)));
+		sHeight = *const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(4)));
+		UT_sint32 iWidth = atoi(sWidth.c_str());
+		UT_sint32 iHeight = atoi(sHeight.c_str());
+		if((iWidth <= 0) || (iHeight <= 0))
+		{
+			return -1;
+		}
+		AP_UnixApp * pUnixApp = static_cast<AP_UnixApp *>(m_pApp);
+		bool res = pUnixApp->makePngPreview(src.c_str(),destPNG.c_str(),iWidth,iHeight);
+		if(!res)
+		{
+			return -1;
+		}
+		return 0;
+	}
+//
 // Help
 //
 	else if(UT_strcmp(pCom0->c_str(),"help")==0)
 	{
 		printf("Currently implemented commands are...\n");
 		printf("help - prints this message \n");
+		printf("quit - exits the program \n");
 		printf("new - create a new empty document.\n");
 		printf("load <filename> - load <filename> replacing the current document. \n");
 		printf("printfile <filename1> <filename2>... -  print the current document into the filenames listed.\n");
@@ -682,6 +809,10 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 		printf("save <filename> - Save the current document.\n");
 		printf("  If filename is omitted the file is saved to it's original name \n");
 		printf("  Otherwise the extension of the filename is used to determine the format of the file\n");
+		printf("converttotext <src> <dest> - convert the file given in <src> to the plain text file named <dest> \n");
+		printf("writepid <file> - write the PID of this process to the file <file> \n");
+		printf("server - This is being run as remote process. Write an error file on error. (/tmp/AbiCommandErrorPID.txt) \n");
+		printf("previewpng Document preview.png width height - Create a PNG preview of Document with name preview.png of width pixels widde and hieght pixels in height. \n");
 		printf("visualedit - popup a visual window and edit the file or just preview what you've done \n");
 		printf("  close the window when finished.\n");
 
@@ -696,7 +827,7 @@ UT_sint32 AbiCommand::parseTokens(UT_Vector * pToks)
 		
 		for ( int i = 1; i < count; i++ )
 		  {
-		    UT_String * pComm = static_cast<UT_String *>(pToks->getNthItem(i));
+		    UT_String * pComm = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(i)));
 		    calldata += *pComm ;
 		  }
 		printf ( "EditMethod %s exists. Calling with %s\n", pCom0->c_str(), calldata.c_str() ) ;
@@ -773,7 +904,7 @@ bool AbiCommand::movePoint(UT_Vector * pToks)
     UT_sint32 amt = 0;
 	if(m_pCurView != NULL)
 	{
-		UT_String * pTarget = static_cast<UT_String *>(pToks->getNthItem(1));
+		UT_String * pTarget = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 		FV_DocPos docpos;
 		if(UT_stricmp(pTarget->c_str(),"BOD") == 0)
 		{
@@ -872,8 +1003,8 @@ bool AbiCommand::replaceAll(UT_Vector * pToks)
 {
 	if(m_pCurView != NULL)
 	{
-		UT_String * pFind = static_cast<UT_String *>(pToks->getNthItem(1));
-		UT_String * pReplace = static_cast<UT_String *>(pToks->getNthItem(2));
+		UT_String * pFind = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+		UT_String * pReplace = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(2)));
 		const UT_UCSChar * pUCSFind = reinterpret_cast<UT_UCSChar *>(UT_calloc(pFind->size()+1,sizeof(UT_UCSChar)));
 		const UT_UCSChar * pUCSReplace = reinterpret_cast<UT_UCSChar *>(UT_calloc(pReplace->size()+1,sizeof(UT_UCSChar)));
 		UT_UCS4_strcpy_char(const_cast<UT_UCS4Char *>(pUCSFind), pFind->c_str());
@@ -897,7 +1028,7 @@ bool AbiCommand::insertText(UT_Vector * pToks)
 {
 	if(m_pCurView != NULL && pToks->getItemCount() > 1)
 	{
-		UT_String * pText = static_cast<UT_String *>(pToks->getNthItem(1));
+		UT_String * pText = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 		UT_UCSChar * pUCSText = static_cast<UT_UCSChar *>(UT_calloc(pText->size()+1,sizeof(UT_UCSChar)));
 		UT_UCS4_strcpy_char(static_cast<UT_UCS4Char *>(pUCSText), pText->c_str());
 		static_cast<FV_View *>(m_pCurView)->cmdCharInsert(pUCSText,pText->size());
@@ -914,7 +1045,7 @@ bool AbiCommand::deleteText(UT_Vector * pToks)
 {
 	if(m_pCurView != NULL)
 	{
-		UT_String * pCom1 = static_cast<UT_String *>(pToks->getNthItem(1));
+		UT_String * pCom1 = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
 		UT_sint32 count = atoi(pCom1->c_str());
 		bool bForward = (count > 0);
 		static_cast<FV_View *>(m_pCurView)->cmdCharDelete(bForward,count);
@@ -931,8 +1062,8 @@ bool AbiCommand::replaceNext(UT_Vector * pToks)
 {
 	if(m_pCurView != NULL)
 	{
-		UT_String * pFind = static_cast<UT_String *>(pToks->getNthItem(1));
-		UT_String * pReplace = static_cast<UT_String *>(pToks->getNthItem(2));
+		UT_String * pFind = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(1)));
+		UT_String * pReplace = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(2)));
 		const UT_UCSChar * pUCSFind = reinterpret_cast<UT_UCSChar *>(UT_calloc(pFind->size()+1,sizeof(UT_UCSChar)));
 		const UT_UCSChar * pUCSReplace = reinterpret_cast<UT_UCSChar *>(UT_calloc(pReplace->size()+1,sizeof(UT_UCSChar)));
 		UT_UCS4_strcpy_char(const_cast<UT_UCS4Char *>(pUCSFind), pFind->c_str());
@@ -963,13 +1094,16 @@ bool AbiCommand::printFiles(UT_Vector * pToks)
 	UT_uint32 i=0;
 	for(i=1; i< pToks->getItemCount();i++)
 	{
-		UT_String * pFile = static_cast<UT_String *>(pToks->getNthItem(i));
-		PS_Graphics * pG = new PS_Graphics (pFile->c_str(),
-											m_pCurDoc->getFileName(), 
-											m_pApp->getApplicationName(), 
-											static_cast<XAP_UnixApp *>(m_pApp)->getFontManager(),
-											true, 
-											m_pApp);
+		UT_String * pFile = const_cast<UT_String *>(static_cast<const UT_String *>(pToks->getNthItem(i)));
+		PS_GraphicsAllocInfo ai(pFile->c_str(),
+						    m_pCurFile->utf8_str(), 
+						    m_pApp->getApplicationName(), 
+						    static_cast<XAP_UnixApp *>(m_pApp)->getFontManager(),
+						    true, 
+								m_pApp);
+
+		PS_Graphics * pG =  (PS_Graphics *)m_pApp->newGraphics(ai);
+
 		// create a new layout and view object for the doc
 		FL_DocLayout *pDocLayout = new FL_DocLayout(m_pCurDoc,pG);
 		FV_View *printView = new FV_View(m_pApp,0,pDocLayout);
@@ -1037,7 +1171,9 @@ bool AbiCommand::replaceDocument(PD_Document * pDoc)
 	m_pCurFrame = new AP_UnixFrame(pUnixApp);
 	UT_String extension(".bak~");
 	m_pCurFrame->setAutoSaveFileExt(extension);
-	m_pG = new UnixNull_Graphics(pUnixApp->getFontManager(), m_pApp);
+	XAP_UnixNullGraphicsAllocInfo ai(pUnixApp->getFontManager(), m_pApp);
+	m_pG = (UnixNull_Graphics*) m_pApp->newGraphics(ai);
+
 	
 	m_pLayout = new FL_DocLayout(m_pCurDoc, static_cast<GR_Graphics *>(m_pG));
 	m_pCurView = new FV_View(m_pApp, m_pCurFrame, m_pLayout);
